@@ -9,10 +9,8 @@
 #   not, an error message is printed, quoting ERROR_LOCATION, which should
 #   probably be the filename and line number of the erroneous call to
 #   check_source_count.py.
-from __future__ import absolute_import
-from __future__ import print_function
-import sys
 import re
+import sys
 
 search_string = sys.argv[1]
 expected_count = int(sys.argv[2])
@@ -32,34 +30,28 @@ for f in files:
         details[f] = num
 
 if count == expected_count:
-    print(
-        "TEST-PASS | check_source_count.py {0} | {1}".format(
-            search_string, expected_count
-        )
-    )
+    print(f"TEST-PASS | check_source_count.py {search_string} | {expected_count}")
 
 else:
     print(
-        "TEST-UNEXPECTED-FAIL | check_source_count.py {0} | ".format(search_string),
+        f"TEST-UNEXPECTED-FAIL | check_source_count.py {search_string} | ",
         end="",
     )
     if count < expected_count:
         print(
-            "There are fewer occurrences of /{0}/ than expected. "
+            f"There are fewer occurrences of /{search_string}/ than expected. "
             "This may mean that you have removed some, but forgotten to "
-            "account for it {1}.".format(search_string, error_location)
+            f"account for it {error_location}."
         )
     else:
         print(
-            "There are more occurrences of /{0}/ than expected. We're trying "
-            "to prevent an increase in the number of {1}'s, using {2} if "
+            f"There are more occurrences of /{search_string}/ than expected. We're trying "
+            f"to prevent an increase in the number of {search_string}'s, using {replacement} if "
             "possible. If it is unavoidable, you should update the expected "
-            "count {3}.".format(
-                search_string, search_string, replacement, error_location
-            )
+            f"count {error_location}."
         )
 
-    print("Expected: {0}; found: {1}".format(expected_count, count))
+    print(f"Expected: {expected_count}; found: {count}")
     for k in sorted(details):
-        print("Found {0} occurences in {1}".format(details[k], k))
+        print(f"Found {details[k]} occurences in {k}")
     sys.exit(-1)
