@@ -2,20 +2,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import
 import inspect
 import io
 import os
-from six import (
-    BytesIO,
-    StringIO,
-)
 import sys
 import unittest
-from unittest import TextTestRunner as _TestRunner, TestResult as _TestResult
+from unittest import TestResult as _TestResult
+from unittest import TextTestRunner as _TestRunner
 
 import pytest
-import six
 
 here = os.path.abspath(os.path.dirname(__file__))
 
@@ -85,13 +80,13 @@ class _MozTestResult(_TestResult):
     def addError(self, test, err):
         _TestResult.addError(self, test, err)
         self.printFail(test, err)
-        self.stream.writeln("ERROR: {0}".format(self.getDescription(test)))
+        self.stream.writeln(f"ERROR: {self.getDescription(test)}")
         self.stream.writeln(self.errors[-1][1])
 
     def addFailure(self, test, err):
         _TestResult.addFailure(self, test, err)
         self.printFail(test, err)
-        self.stream.writeln("FAIL: {0}".format(self.getDescription(test)))
+        self.stream.writeln(f"FAIL: {self.getDescription(test)}")
         self.stream.writeln(self.failures[-1][1])
 
     def printFail(self, test, err):
@@ -104,7 +99,7 @@ class _MozTestResult(_TestResult):
             tb = tb.tb_next
         if tb:
             _, ln, _ = inspect.getframeinfo(tb)[:3]
-            message = "line {0}: {1}".format(ln, message)
+            message = f"line {ln}: {message}"
         self.printStatus("TEST-UNEXPECTED-FAIL", test, message)
 
 
@@ -140,8 +135,8 @@ def _mocked_file(cls):
     return MockedFile
 
 
-MockedStringFile = _mocked_file(StringIO)
-MockedBytesFile = _mocked_file(BytesIO)
+MockedStringFile = _mocked_file(io.StringIO)
+MockedBytesFile = _mocked_file(io.BytesIO)
 
 
 def normcase(path):
@@ -156,7 +151,7 @@ def normcase(path):
     return path
 
 
-class _MockBaseOpen(object):
+class _MockBaseOpen:
     """Callable that acts like the open() function; see MockedOpen for more
     info.
     """
@@ -166,7 +161,15 @@ class _MockBaseOpen(object):
         self.files = files
 
     def __call__(
-        self, name, mode="r", buffering=None, encoding=None, newline=None, errors=None
+        self,
+        name,
+        mode="r",
+        buffering=-1,
+        encoding=None,
+        errors=None,
+        newline=None,
+        closefd=True,
+        opener=None,
     ):
         # open() can be called with an integer "name" (i.e. a file descriptor).
         # We don't generally do this in our codebase, but internal Python
@@ -177,8 +180,10 @@ class _MockBaseOpen(object):
                 mode=mode,
                 buffering=buffering,
                 encoding=encoding,
-                newline=newline,
                 errors=errors,
+                newline=newline,
+                closefd=closefd,
+                opener=opener,
             )
         # buffering is ignored.
         absname = normcase(os.path.abspath(name))
@@ -187,7 +192,7 @@ class _MockBaseOpen(object):
         elif absname in self.files:
             content = self.files[absname]
             if content is None:
-                raise IOError(2, "No such file or directory")
+                raise OSError(2, "No such file or directory")
             file = self._mocked_file(absname, mode, content)
         elif "a" in mode:
             read_mode = "rb" if "b" in mode else "r"
@@ -202,23 +207,21 @@ class _MockBaseOpen(object):
         raise NotImplementedError("subclass must implement")
 
 
-class _MockPy2Open(_MockBaseOpen):
-    def _mocked_file(self, name, mode, content=None):
-        content = six.ensure_binary(content or b"")
-        return MockedBytesFile(self, name, content)
-
-
 class _MockOpen(_MockBaseOpen):
     def _mocked_file(self, name, mode, content=None):
         if "b" in mode:
-            content = six.ensure_binary(content or b"")
+            content = content or b""
+            if isinstance(content, str):
+                content = content.encode()
             return MockedBytesFile(self, name, content)
         else:
-            content = six.ensure_text(content or u"")
+            content = content or ""
+            if isinstance(content, bytes):
+                content = content.decode()
             return MockedStringFile(self, name, content)
 
 
-class MockedOpen(object):
+class MockedOpen:
     """
     Context manager diverting the open builtin such that opening files
     can open "virtual" file instances given when creating a MockedOpen.
@@ -247,24 +250,23 @@ class MockedOpen(object):
             self.files[normcase(os.path.abspath(name))] = content
 
     def __enter__(self):
-        import six.moves.builtins
+        import builtins
 
-        self.open = six.moves.builtins.open
+        self.open = open
         self.io_open = io.open
         self._orig_path_exists = os.path.exists
         self._orig_path_isdir = os.path.isdir
         self._orig_path_isfile = os.path.isfile
-        builtin_cls = _MockPy2Open if six.PY2 else _MockOpen
-        six.moves.builtins.open = builtin_cls(self.open, self.files)
+        builtins.open = _MockOpen(self.open, self.files)
         io.open = _MockOpen(self.io_open, self.files)
         os.path.exists = self._wrapped_exists
         os.path.isdir = self._wrapped_isdir
         os.path.isfile = self._wrapped_isfile
 
     def __exit__(self, type, value, traceback):
-        import six.moves.builtins
+        import builtins
 
-        six.moves.builtins.open = self.open
+        builtins.open = self.open
         io.open = self.io_open
         os.path.exists = self._orig_path_exists
         os.path.isdir = self._orig_path_isdir
