@@ -813,11 +813,6 @@ enum class ESClass {
   BigInt,
   Function,  // Note: Only JSFunction objects.
 
-#ifdef ENABLE_RECORD_TUPLE
-  Record,
-  Tuple,
-#endif
-
   /** None of the above. */
   Other
 };
