@@ -50,9 +50,6 @@ using JS::NullValue;
 using JS::NumberValue;
 using JS::ObjectOrNullValue;
 using JS::ObjectValue;
-#ifdef ENABLE_RECORD_TUPLE
-using JS::ExtendedPrimitiveValue;
-#endif
 using JS::PrivateGCThingValue;
 using JS::PrivateUint32Value;
 using JS::PrivateValue;
@@ -96,12 +93,14 @@ using JS::NativeImpl;
 
 using JS::Rooted;
 using JS::RootedBigInt;
+using JS::RootedField;
 using JS::RootedFunction;
 using JS::RootedId;
 using JS::RootedObject;
 using JS::RootedScript;
 using JS::RootedString;
 using JS::RootedSymbol;
+using JS::RootedTuple;
 using JS::RootedValue;
 
 using JS::PersistentRooted;
@@ -152,11 +151,6 @@ using JS::Zone;
 
 using JS::BigInt;
 
-#ifdef ENABLE_RECORD_TUPLE
-using JS::RecordType;
-using JS::TupleType;
-#endif
-
 } /* namespace js */
 
 #endif /* NamespaceImports_h */
