@@ -10,14 +10,12 @@
 
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Assertions.h"
-#include "mozilla/Casting.h"
 #include "mozilla/intl/Locale.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/Span.h"
 #include "mozilla/TextUtils.h"
 
 #include <algorithm>
-#include <iterator>
 #include <string>
 #include <string.h>
 #include <utility>
@@ -28,18 +26,16 @@
 #include "builtin/intl/LanguageTag.h"
 #include "builtin/intl/StringAsciiChars.h"
 #include "builtin/String.h"
-#include "gc/Rooting.h"
 #include "js/Conversions.h"
 #include "js/friend/ErrorMessages.h"  // js::GetErrorMessage, JSMSG_*
+#include "js/Printer.h"
 #include "js/TypeDecls.h"
 #include "js/Wrapper.h"
-#include "util/StringBuffer.h"
+#include "vm/Compartment.h"
 #include "vm/GlobalObject.h"
 #include "vm/JSContext.h"
 #include "vm/PlainObject.h"  // js::PlainObject
-#include "vm/Printer.h"
 #include "vm/StringType.h"
-#include "vm/WellKnownAtom.h"  // js_*_str
 
 #include "vm/JSObject-inl.h"
 #include "vm/NativeObject-inl.h"
@@ -51,7 +47,9 @@ const JSClass LocaleObject::class_ = {
     "Intl.Locale",
     JSCLASS_HAS_RESERVED_SLOTS(LocaleObject::SLOT_COUNT) |
         JSCLASS_HAS_CACHED_PROTO(JSProto_Locale),
-    JS_NULL_CLASS_OPS, &LocaleObject::classSpec_};
+    JS_NULL_CLASS_OPS,
+    &LocaleObject::classSpec_,
+};
 
 const JSClass& LocaleObject::protoClass_ = PlainObject::class_;
 
@@ -78,7 +76,7 @@ struct IndexAndLength {
   size_t index;
   size_t length;
 
-  IndexAndLength(size_t index, size_t length) : index(index), length(length){};
+  IndexAndLength(size_t index, size_t length) : index(index), length(length) {};
 
   template <typename T>
   mozilla::Span<const T> spanOf(const T* ptr) const {
@@ -149,7 +147,7 @@ static LocaleObject* CreateLocaleObject(JSContext* cx, HandleObject prototype,
 }
 
 static inline bool IsValidUnicodeExtensionValue(JSContext* cx,
-                                                JSLinearString* linear,
+                                                const JSLinearString* linear,
                                                 bool* isValid) {
   if (linear->length() == 0) {
     *isValid = false;
@@ -171,7 +169,11 @@ static inline bool IsValidUnicodeExtensionValue(JSContext* cx,
   return true;
 }
 
-/** Iterate through (sep keyword) in a valid, lowercased Unicode extension. */
+/**
+ * Iterate through (sep keyword) in a valid Unicode extension.
+ *
+ * The Unicode extension value is not required to be in canonical case.
+ */
 template <typename CharT>
 class SepKeywordIterator {
   const CharT* iter_;
@@ -197,7 +199,8 @@ class SepKeywordIterator {
                "overall Unicode locale extension or non-leading subtags must "
                "be at least key-sized");
 
-    MOZ_ASSERT((iter_[0] == 'u' && iter_[1] == '-') || iter_[0] == '-');
+    MOZ_ASSERT(((iter_[0] == 'u' || iter_[0] == 'U') && iter_[1] == '-') ||
+               iter_[0] == '-');
 
     while (true) {
       // Skip past '-' so |std::char_traits::find| makes progress. Skipping
@@ -222,9 +225,8 @@ class SepKeywordIterator {
     }
 
     MOZ_ASSERT(iter_[0] == '-');
-    MOZ_ASSERT(mozilla::IsAsciiLowercaseAlpha(iter_[1]) ||
-               mozilla::IsAsciiDigit(iter_[1]));
-    MOZ_ASSERT(mozilla::IsAsciiLowercaseAlpha(iter_[2]));
+    MOZ_ASSERT(mozilla::IsAsciiAlphanumeric(iter_[1]));
+    MOZ_ASSERT(mozilla::IsAsciiAlpha(iter_[2]));
     MOZ_ASSERT_IF(iter_ + SepKeyLength < end_, iter_[SepKeyLength] == '-');
     return iter_;
   }
@@ -237,7 +239,7 @@ class SepKeywordIterator {
  * to |ToString(value)|. Otherwise set the result string to nullptr.
  */
 static bool GetStringOption(JSContext* cx, HandleObject options,
-                            HandlePropertyName name,
+                            Handle<PropertyName*> name,
                             MutableHandle<JSLinearString*> string) {
   // Step 1.
   RootedValue option(cx);
@@ -273,7 +275,7 @@ static bool GetStringOption(JSContext* cx, HandleObject options,
  * to |ToString(ToBoolean(value))|. Otherwise set the result string to nullptr.
  */
 static bool GetBooleanOption(JSContext* cx, HandleObject options,
-                             HandlePropertyName name,
+                             Handle<PropertyName*> name,
                              MutableHandle<JSLinearString*> string) {
   // Step 1.
   RootedValue option(cx);
@@ -302,7 +304,7 @@ static bool ApplyOptionsToTag(JSContext* cx, mozilla::intl::Locale& tag,
                               HandleObject options) {
   // Steps 1-2 (Already performed in caller).
 
-  RootedLinearString option(cx);
+  Rooted<JSLinearString*> option(cx);
 
   // Step 3.
   if (!GetStringOption(cx, options, cx->names().language, &option)) {
@@ -534,7 +536,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     return false;
   }
 
-  RootedLinearString tagLinearStr(cx, tagStr->ensureLinear(cx));
+  Rooted<JSLinearString*> tagLinearStr(cx, tagStr->ensureLinear(cx));
   if (!tagLinearStr) {
     return false;
   }
@@ -554,6 +556,12 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     return false;
   }
 
+  if (tag.Language().Length() > 4) {
+    MOZ_ASSERT(cx->global());
+    cx->runtime()->setUseCounter(cx->global(),
+                                 JSUseCounter::LEGACY_LANG_SUBTAG);
+  }
+
   if (auto result = tag.CanonicalizeBaseName(); result.isErr()) {
     if (result.unwrapErr() ==
         mozilla::intl::Locale::CanonicalizationError::DuplicateVariant) {
@@ -575,7 +583,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     JS::RootedVector<intl::UnicodeExtensionKeyword> keywords(cx);
 
     // Step 14.
-    RootedLinearString calendar(cx);
+    Rooted<JSLinearString*> calendar(cx);
     if (!GetStringOption(cx, options, cx->names().calendar, &calendar)) {
       return false;
     }
@@ -602,7 +610,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     }
 
     // Step 17.
-    RootedLinearString collation(cx);
+    Rooted<JSLinearString*> collation(cx);
     if (!GetStringOption(cx, options, cx->names().collation, &collation)) {
       return false;
     }
@@ -629,7 +637,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     }
 
     // Step 20 (without validation).
-    RootedLinearString hourCycle(cx);
+    Rooted<JSLinearString*> hourCycle(cx);
     if (!GetStringOption(cx, options, cx->names().hourCycle, &hourCycle)) {
       return false;
     }
@@ -654,7 +662,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     }
 
     // Step 22 (without validation).
-    RootedLinearString caseFirst(cx);
+    Rooted<JSLinearString*> caseFirst(cx);
     if (!GetStringOption(cx, options, cx->names().caseFirst, &caseFirst)) {
       return false;
     }
@@ -678,7 +686,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     }
 
     // Steps 24-25.
-    RootedLinearString numeric(cx);
+    Rooted<JSLinearString*> numeric(cx);
     if (!GetBooleanOption(cx, options, cx->names().numeric, &numeric)) {
       return false;
     }
@@ -691,7 +699,7 @@ static bool Locale(JSContext* cx, unsigned argc, Value* vp) {
     }
 
     // Step 27.
-    RootedLinearString numberingSystem(cx);
+    Rooted<JSLinearString*> numberingSystem(cx);
     if (!GetStringOption(cx, options, cx->names().numberingSystem,
                          &numberingSystem)) {
       return false;
@@ -801,12 +809,14 @@ static mozilla::Maybe<IndexAndLength> FindUnicodeExtensionType(
                                       size_t(endType - beginType)});
 }
 
-static inline auto FindUnicodeExtensionType(JSLinearString* unicodeExtension,
-                                            UnicodeKey key) {
+static inline auto FindUnicodeExtensionType(
+    const JSLinearString* unicodeExtension, UnicodeKey key) {
   JS::AutoCheckCannotGC nogc;
   return unicodeExtension->hasLatin1Chars()
-             ? FindUnicodeExtensionType(unicodeExtension->latin1Chars(nogc),
-                                        unicodeExtension->length(), key)
+             ? FindUnicodeExtensionType(
+                   reinterpret_cast<const char*>(
+                       unicodeExtension->latin1Chars(nogc)),
+                   unicodeExtension->length(), key)
              : FindUnicodeExtensionType(unicodeExtension->twoByteChars(nogc),
                                         unicodeExtension->length(), key);
 }
@@ -920,10 +930,12 @@ static BaseNamePartsResult BaseNameParts(const CharT* baseName, size_t length) {
   return {language, script, region};
 }
 
-static inline auto BaseNameParts(JSLinearString* baseName) {
+static inline auto BaseNameParts(const JSLinearString* baseName) {
   JS::AutoCheckCannotGC nogc;
   return baseName->hasLatin1Chars()
-             ? BaseNameParts(baseName->latin1Chars(nogc), baseName->length())
+             ? BaseNameParts(
+                   reinterpret_cast<const char*>(baseName->latin1Chars(nogc)),
+                   baseName->length())
              : BaseNameParts(baseName->twoByteChars(nogc), baseName->length());
 }
 
@@ -933,7 +945,7 @@ static bool Locale_maximize(JSContext* cx, const CallArgs& args) {
 
   // Step 3.
   auto* locale = &args.thisv().toObject().as<LocaleObject>();
-  RootedLinearString tagStr(cx, locale->languageTag()->ensureLinear(cx));
+  Rooted<JSLinearString*> tagStr(cx, locale->languageTag()->ensureLinear(cx));
   if (!tagStr) {
     return false;
   }
@@ -970,7 +982,7 @@ static bool Locale_minimize(JSContext* cx, const CallArgs& args) {
 
   // Step 3.
   auto* locale = &args.thisv().toObject().as<LocaleObject>();
-  RootedLinearString tagStr(cx, locale->languageTag()->ensureLinear(cx));
+  Rooted<JSLinearString*> tagStr(cx, locale->languageTag()->ensureLinear(cx));
   if (!tagStr) {
     return false;
   }
@@ -1270,8 +1282,10 @@ static bool Locale_toSource(JSContext* cx, unsigned argc, Value* vp) {
 static const JSFunctionSpec locale_methods[] = {
     JS_FN("maximize", Locale_maximize, 0, 0),
     JS_FN("minimize", Locale_minimize, 0, 0),
-    JS_FN(js_toString_str, Locale_toString, 0, 0),
-    JS_FN(js_toSource_str, Locale_toSource, 0, 0), JS_FS_END};
+    JS_FN("toString", Locale_toString, 0, 0),
+    JS_FN("toSource", Locale_toSource, 0, 0),
+    JS_FS_END,
+};
 
 static const JSPropertySpec locale_properties[] = {
     JS_PSG("baseName", Locale_baseName, 0),
@@ -1285,7 +1299,8 @@ static const JSPropertySpec locale_properties[] = {
     JS_PSG("script", Locale_script, 0),
     JS_PSG("region", Locale_region, 0),
     JS_STRING_SYM_PS(toStringTag, "Intl.Locale", JSPROP_READONLY),
-    JS_PS_END};
+    JS_PS_END,
+};
 
 const ClassSpec LocaleObject::classSpec_ = {
     GenericCreateConstructor<Locale, 1, gc::AllocKind::FUNCTION>,
@@ -1295,7 +1310,8 @@ const ClassSpec LocaleObject::classSpec_ = {
     locale_methods,
     locale_properties,
     nullptr,
-    ClassSpec::DontDefineConstructor};
+    ClassSpec::DontDefineConstructor,
+};
 
 bool js::intl_ValidateAndCanonicalizeLanguageTag(JSContext* cx, unsigned argc,
                                                  Value* vp) {
@@ -1326,7 +1342,7 @@ bool js::intl_ValidateAndCanonicalizeLanguageTag(JSContext* cx, unsigned argc,
     return false;
   }
 
-  RootedLinearString tagLinearStr(cx, tagStr->ensureLinear(cx));
+  Rooted<JSLinearString*> tagLinearStr(cx, tagStr->ensureLinear(cx));
   if (!tagLinearStr) {
     return false;
   }
@@ -1381,7 +1397,7 @@ bool js::intl_TryValidateAndCanonicalizeLanguageTag(JSContext* cx,
   CallArgs args = CallArgsFromVp(argc, vp);
   MOZ_ASSERT(args.length() == 1);
 
-  RootedLinearString linear(cx, args[0].toString()->ensureLinear(cx));
+  Rooted<JSLinearString*> linear(cx, args[0].toString()->ensureLinear(cx));
   if (!linear) {
     return false;
   }
@@ -1447,7 +1463,7 @@ bool js::intl_ValidateAndCanonicalizeUnicodeExtensionType(JSContext* cx,
   HandleValue keyArg = args[2];
   MOZ_ASSERT(keyArg.isString(), "key must be a string");
 
-  RootedLinearString unicodeType(cx, typeArg.toString()->ensureLinear(cx));
+  Rooted<JSLinearString*> unicodeType(cx, typeArg.toString()->ensureLinear(cx));
   if (!unicodeType) {
     return false;
   }
