@@ -8,6 +8,7 @@
 
 loadRelativeToScript('utility.js');
 loadRelativeToScript('annotations.js');
+loadRelativeToScript('callgraph.js');
 loadRelativeToScript('CFG.js');
 loadRelativeToScript('dumpCFG.js');
 
@@ -31,10 +32,6 @@ try {
             name: "gcFunctions",
             default: "gcFunctions.lst"
         },
-        {
-            name: "gcEdges",
-            default: "gcEdges.json"
-        },
         {
             name: "limitedFunctions",
             default: "limitedFunctions.lst"
@@ -64,7 +61,7 @@ try {
     ]);
 } catch (e) {
     printErr(e);
-    printErr("Usage: analyzeRoots.js [-f function_name] <gcFunctions.lst> <gcEdges.txt> <limitedFunctions.lst> <gcTypes.txt> <typeInfo.txt> [start end [tmpfile]]");
+    printErr("Usage: analyzeRoots.js [-f function_name] <gcFunctions.lst> <limitedFunctions.lst> <gcTypes.txt> <typeInfo.txt> [start end [tmpfile]]");
     quit(1);
 }
 var gcFunctions = {};
@@ -78,80 +75,89 @@ text = null;
 
 var typeInfo = loadTypeInfo(options.typeInfo);
 
-var gcEdges = JSON.parse(os.file.readFile(options.gcEdges));
-
 var match;
-var gcThings = {};
-var gcPointers = {};
+var gcThings = new Set();
+var gcPointers = new Set();
+var gcRefs = new Set(typeInfo.GCRefs);
 
 text = snarf(options.gcTypes).split("\n");
 for (var line of text) {
     if (match = /^GCThing: (.*)/.exec(line))
-        gcThings[match[1]] = true;
+        gcThings.add(match[1]);
     if (match = /^GCPointer: (.*)/.exec(line))
-        gcPointers[match[1]] = true;
+        gcPointers.add(match[1]);
 }
 text = null;
 
+function isGCRef(type)
+{
+    if (type.Kind == "CSU")
+        return gcRefs.has(type.Name);
+    return false;
+}
+
 function isGCType(type)
 {
     if (type.Kind == "CSU")
-        return type.Name in gcThings;
+        return gcThings.has(type.Name);
     else if (type.Kind == "Array")
         return isGCType(type.Type);
     return false;
 }
 
-function isUnrootedType(type)
+function isUnrootedPointerDeclType(decl)
 {
-    if (type.Kind == "Pointer")
+    // Treat non-temporary T& references as if they were the underlying type T.
+    // For now, restrict this to only the types specifically annotated with JS_HAZ_GC_REF
+    // to avoid lots of false positives with other types.
+    let type = isReferenceDecl(decl) && isGCRef(decl.Type.Type) ? decl.Type.Type : decl.Type;
+
+    while (type.Kind == "Array") {
+        type = type.Type;
+    }
+
+    if (type.Kind == "Pointer") {
         return isGCType(type.Type);
-    else if (type.Kind == "Array") {
-        if (!type.Type) {
-            printErr("Received Array Kind with no Type");
-            printErr(JSON.stringify(type));
-            printErr(getBacktrace({args: true, locals: true}));
-        }
-        return isUnrootedType(type.Type);
-    } else if (type.Kind == "CSU")
-        return type.Name in gcPointers;
-    else
+    } else if (type.Kind == "CSU") {
+        return gcPointers.has(type.Name);
+    } else {
         return false;
+    }
 }
 
-function edgeCanGC(edge)
+function edgeCanGC(functionName, body, edge, scopeAttrs, functionBodies)
 {
-    if (edge.Kind != "Call")
+    if (edge.Kind != "Call") {
         return false;
+    }
 
-    var callee = edge.Exp[0];
+    for (const { callee, attrs } of getCallees(body, edge, scopeAttrs, functionBodies)) {
+        if (attrs & (ATTR_GC_SUPPRESSED | ATTR_REPLACED)) {
+            continue;
+        }
 
-    while (callee.Kind == "Drf")
-        callee = callee.Exp[0];
-
-    if (callee.Kind == "Var") {
-        var variable = callee.Variable;
-
-        if (variable.Kind == "Func") {
-            var func = mangled(variable.Name[0]);
+        if (callee.kind == "direct") {
+            const func = mangled(callee.name);
             if ((func in gcFunctions) || ((func + internalMarker) in gcFunctions))
                 return `'${func}$${gcFunctions[func]}'`;
             return false;
+        } else if (callee.kind == "indirect") {
+            if (!indirectCallCannotGC(functionName, callee.variable)) {
+                return "'*" + callee.variable + "'";
+            }
+        } else if (callee.kind == "field") {
+            if (fieldCallCannotGC(callee.staticCSU, callee.field)) {
+                continue;
+            }
+            const fieldkey = callee.fieldKey;
+            if (fieldkey in gcFunctions) {
+                return `'${fieldkey}'`;
+            }
+        } else {
+            return "<unknown>";
         }
-
-        var varName = variable.Name[0];
-        return indirectCallCannotGC(functionName, varName) ? false : "'*" + varName + "'";
     }
 
-    assert(callee.Kind == "Fld");
-    const staticCSU = getFieldCallInstanceCSU(edge, callee.Field);
-
-    if (fieldCallCannotGC(staticCSU, callee.Field.Name[0]))
-        return false;
-
-    const fieldkey = fieldKey(staticCSU, callee.Field);
-    if (fieldkey in gcFunctions)
-        return `'${fieldkey}'`;
     return false;
 }
 
@@ -223,10 +229,8 @@ function edgeCanGC(edge)
 //
 //  - 'gcInfo': a direct pointer to the GC call edge
 //
-function findGCBeforeValueUse(start_body, start_point, suppressed_bits, variable)
+function findGCBeforeValueUse(start_body, start_point, funcAttrs, variable)
 {
-    const isGCSuppressed = Boolean(suppressed_bits & ATTR_GC_SUPPRESSED);
-
     // Scan through all edges preceding an unrooted variable use, using an
     // explicit worklist, looking for a GC call and a preceding point where the
     // variable is known to be live. A worklist contains an incoming edge
@@ -394,12 +398,14 @@ function findGCBeforeValueUse(start_body, start_point, suppressed_bits, variable
                 return null;
             }
 
-            // The value is live across this edge. Check whether this edge can GC (if we don't have a GC yet on this path.)
+            // The value is live across this edge. Check whether this edge can
+            // GC (if we don't have a GC yet on this path.)
             const had_gcInfo = Boolean(path.gcInfo);
-            if (!path.gcInfo && !(body.attrs[ppoint] & ATTR_GC_SUPPRESSED) && !isGCSuppressed) {
-                var gcName = edgeCanGC(edge, body);
+            const edgeAttrs = body.attrs[ppoint] | funcAttrs;
+            if (!path.gcInfo && !(edgeAttrs & (ATTR_GC_SUPPRESSED | ATTR_REPLACED))) {
+                var gcName = edgeCanGC(functionName, body, edge, edgeAttrs, functionBodies);
                 if (gcName) {
-                    path.gcInfo = {name:gcName, body, ppoint};
+                    path.gcInfo = {name:gcName, body, ppoint, edge: edge.Index};
                 }
             }
 
@@ -497,10 +503,15 @@ function findGCBeforeValueUse(start_body, start_point, suppressed_bits, variable
         };
     };
 
-    return BFS_upwards(start_body, start_point, functionBodies, visitor, new Path()) || bestPathWithAnyUse;
+    const result = BFS_upwards(start_body, start_point, functionBodies, visitor, new Path());
+    if (result && result.gcInfo && result.anyUse) {
+        return result;
+    } else {
+        return bestPathWithAnyUse;
+    }
 }
 
-function variableLiveAcrossGC(suppressed, variable)
+function variableLiveAcrossGC(funcAttrs, variable, liveToEnd=false)
 {
     // A variable is live across a GC if (1) it is used by an edge (as in, it
     // was at least initialized), and (2) it is used after a GC in a successor
@@ -541,9 +552,9 @@ function variableLiveAcrossGC(suppressed, variable)
             if (edgeEndsValueLiveRange(edge, variable, body))
                 continue;
 
-            var usePoint = edgeUsesVariable(edge, variable, body);
+            var usePoint = edgeUsesVariable(edge, variable, body, liveToEnd);
             if (usePoint) {
-                var call = findGCBeforeValueUse(body, usePoint, suppressed, variable);
+                var call = findGCBeforeValueUse(body, usePoint, funcAttrs, variable);
                 if (!call)
                     continue;
 
@@ -561,15 +572,19 @@ function variableLiveAcrossGC(suppressed, variable)
 // live across a GC. If it is passed into a function that can GC, then it's
 // sort of like a Handle to an unrooted location, and the callee could GC
 // before overwriting it or rooting it.
-function unsafeVariableAddressTaken(suppressed, variable)
+function unsafeVariableAddressTaken(funcAttrs, variable)
 {
     for (var body of functionBodies) {
         if (!("PEdge" in body))
             continue;
         for (var edge of body.PEdge) {
             if (edgeTakesVariableAddress(edge, variable, body)) {
-                if (edge.Kind == "Assign" || (!(suppressed & ATTR_GC_SUPPRESSED) && edgeCanGC(edge)))
+                if (funcAttrs & (ATTR_GC_SUPPRESSED | ATTR_REPLACED)) {
+                    continue;
+                }
+                if (edge.Kind == "Assign" || edgeCanGC(functionName, body, edge, funcAttrs, functionBodies)) {
                     return {body:body, ppoint:edge.Index[0]};
+                }
             }
         }
     }
@@ -615,7 +630,7 @@ function loadPrintedLines(functionName)
 
 function findLocation(body, ppoint, opts={brief: false})
 {
-    var location = body.PPoint[ppoint - 1].Location;
+    var location = body.PPoint[ppoint ? ppoint - 1 : 0].Location;
     var file = location.CacheString;
 
     if (file.indexOf(sourceRoot) == 0)
@@ -637,8 +652,10 @@ function locationLine(text)
     return 0;
 }
 
-function printEntryTrace(functionName, entry)
+function getEntryTrace(functionName, entry)
 {
+    const trace = [];
+
     var gcPoint = entry.gcInfo ? entry.gcInfo.ppoint : 0;
 
     if (!functionBodies[0].lines)
@@ -685,32 +702,25 @@ function printEntryTrace(functionName, entry)
                 edgeText += " [[end of function]]";
         }
 
-        print("    " + lineText + (edgeText.length ? ": " + edgeText : ""));
+        // TODO: Store this in a more structured form for better markup, and perhaps
+        // linking to line numbers.
+        trace.push({lineText, edgeText});
         entry = entry.successor;
     }
+
+    return trace;
 }
 
-function isRootedType(type)
+function isRootedDeclType(decl)
 {
+    // Treat non-temporary T& references as if they were the underlying type T.
+    const type = isReferenceDecl(decl) ? decl.Type.Type : decl.Type;
     return type.Kind == "CSU" && ((type.Name in typeInfo.RootedPointers) ||
                                   (type.Name in typeInfo.RootedGCThings));
 }
 
-function typeDesc(type)
-{
-    if (type.Kind == "CSU") {
-        return type.Name;
-    } else if ('Type' in type) {
-        var inner = typeDesc(type.Type);
-        if (type.Kind == 'Pointer')
-            return inner + '*';
-        else if (type.Kind == 'Array')
-            return inner + '[]';
-        else
-            return inner + '?';
-    } else {
-        return '???';
-    }
+function printRecord(record) {
+    print(JSON.stringify(record));
 }
 
 function processBodies(functionName, wholeBodyAttrs)
@@ -718,9 +728,9 @@ function processBodies(functionName, wholeBodyAttrs)
     if (!("DefineVariable" in functionBodies[0]))
       return;
     const funcInfo = limitedFunctions[mangled(functionName)] || { attributes: 0 };
-    const suppressed = funcInfo.attributes | wholeBodyAttrs;
+    const funcAttrs = funcInfo.attributes | wholeBodyAttrs;
 
-    // Look for the JS_EXPECT_HAZARDS annotation, and output a different
+    // Look for the JS_EXPECT_HAZARDS annotation, so as to output a different
     // message in that case that won't be counted as a hazard.
     var annotations = new Set();
     for (const variable of functionBodies[0].DefineVariable) {
@@ -732,7 +742,7 @@ function processBodies(functionName, wholeBodyAttrs)
         }
     }
 
-    var missingExpectedHazard = annotations.has("Expect Hazards");
+    let missingExpectedHazard = annotations.has("Expect Hazards");
 
     // Awful special case, hopefully temporary:
     //
@@ -775,20 +785,33 @@ function processBodies(functionName, wholeBodyAttrs)
         }
     }
 
-    for (const variable of functionBodies[0].DefineVariable) {
+    const [mangledSymbol, readable] = splitFunction(functionName);
+
+    for (let decl of functionBodies[0].DefineVariable) {
         var name;
-        if (variable.Variable.Kind == "This")
+        if (decl.Variable.Kind == "This")
             name = "this";
-        else if (variable.Variable.Kind == "Return")
+        else if (decl.Variable.Kind == "Return")
             name = "<returnvalue>";
         else
-            name = variable.Variable.Name[0];
+            name = decl.Variable.Name[0];
 
         if (ignoreVars.has(name))
             continue;
 
-        if (isRootedType(variable.Type)) {
-            if (!variableLiveAcrossGC(suppressed, variable.Variable)) {
+        let liveToEnd = false;
+        if (decl.Variable.Kind == "Arg" && isReferenceDecl(decl) && decl.Type.Reference == 2) {
+            // References won't run destructors, so they would normally not be
+            // considered live at the end of the function. In order to handle
+            // the pattern of moving a GC-unsafe value into a function (eg an
+            // AutoCheckCannotGC&&), assume all argument rvalue references live to the
+            // end of the function unless their liveness is terminated by
+            // calling reset() or moving them into another function call.
+            liveToEnd = true;
+        }
+
+        if (isRootedDeclType(decl)) {
+            if (!variableLiveAcrossGC(funcAttrs, decl.Variable)) {
                 // The earliest use of the variable should be its constructor.
                 var lineText;
                 for (var body of functionBodies) {
@@ -798,37 +821,58 @@ function processBodies(functionName, wholeBodyAttrs)
                             lineText = text;
                     }
                 }
-                print("\nFunction '" + functionName + "'" +
-                      " has unnecessary root '" + name + "' at " + lineText);
+                const record = {
+                    record: "unnecessary",
+                    functionName,
+                    mangled: mangledSymbol,
+                    readable,
+                    variable: name,
+                    type: str_Type(decl.Type),
+                    loc: lineText || "???",
+                }
+                print(",");
+                printRecord(record);
             }
-        } else if (isUnrootedType(variable.Type)) {
-            var result = variableLiveAcrossGC(suppressed, variable.Variable);
+        } else if (isUnrootedPointerDeclType(decl)) {
+            var result = variableLiveAcrossGC(funcAttrs, decl.Variable, liveToEnd);
             if (result) {
                 assert(result.gcInfo);
-                var lineText = findLocation(result.gcInfo.body, result.gcInfo.ppoint);
-                if (annotations.has('Expect Hazards')) {
-                    print("\nThis is expected, but '" + functionName + "'" +
-                          " has unrooted '" + name + "'" +
-                          " of type '" + typeDesc(variable.Type) + "'" +
-                          " live across GC call " + result.gcInfo.name +
-                          " at " + lineText);
-                    missingExpectedHazard = false;
-                } else {
-                    print("\nFunction '" + functionName + "'" +
-                          " has unrooted '" + name + "'" +
-                          " of type '" + typeDesc(variable.Type) + "'" +
-                          " live across GC call " + result.gcInfo.name +
-                          " at " + lineText);
-                }
-                printEntryTrace(functionName, result);
+                const edge = result.gcInfo.edge;
+                const body = result.gcInfo.body;
+                const lineText = findLocation(body, result.gcInfo.ppoint);
+                const makeLoc = l => [l.Location.CacheString, l.Location.Line];
+                const range = [makeLoc(body.PPoint[edge[0] - 1]), makeLoc(body.PPoint[edge[1] - 1])];
+                const record = {
+                    record: "unrooted",
+                    expected: annotations.has("Expect Hazards"),
+                    functionName,
+                    mangled: mangledSymbol,
+                    readable,
+                    variable: name,
+                    type: str_Type(decl.Type),
+                    gccall: result.gcInfo.name.replaceAll("'", ""),
+                    gcrange: range,
+                    loc: lineText,
+                    trace: getEntryTrace(functionName, result),
+                };
+                missingExpectedHazard = false;
+                print(",");
+                printRecord(record);
             }
-            result = unsafeVariableAddressTaken(suppressed, variable.Variable);
+            result = unsafeVariableAddressTaken(funcAttrs, decl.Variable);
             if (result) {
                 var lineText = findLocation(result.body, result.ppoint);
-                print("\nFunction '" + functionName + "'" +
-                      " takes unsafe address of unrooted '" + name + "'" +
-                      " at " + lineText);
-                printEntryTrace(functionName, {body:result.body, ppoint:result.ppoint});
+                const record = {
+                    record: "address",
+                    functionName,
+                    mangled: mangledSymbol,
+                    readable,
+                    variable: name,
+                    loc: lineText,
+                    trace: getEntryTrace(functionName, {body:result.body, ppoint:result.ppoint}),
+                };
+                print(",");
+                printRecord(record);
             }
         }
     }
@@ -844,12 +888,21 @@ function processBodies(functionName, wholeBodyAttrs)
         const loc = (startfile == endfile) ? `${startfile}:${startline}-${endline}`
               : `${startfile}:${startline}`;
 
-        print("\nFunction '" + functionName + "' expected hazard(s) but none were found at " + loc);
+        const record = {
+            record: "missing",
+            functionName,
+            mangled: mangledSymbol,
+            readable,
+            loc,
+        }
+        print(",");
+        printRecord(record);
     }
 }
 
-if (options.batch == 1)
-    print("Time: " + new Date);
+print("[\n");
+var now = new Date();
+printRecord({record: "time", iso: "" + now, t: now.getTime()});
 
 var xdb = xdbLibrary();
 xdb.open("src_body.xdb");
@@ -877,21 +930,9 @@ function process(name, json) {
             if (attrs)
                 pbody.attrs[id] = attrs;
         }
-        for (const edgeAttr of gcEdges[blockIdentifier(body)] || []) {
-            body.attrs[edgeAttr.Index[0]] |= edgeAttr.attrs;
-        }
     }
 
-    // Special case: std::swap of two refcounted values thinks it can drop the
-    // ref count to zero. Or rather, it just calls operator=() in a context
-    // where the refcount will never drop to zero. Limit all calls within the
-    // body with LIMIT_CANNOT_GC.
-    let wholeBodyAttrs = 0;
-    if (functionName.includes("std::swap") || functionName.includes("mozilla::Swap")) {
-        wholeBodyAttrs = ATTR_GC_SUPPRESSED;
-    }
-
-    processBodies(functionName, wholeBodyAttrs);
+    processBodies(functionName);
 }
 
 if (options.function) {
@@ -900,6 +941,7 @@ if (options.function) {
     debugger;
     process(options.function, json);
     xdb.free_string(data);
+    print("\n]\n");
     quit(0);
 }
 
@@ -917,3 +959,5 @@ for (var nameIndex = start; nameIndex <= end; nameIndex++) {
     }
     xdb.free_string(data);
 }
+
+print("\n]\n");
