@@ -15,20 +15,22 @@
 #include "frontend/AbstractScopePtr.h"
 #include "frontend/NameAnalysisTypes.h"
 #include "frontend/NameCollections.h"
-#include "frontend/ParseContext.h"
-#include "frontend/ParserAtom.h"  // TaggedParserAtomIndex
-#include "frontend/SharedContext.h"
-#include "js/TypeDecls.h"
-#include "vm/BytecodeUtil.h"   // JSOp
+#include "frontend/Stencil.h"
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+#  include "frontend/UsingEmitter.h"
+#endif
+#include "vm/Opcodes.h"        // JSOp
 #include "vm/SharedStencil.h"  // GCThingIndex
 
 namespace js {
-
-class Scope;
-
 namespace frontend {
 
 struct BytecodeEmitter;
+class EvalSharedContext;
+class FunctionBox;
+class GlobalSharedContext;
+class ModuleSharedContext;
+class TaggedParserAtomIndex;
 
 // A scope that introduces bindings.
 class EmitterScope : public Nestable<EmitterScope> {
@@ -46,6 +48,13 @@ class EmitterScope : public Nestable<EmitterScope> {
   // chain, false if all bindings are stored in frame slots on the stack.
   bool hasEnvironment_;
 
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+  mozilla::Maybe<UsingEmitter> usingEmitter_;
+
+ private:
+  BlockKind blockKind_ = BlockKind::Other;
+#endif
+
   // The number of enclosing environments. Used for error checking.
   uint8_t environmentChainLength_;
 
@@ -106,13 +115,26 @@ class EmitterScope : public Nestable<EmitterScope> {
     return clearFrameSlotRange(bce, JSOp::Uninitialized, slotStart, slotEnd);
   }
 
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+  void setHasDisposables(BytecodeEmitter* bce) {
+    if (!usingEmitter_.isSome()) {
+      usingEmitter_.emplace(bce);
+    }
+  }
+#endif
+
  public:
   explicit EmitterScope(BytecodeEmitter* bce);
 
   void dump(BytecodeEmitter* bce);
 
   [[nodiscard]] bool enterLexical(BytecodeEmitter* bce, ScopeKind kind,
-                                  LexicalScope::ParserData* bindings);
+                                  LexicalScope::ParserData* bindings
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+                                  ,
+                                  BlockKind blockKind = BlockKind::Other
+#endif
+  );
   [[nodiscard]] bool enterClassBody(BytecodeEmitter* bce, ScopeKind kind,
                                     ClassBodyScope::ParserData* bindings);
   [[nodiscard]] bool enterNamedLambda(BytecodeEmitter* bce,
@@ -143,6 +165,29 @@ class EmitterScope : public Nestable<EmitterScope> {
 
   bool hasEnvironment() const { return hasEnvironment_; }
 
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+ private:
+  // Disposable Scope here refers to any scope
+  // with using bindings in it for now that is
+  // a lexical scope and a module scope.
+  [[nodiscard]] bool prepareForDisposableScopeBody(BytecodeEmitter* bce);
+
+  [[nodiscard]] bool emitDisposableScopeBodyEnd(BytecodeEmitter* bce);
+
+ public:
+  [[nodiscard]] bool prepareForModuleDisposableScopeBody(BytecodeEmitter* bce);
+
+  [[nodiscard]] bool emitModuleDisposableScopeBodyEnd(BytecodeEmitter* bce);
+
+  [[nodiscard]] bool prepareForDisposableAssignment(UsingHint hint);
+
+  bool hasDisposables() const { return usingEmitter_.isSome(); }
+
+  bool hasAsyncDisposables() const {
+    return hasDisposables() && usingEmitter_->hasAwaitUsing();
+  }
+#endif
+
   // The first frame slot used.
   uint32_t frameSlotStart() const {
     if (EmitterScope* inFrame = enclosingInFrame()) {
@@ -192,10 +237,7 @@ class EmitterScope : public Nestable<EmitterScope> {
   // shadowing. If `name` refers to a name that is actually stamped onto the
   // target object (anything other than a non-static private method), then
   // `brandLoc` is set to Nothing.
-  //
-  // To handle cases where it's not possible to find the private brand, this
-  // method has to be fallible.
-  bool lookupPrivate(BytecodeEmitter* bce, TaggedParserAtomIndex name,
+  void lookupPrivate(BytecodeEmitter* bce, TaggedParserAtomIndex name,
                      NameLocation& loc, mozilla::Maybe<NameLocation>& brandLoc);
 
   mozilla::Maybe<NameLocation> locationBoundInScope(TaggedParserAtomIndex name,
