@@ -290,7 +290,7 @@
  *
  * Barriers for use outside of the JS engine call into the same barrier
  * implementations at InternalBarrierMethods<T>::post via an indirect call to
- * Heap(.+)PostWriteBarrier.
+ * Heap(.+)WriteBarriers.
  *
  * These clases are designed to be used to wrap GC thing pointers or values that
  * act like them (i.e. JS::Value and jsid).  It is possible to use them for
@@ -364,7 +364,7 @@ struct InternalBarrierMethods<T*> {
 namespace gc {
 MOZ_ALWAYS_INLINE void ValuePostWriteBarrier(Value* vp, const Value& prev,
                                              const Value& next) {
-  MOZ_ASSERT(!CurrentThreadIsIonCompiling());
+  MOZ_ASSERT(!CurrentThreadIsOffThreadCompiling());
   MOZ_ASSERT(vp);
 
   // If the target needs an entry, add it.
@@ -617,11 +617,13 @@ class GCPtr : public WriteBarriered<T> {
 
 #ifdef DEBUG
   ~GCPtr() {
-    // No barriers are necessary as this only happens when the GC is sweeping.
+    // No barriers are necessary as this only happens when the GC is sweeping or
+    // before this has been initialized (see above comment).
     //
     // If this assertion fails you may need to make the containing object use a
     // HeapPtr instead, as this can be deleted from outside of GC.
-    MOZ_ASSERT(CurrentThreadIsGCSweeping() || CurrentThreadIsGCFinalizing());
+    MOZ_ASSERT(CurrentThreadIsGCSweeping() || CurrentThreadIsGCFinalizing() ||
+               this->value == JS::SafelyInitialized<T>::create());
 
     Poison(this, JS_FREED_HEAP_PTR_PATTERN, sizeof(*this),
            MemCheckKind::MakeNoAccess);
@@ -769,7 +771,7 @@ template <class T>
 class GCStructPtr : public BarrieredBase<T> {
  public:
   // This is sometimes used to hold tagged pointers.
-  static constexpr uintptr_t MaxTaggedPointer = 0x2;
+  static constexpr uintptr_t MaxTaggedPointer = 0x5;
 
   GCStructPtr() : BarrieredBase<T>(JS::SafelyInitialized<T>::create()) {}
 
