@@ -94,7 +94,7 @@ void PreventGCDuringInteractiveDebug() { TlsContext.get()->suppressGC++; }
 #endif
 
 void js::ReleaseAllJITCode(JS::GCContext* gcx) {
-  js::CancelOffThreadIonCompile(gcx->runtime());
+  js::CancelOffThreadCompile(gcx->runtime());
 
   for (ZonesIter zone(gcx->runtime(), SkipAtoms); !zone.done(); zone.next()) {
     zone->forceDiscardJitCode(gcx);
@@ -102,6 +102,8 @@ void js::ReleaseAllJITCode(JS::GCContext* gcx) {
       jitZone->discardStubs();
     }
   }
+
+  gcx->runtime()->clearSelfHostedJitCache();
 }
 
 AutoSuppressGC::AutoSuppressGC(JSContext* cx)
@@ -302,7 +304,7 @@ JS_PUBLIC_API void JS::NonIncrementalGC(JSContext* cx, JS::GCOptions options,
 
 JS_PUBLIC_API void JS::StartIncrementalGC(JSContext* cx, JS::GCOptions options,
                                           GCReason reason,
-                                          const js::SliceBudget& budget) {
+                                          const JS::SliceBudget& budget) {
   AssertHeapIsIdle();
   CHECK_THREAD(cx);
   CheckGCOptions(options);
@@ -311,7 +313,7 @@ JS_PUBLIC_API void JS::StartIncrementalGC(JSContext* cx, JS::GCOptions options,
 }
 
 JS_PUBLIC_API void JS::IncrementalGCSlice(JSContext* cx, GCReason reason,
-                                          const js::SliceBudget& budget) {
+                                          const JS::SliceBudget& budget) {
   AssertHeapIsIdle();
   CHECK_THREAD(cx);
 
@@ -431,13 +433,8 @@ JS_PUBLIC_API void JS::SetLowMemoryState(JSContext* cx, bool newState) {
   return cx->runtime()->gc.setLowMemoryState(newState);
 }
 
-JS_PUBLIC_API void JS::DisableIncrementalGC(JSContext* cx) {
-  cx->runtime()->gc.disallowIncrementalGC();
-}
-
 JS_PUBLIC_API bool JS::IsIncrementalGCEnabled(JSContext* cx) {
-  GCRuntime& gc = cx->runtime()->gc;
-  return gc.isIncrementalGCEnabled() && gc.isIncrementalGCAllowed();
+  return cx->runtime()->gc.isIncrementalGCEnabled();
 }
 
 JS_PUBLIC_API bool JS::IsIncrementalGCInProgress(JSContext* cx) {
