@@ -6,15 +6,15 @@
 
 #include "gc/GCParallelTask.h"
 
-#include "mozilla/MathAlgorithms.h"
 #include "mozilla/Maybe.h"
+#include "mozilla/TimeStamp.h"
 
 #include "gc/GCContext.h"
 #include "gc/GCInternals.h"
 #include "gc/ParallelWork.h"
 #include "vm/HelperThreadState.h"
 #include "vm/Runtime.h"
-#include "vm/TraceLogging.h"
+#include "vm/Time.h"
 
 using namespace js;
 using namespace js::gc;
@@ -24,6 +24,10 @@ using mozilla::TimeDuration;
 using mozilla::TimeStamp;
 
 js::GCParallelTask::~GCParallelTask() {
+  // The LinkedListElement destructor will remove us from any list we are part
+  // of without synchronization, so ensure that doesn't happen.
+  MOZ_DIAGNOSTIC_ASSERT(!isInList());
+
   // Only most-derived classes' destructors may do the join: base class
   // destructors run after those for derived classes' members, so a join in a
   // base class can't ensure that the task is done using the members. All we
@@ -31,13 +35,25 @@ js::GCParallelTask::~GCParallelTask() {
   assertIdle();
 }
 
+static bool ShouldMeasureTaskStartDelay() {
+  // We use many tasks during GC so randomly sample a small fraction for the
+  // purposes of recording telemetry.
+  return (rand() % 100) == 0;
+}
+
 void js::GCParallelTask::startWithLockHeld(AutoLockHelperThreadState& lock) {
   MOZ_ASSERT(CanUseExtraThreads());
   MOZ_ASSERT(HelperThreadState().isInitialized(lock));
   assertIdle();
 
-  setDispatched(lock);
-  HelperThreadState().submitTask(this, lock);
+  maybeQueueTime_ = TimeStamp();
+  if (ShouldMeasureTaskStartDelay()) {
+    maybeQueueTime_ = TimeStamp::Now();
+  }
+
+  dispatchedToThreadPool = false;
+
+  gc->dispatchOrQueueParallelTask(this, lock);
 }
 
 void js::GCParallelTask::start() {
@@ -60,8 +76,7 @@ void js::GCParallelTask::startOrRunIfIdle(AutoLockHelperThreadState& lock) {
   joinWithLockHeld(lock);
 
   if (!CanUseExtraThreads()) {
-    AutoUnlockHelperThreadState unlock(lock);
-    runFromMainThread();
+    runFromMainThread(lock);
     return;
   }
 
@@ -87,22 +102,35 @@ void js::GCParallelTask::joinWithLockHeld(AutoLockHelperThreadState& lock,
     return;
   }
 
-  if (isDispatched(lock) && deadline.isNothing()) {
+  if (lock.hasQueuedTasks()) {
+    // Unlock to allow task dispatch without lock held, otherwise we could wait
+    // forever.
+    AutoUnlockHelperThreadState unlock(lock);
+  }
+
+  if (isNotYetRunning(lock) && !dispatchedToThreadPool &&
+      deadline.isNothing()) {
     // If the task was dispatched but has not yet started then cancel the task
     // and run it from the main thread. This stops us from blocking here when
     // the helper threads are busy with other tasks.
-    cancelDispatchedTask(lock);
-    AutoUnlockHelperThreadState unlock(lock);
-    runFromMainThread();
+    MOZ_ASSERT(isInList());
+    MOZ_ASSERT_IF(isDispatched(lock), gc->dispatchedParallelTasks != 0);
+
+    remove();
+    runFromMainThread(lock);
   } else {
     // Otherwise wait for the task to complete.
     joinNonIdleTask(deadline, lock);
   }
 
   if (isIdle(lock)) {
-    if (phaseKind != gcstats::PhaseKind::NONE) {
-      gc->stats().recordParallelPhase(phaseKind, duration());
-    }
+    recordDuration();
+  }
+}
+
+void GCParallelTask::recordDuration() {
+  if (phaseKind != gcstats::PhaseKind::NONE) {
+    gc->stats().recordParallelPhase(phaseKind, duration_);
   }
 }
 
@@ -113,7 +141,7 @@ void js::GCParallelTask::joinNonIdleTask(Maybe<TimeStamp> deadline,
   while (!isFinished(lock)) {
     TimeDuration timeout = TimeDuration::Forever();
     if (deadline) {
-      TimeStamp now = ReallyNow();
+      TimeStamp now = TimeStamp::Now();
       if (*deadline <= now) {
         break;
       }
@@ -128,28 +156,23 @@ void js::GCParallelTask::joinNonIdleTask(Maybe<TimeStamp> deadline,
   }
 }
 
-void js::GCParallelTask::cancelDispatchedTask(AutoLockHelperThreadState& lock) {
-  MOZ_ASSERT(isDispatched(lock));
-  MOZ_ASSERT(isInList());
-  remove();
-  setIdle(lock);
-}
-
-static inline TimeDuration TimeSince(TimeStamp prev) {
-  TimeStamp now = ReallyNow();
-  // Sadly this happens sometimes.
-  MOZ_ASSERT(now >= prev);
-  if (now < prev) {
-    now = prev;
-  }
-  return now - prev;
-}
-
 void js::GCParallelTask::runFromMainThread() {
-  assertIdle();
-  MOZ_ASSERT(js::CurrentThreadCanAccessRuntime(gc->rt));
   AutoLockHelperThreadState lock;
-  runTask(lock);
+  runFromMainThread(lock);
+}
+
+void js::GCParallelTask::runFromMainThread(AutoLockHelperThreadState& lock) {
+  MOZ_ASSERT(isNotYetRunning(lock));
+  MOZ_ASSERT(js::CurrentThreadCanAccessRuntime(gc->rt));
+
+  if (lock.hasQueuedTasks()) {
+    // Unlock to allow task dispatch without lock held, otherwise we can wait
+    // forever.
+    AutoUnlockHelperThreadState unlock(lock);
+  }
+
+  runTask(gc->rt->gcContext(), lock);
+  setIdle(lock);
 }
 
 class MOZ_RAII AutoGCContext {
@@ -168,33 +191,75 @@ class MOZ_RAII AutoGCContext {
     MOZ_ASSERT(TlsGCContext.get() == &context);
     TlsGCContext.set(nullptr);
   }
+
+  JS::GCContext* get() { return &context; }
 };
 
 void js::GCParallelTask::runHelperThreadTask(AutoLockHelperThreadState& lock) {
-  TraceLoggerThread* logger = TraceLoggerForCurrentThread();
-  AutoTraceLog logCompile(logger, TraceLogger_GC);
-
-  setRunning(lock);
-
   AutoGCContext gcContext(gc->rt);
-
-  runTask(lock);
-
-  setFinished(lock);
+  runTask(gcContext.get(), lock);
+  MOZ_ASSERT(isFinished(lock));
 }
 
-void GCParallelTask::runTask(AutoLockHelperThreadState& lock) {
+void GCParallelTask::runTask(JS::GCContext* gcx,
+                             AutoLockHelperThreadState& lock) {
   // Run the task from either the main thread or a helper thread.
 
-  gc::AutoSetThreadIsPerformingGC performingGC;
+  bool wasDispatched = isDispatched(lock);
+  setRunning(lock);
+
+  AutoSetThreadGCUse setUse(gcx, use);
 
   // The hazard analysis can't tell what the call to func_ will do but it's not
   // allowed to GC.
   JS::AutoSuppressGCAnalysis nogc;
 
-  TimeStamp timeStart = ReallyNow();
+  TimeStamp timeStart = TimeStamp::Now();
   run(lock);
   duration_ = TimeSince(timeStart);
+
+  if (maybeQueueTime_) {
+    TimeDuration delay = timeStart - maybeQueueTime_;
+    gc->rt->metrics().GC_TASK_START_DELAY_US(delay);
+  }
+
+  setFinished(lock);
+  gc->onParallelTaskEnd(wasDispatched, lock);
+}
+
+void GCParallelTask::onThreadPoolDispatch() {
+  MOZ_ASSERT(!dispatchedToThreadPool);
+  dispatchedToThreadPool = true;
+}
+
+void GCRuntime::dispatchOrQueueParallelTask(
+    GCParallelTask* task, const AutoLockHelperThreadState& lock) {
+  task->setQueued(lock);
+  queuedParallelTasks.ref().insertBack(task, lock);
+  maybeDispatchParallelTasks(lock);
+}
+
+void GCRuntime::maybeDispatchParallelTasks(
+    const AutoLockHelperThreadState& lock) {
+  MOZ_ASSERT(maxParallelThreads != 0);
+  MOZ_ASSERT(dispatchedParallelTasks <= maxParallelThreads);
+
+  while (dispatchedParallelTasks < maxParallelThreads &&
+         !queuedParallelTasks.ref().isEmpty(lock)) {
+    GCParallelTask* task = queuedParallelTasks.ref().popFirst(lock);
+    task->setDispatched(lock);
+    HelperThreadState().submitTask(task, lock);
+    dispatchedParallelTasks++;
+  }
+}
+
+void GCRuntime::onParallelTaskEnd(bool wasDispatched,
+                                  const AutoLockHelperThreadState& lock) {
+  if (wasDispatched) {
+    MOZ_ASSERT(dispatchedParallelTasks != 0);
+    dispatchedParallelTasks--;
+  }
+  maybeDispatchParallelTasks(lock);
 }
 
 bool js::GCParallelTask::isIdle() const {
