@@ -25,14 +25,13 @@
 
 #include "gc/ArenaList-inl.h"
 #include "gc/GC-inl.h"
+#include "gc/Heap-inl.h"
 #include "gc/Marking-inl.h"
 #include "gc/PrivateIterators-inl.h"
 
 using namespace js;
 using namespace js::gc;
 
-using mozilla::DebugOnly;
-
 #ifdef JS_GC_ZEAL
 
 /*
@@ -120,6 +119,21 @@ class js::VerifyPreTracer final : public JS::CallbackTracer {
   ~VerifyPreTracer() { js_free(root); }
 };
 
+inline bool IgnoreForPreBarrierVerifier(JSRuntime* runtime,
+                                        JS::GCCellPtr thing) {
+  // Skip things in other runtimes.
+  if (thing.asCell()->asTenured().runtimeFromAnyThread() != runtime) {
+    return true;
+  }
+
+  // Ignore buffers as these don't escape and are not barriered.
+  if (thing.kind() == JS::TraceKind::SmallBuffer) {
+    return true;
+  }
+
+  return false;
+}
+
 /*
  * This function builds up the heap snapshot by adding edges to the current
  * node.
@@ -127,8 +141,7 @@ class js::VerifyPreTracer final : public JS::CallbackTracer {
 void VerifyPreTracer::onChild(JS::GCCellPtr thing, const char* name) {
   MOZ_ASSERT(!IsInsideNursery(thing.asCell()));
 
-  // Skip things in other runtimes.
-  if (thing.asCell()->asTenured().runtimeFromAnyThread() != runtime()) {
+  if (IgnoreForPreBarrierVerifier(runtime(), thing)) {
     return;
   }
 
@@ -197,10 +210,7 @@ void gc::GCRuntime::startVerifyPreBarriers() {
   }
 
   JSContext* cx = rt->mainContextFromOwnThread();
-
-  if (IsIncrementalGCUnsafe(rt) != GCAbortReason::None) {
-    return;
-  }
+  MOZ_ASSERT(!cx->suppressGC);
 
   number++;
 
@@ -211,6 +221,12 @@ void gc::GCRuntime::startVerifyPreBarriers() {
 
   AutoPrepareForTracing prep(cx);
 
+#  ifdef DEBUG
+  for (AllZonesIter zone(this); !zone.done(); zone.next()) {
+    zone->bufferAllocator.checkGCStateNotInUse();
+  }
+#  endif
+
   ClearMarkBits<AllZonesIter>(this);
 
   gcstats::AutoPhase ap(stats(), gcstats::PhaseKind::TRACE_HEAP);
@@ -294,8 +310,7 @@ static const uint32_t MAX_VERIFIER_EDGES = 1000;
  * been modified) must point to marked objects.
  */
 void CheckEdgeTracer::onChild(JS::GCCellPtr thing, const char* name) {
-  // Skip things in other runtimes.
-  if (thing.asCell()->asTenured().runtimeFromAnyThread() != runtime()) {
+  if (IgnoreForPreBarrierVerifier(runtime(), thing)) {
     return;
   }
 
@@ -362,7 +377,7 @@ void gc::GCRuntime::endVerifyPreBarriers() {
   MOZ_ASSERT(incrementalState == State::Mark);
   incrementalState = State::NotActive;
 
-  if (!compartmentCreated && IsIncrementalGCUnsafe(rt) == GCAbortReason::None) {
+  if (!compartmentCreated) {
     CheckEdgeTracer cetrc(rt);
 
     /* Start after the roots. */
@@ -395,11 +410,25 @@ void gc::GCRuntime::endVerifyPreBarriers() {
   marker().reset();
   resetDelayedMarking();
 
+  for (AllZonesIter zone(this); !zone.done(); zone.next()) {
+    zone->bufferAllocator.clearMarkStateAfterBarrierVerification();
+  }
+
   js_delete(trc);
 }
 
 /*** Barrier Verifier Scheduling ***/
 
+void gc::VerifyBarriers(JSRuntime* rt, VerifierType type) {
+  if (type == PreBarrierVerifier) {
+    rt->gc.verifyPreBarriers();
+  }
+
+  if (type == PostBarrierVerifier) {
+    rt->gc.verifyPostBarriers();
+  }
+}
+
 void gc::GCRuntime::verifyPreBarriers() {
   if (verifyPreData) {
     endVerifyPreBarriers();
@@ -408,9 +437,11 @@ void gc::GCRuntime::verifyPreBarriers() {
   }
 }
 
-void gc::VerifyBarriers(JSRuntime* rt, VerifierType type) {
-  if (type == PreBarrierVerifier) {
-    rt->gc.verifyPreBarriers();
+void gc::GCRuntime::verifyPostBarriers() {
+  if (hasZealMode(ZealMode::VerifierPost)) {
+    clearZealMode(ZealMode::VerifierPost);
+  } else {
+    setZeal(uint8_t(ZealMode::VerifierPost), JS::ShellDefaultGCZealFrequency);
   }
 }
 
@@ -447,18 +478,18 @@ void js::gc::GCRuntime::finishVerifier() {
 }
 
 struct GCChunkHasher {
-  using Lookup = gc::TenuredChunk*;
+  using Lookup = gc::ArenaChunk*;
 
   /*
    * Strip zeros for better distribution after multiplying by the golden
    * ratio.
    */
-  static HashNumber hash(gc::TenuredChunk* chunk) {
+  static HashNumber hash(gc::ArenaChunk* chunk) {
     MOZ_ASSERT(!(uintptr_t(chunk) & gc::ChunkMask));
     return HashNumber(uintptr_t(chunk) >> gc::ChunkShift);
   }
 
-  static bool match(gc::TenuredChunk* k, gc::TenuredChunk* l) {
+  static bool match(gc::ArenaChunk* k, gc::ArenaChunk* l) {
     MOZ_ASSERT(!(uintptr_t(k) & gc::ChunkMask));
     MOZ_ASSERT(!(uintptr_t(l) & gc::ChunkMask));
     return k == l;
@@ -475,8 +506,8 @@ class js::gc::MarkingValidator {
   GCRuntime* gc;
   bool initialized;
 
-  using BitmapMap = HashMap<TenuredChunk*, UniquePtr<MarkBitmap>, GCChunkHasher,
-                            SystemAllocPolicy>;
+  using BitmapMap = HashMap<ArenaChunk*, UniquePtr<ChunkMarkBitmap>,
+                            GCChunkHasher, SystemAllocPolicy>;
   BitmapMap map;
 };
 
@@ -487,11 +518,8 @@ void js::gc::MarkingValidator::nonIncrementalMark(AutoGCSession& session) {
   /*
    * Perform a non-incremental mark for all collecting zones and record
    * the results for later comparison.
-   *
-   * Currently this does not validate gray marking.
    */
 
-  JSRuntime* runtime = gc->rt;
   GCMarker* gcmarker = &gc->marker();
 
   MOZ_ASSERT(!gcmarker->isWeakMarking());
@@ -512,16 +540,12 @@ void js::gc::MarkingValidator::nonIncrementalMark(AutoGCSession& session) {
          chunk.next()) {
       // Bug 1842582: Allocate mark bit buffer in two stages to avoid alignment
       // restriction which we currently can't support.
-      void* buffer = js_malloc(sizeof(MarkBitmap));
+      void* buffer = js_malloc(sizeof(ChunkMarkBitmap));
       if (!buffer) {
         return;
       }
-      UniquePtr<MarkBitmap> entry(new (buffer) MarkBitmap);
-
-      MarkBitmap* bitmap = &chunk->markBits;
-      memcpy((void*)entry->bitmap, (void*)bitmap->bitmap,
-             sizeof(bitmap->bitmap));
-
+      UniquePtr<ChunkMarkBitmap> entry(new (buffer) ChunkMarkBitmap);
+      entry->copyFrom(chunk->markBits);
       if (!map.putNew(chunk, std::move(entry))) {
         return;
       }
@@ -539,11 +563,7 @@ void js::gc::MarkingValidator::nonIncrementalMark(AutoGCSession& session) {
    * For saving, smush all of the keys into one big table and split them back
    * up into per-zone tables when restoring.
    */
-  gc::EphemeronEdgeTable savedEphemeronEdges(
-      SystemAllocPolicy(), runtime->randomHashCodeScrambler());
-  if (!savedEphemeronEdges.init()) {
-    return;
-  }
+  gc::EphemeronEdgeTable savedEphemeronEdges;
 
   for (GCZonesIter zone(gc); !zone.done(); zone.next()) {
     if (!WeakMapBase::saveZoneMarkedWeakMaps(zone, markedWeakMaps)) {
@@ -551,23 +571,25 @@ void js::gc::MarkingValidator::nonIncrementalMark(AutoGCSession& session) {
     }
 
     AutoEnterOOMUnsafeRegion oomUnsafe;
-    for (auto r = zone->gcEphemeronEdges().mutableAll(); !r.empty();
-         r.popFront()) {
-      MOZ_ASSERT(r.front().key->asTenured().zone() == zone);
-      if (!savedEphemeronEdges.put(r.front().key, std::move(r.front().value))) {
+    for (auto r = zone->gcEphemeronEdges().all(); !r.empty(); r.popFront()) {
+      MOZ_ASSERT(r.front().key()->asTenured().zone() == zone);
+      if (!savedEphemeronEdges.putNew(r.front().key(),
+                                      std::move(r.front().value()))) {
         oomUnsafe.crash("saving weak keys table for validator");
       }
     }
 
-    if (!zone->gcEphemeronEdges().clear()) {
-      oomUnsafe.crash("clearing weak keys table for validator");
-    }
+    zone->gcEphemeronEdges().clearAndCompact();
   }
 
-  /* Save and restore test mark queue state. */
 #  ifdef DEBUG
-  size_t savedQueuePos = gc->queuePos;
-  mozilla::Maybe<MarkColor> savedQueueColor = gc->queueMarkColor;
+  // The test mark queue can cause spurious differences if the non-incremental
+  // marking for validation happens before the full queue has been processed,
+  // since the later part of the queue may mark things during sweeping. Disable
+  // validation if there is anything left in the queue at this point.
+  if (gc->testMarkQueueRemaining() > 0) {
+    return;
+  }
 #  endif
 
   /*
@@ -638,40 +660,36 @@ void js::gc::MarkingValidator::nonIncrementalMark(AutoGCSession& session) {
     AutoLockGC lock(gc);
     for (auto chunk = gc->allNonEmptyChunks(lock); !chunk.done();
          chunk.next()) {
-      MarkBitmap* bitmap = &chunk->markBits;
+      ChunkMarkBitmap* bitmap = &chunk->markBits;
       auto ptr = map.lookup(chunk);
       MOZ_RELEASE_ASSERT(ptr, "Chunk not found in map");
-      MarkBitmap* entry = ptr->value().get();
-      for (size_t i = 0; i < MarkBitmap::WordCount; i++) {
-        uintptr_t v = entry->bitmap[i];
-        entry->bitmap[i] = uintptr_t(bitmap->bitmap[i]);
-        bitmap->bitmap[i] = v;
-      }
+      ChunkMarkBitmap* entry = ptr->value().get();
+      ChunkMarkBitmap temp;
+      temp.copyFrom(*entry);
+      entry->copyFrom(*bitmap);
+      bitmap->copyFrom(temp);
     }
   }
 
   for (GCZonesIter zone(gc); !zone.done(); zone.next()) {
     WeakMapBase::unmarkZone(zone);
-    AutoEnterOOMUnsafeRegion oomUnsafe;
-    if (!zone->gcEphemeronEdges().clear()) {
-      oomUnsafe.crash("clearing weak keys table for validator");
-    }
+    MOZ_ASSERT(zone->gcEphemeronEdges().empty(), "unmarkZone clears the map");
   }
 
   WeakMapBase::restoreMarkedWeakMaps(markedWeakMaps);
 
-  for (auto r = savedEphemeronEdges.mutableAll(); !r.empty(); r.popFront()) {
+  for (auto r = savedEphemeronEdges.all(); !r.empty(); r.popFront()) {
     AutoEnterOOMUnsafeRegion oomUnsafe;
-    Zone* zone = r.front().key->asTenured().zone();
-    if (!zone->gcEphemeronEdges().put(r.front().key,
-                                      std::move(r.front().value))) {
+    Zone* zone = r.front().key()->asTenured().zone();
+    if (!zone->gcEphemeronEdges().putNew(r.front().key(),
+                                         std::move(r.front().value()))) {
       oomUnsafe.crash("restoring weak keys table for validator");
     }
   }
 
 #  ifdef DEBUG
-  gc->queuePos = savedQueuePos;
-  gc->queueMarkColor = savedQueueColor;
+  MOZ_ASSERT(gc->testMarkQueueRemaining() == 0);
+  MOZ_ASSERT(gc->queueMarkColor.isNothing());
 #  endif
 
   gc->incrementalState = state;
@@ -699,18 +717,19 @@ void js::gc::MarkingValidator::validate() {
       continue; /* Allocated after we did the non-incremental mark. */
     }
 
-    MarkBitmap* bitmap = ptr->value().get();
-    MarkBitmap* incBitmap = &chunk->markBits;
+    ChunkMarkBitmap* bitmap = ptr->value().get();
+    ChunkMarkBitmap* incBitmap = &chunk->markBits;
 
     for (size_t i = 0; i < ArenasPerChunk; i++) {
-      if (chunk->decommittedPages[chunk->pageIndex(i)]) {
+      size_t pageIndex = ArenaChunk::arenaToPageIndex(i);
+      if (chunk->decommittedPages[pageIndex]) {
         continue;
       }
       Arena* arena = &chunk->arenas[i];
       if (!arena->allocated()) {
         continue;
       }
-      if (!arena->zone->isGCSweeping()) {
+      if (!arena->zone()->isGCSweeping()) {
         continue;
       }
 
@@ -721,44 +740,53 @@ void js::gc::MarkingValidator::validate() {
         auto* cell = reinterpret_cast<TenuredCell*>(thing);
 
         /*
-         * If a non-incremental GC wouldn't have collected a cell, then
-         * an incremental GC won't collect it.
+         * If a non-incremental GC wouldn't have collected a cell, then an
+         * incremental GC should not collect it either. However incremental
+         * marking is conservative and is allowed to mark things that
+         * non-incremental marking would not have marked.
+         *
+         * Further, incremental marking should not result in a cell that is
+         * "less marked" than non-incremental marking. For example where
+         * non-incremental marking would have marked a cell black incremental
+         * marking is not allowed to mark it gray, since the cycle collector
+         * could then consider paths through it to be part of garbage
+         * cycles. It's OK for a cell that would have been marked gray by
+         * non-incremental marking to be marked black by incremental marking.
+         *
+         * It's OK for a cell that would not be marked by non-incremental
+         * marking to end up gray. Since the cell is unreachable according to
+         * the non-incremental marking then the cycle collector will not find
+         * it. This can happen when a barrier marks a weak map key black and the
+         * map is gray, resulting in the value being marked gray.
+         *
+         * In summary:
+         *
+         *   Non-incremental   Incremental:   Outcome:
+         *       result:         result:
+         *
+         *   White              White         OK
+         *                      Gray          OK, conservative
+         *                      Black         OK, conservative
+         *   Gray               White         Fail
+         *                      Gray          OK
+         *                      Black         OK, conservative
+         *   Black              White         Fail
+         *                      Gray          Fail
+         *                      Black         OK
          */
-        if (bitmap->isMarkedAny(cell)) {
-          if (!incBitmap->isMarkedAny(cell)) {
-            ok = false;
-            const char* color =
-                CellColorName(TenuredCell::getColor(bitmap, cell));
-            fprintf(stderr,
-                    "%p: cell not marked, but would be marked %s by "
-                    "non-incremental marking\n",
-                    cell, color);
-#  ifdef DEBUG
-            cell->dump();
-            fprintf(stderr, "\n");
-#  endif
-          }
-        }
 
-        /*
-         * If the cycle collector isn't allowed to collect an object
-         * after a non-incremental GC has run, then it isn't allowed to
-         * collected it after an incremental GC.
-         */
-        if (!bitmap->isMarkedGray(cell)) {
-          if (incBitmap->isMarkedGray(cell)) {
-            ok = false;
-            const char* color =
-                CellColorName(TenuredCell::getColor(bitmap, cell));
-            fprintf(stderr,
-                    "%p: cell marked gray, but would be marked %s by "
-                    "non-incremental marking\n",
-                    cell, color);
+        CellColor incColor = TenuredCell::getColor(incBitmap, cell);
+        CellColor nonIncColor = TenuredCell::getColor(bitmap, cell);
+        if (incColor < nonIncColor) {
+          ok = false;
+          fprintf(stderr,
+                  "%p: cell was marked %s, but would be marked %s by "
+                  "non-incremental marking\n",
+                  cell, CellColorName(incColor), CellColorName(nonIncColor));
 #  ifdef DEBUG
-            cell->dump();
-            fprintf(stderr, "\n");
+          cell->dump();
+          fprintf(stderr, "\n");
 #  endif
-          }
         }
 
         thing += Arena::thingSize(kind);
@@ -798,8 +826,8 @@ void GCRuntime::finishMarkingValidation() {
 class HeapCheckTracerBase : public JS::CallbackTracer {
  public:
   explicit HeapCheckTracerBase(JSRuntime* rt, JS::TraceOptions options);
-  bool traceHeap(AutoTraceSession& session);
-  virtual void checkCell(Cell* cell, const char* name) = 0;
+  bool traceHeap(AutoHeapSession& session);
+  virtual bool checkCell(Cell* cell, const char* name) = 0;
 
  protected:
   void dumpCellInfo(Cell* cell);
@@ -836,7 +864,7 @@ class HeapCheckTracerBase : public JS::CallbackTracer {
 
 HeapCheckTracerBase::HeapCheckTracerBase(JSRuntime* rt,
                                          JS::TraceOptions options)
-    : CallbackTracer(rt, JS::TracerKind::Callback, options),
+    : CallbackTracer(rt, JS::TracerKind::HeapCheck, options),
       failures(0),
       rt(rt),
       oom(false),
@@ -844,8 +872,6 @@ HeapCheckTracerBase::HeapCheckTracerBase(JSRuntime* rt,
 
 void HeapCheckTracerBase::onChild(JS::GCCellPtr thing, const char* name) {
   Cell* cell = thing.asCell();
-  checkCell(cell, name);
-
   if (visited.lookup(cell)) {
     return;
   }
@@ -855,6 +881,11 @@ void HeapCheckTracerBase::onChild(JS::GCCellPtr thing, const char* name) {
     return;
   }
 
+  if (!checkCell(cell, name)) {
+    // Don't trace through known bad cell.
+    return;
+  }
+
   // Don't trace into GC things owned by another runtime.
   if (cell->runtimeFromAnyThread() != rt) {
     return;
@@ -866,7 +897,7 @@ void HeapCheckTracerBase::onChild(JS::GCCellPtr thing, const char* name) {
   }
 }
 
-bool HeapCheckTracerBase::traceHeap(AutoTraceSession& session) {
+bool HeapCheckTracerBase::traceHeap(AutoHeapSession& session) {
   // The analysis thinks that traceRuntime might GC by calling a GC callback.
   JS::AutoSuppressGCAnalysis nogc;
   if (!rt->isBeingDestroyed()) {
@@ -918,13 +949,14 @@ void HeapCheckTracerBase::dumpCellPath(const char* name) {
 
 class CheckHeapTracer final : public HeapCheckTracerBase {
  public:
-  enum GCType { Moving, NonMoving };
+  enum GCType { Moving, NonMoving, VerifyPostBarriers };
 
   explicit CheckHeapTracer(JSRuntime* rt, GCType type);
-  void check(AutoTraceSession& session);
+  void check(AutoHeapSession& session);
 
  private:
-  void checkCell(Cell* cell, const char* name) override;
+  bool checkCell(Cell* cell, const char* name) override;
+  bool cellIsValid(Cell* cell);
   GCType gcType;
 };
 
@@ -936,18 +968,48 @@ inline static bool IsValidGCThingPointer(Cell* cell) {
   return (uintptr_t(cell) & CellAlignMask) == 0;
 }
 
-void CheckHeapTracer::checkCell(Cell* cell, const char* name) {
-  // Moving
-  if (!IsValidGCThingPointer(cell) ||
-      ((gcType == GCType::Moving) && !IsGCThingValidAfterMovingGC(cell)) ||
-      ((gcType == GCType::NonMoving) && cell->isForwarded())) {
-    failures++;
-    fprintf(stderr, "Bad pointer %p\n", cell);
-    dumpCellPath(name);
+bool CheckHeapTracer::checkCell(Cell* cell, const char* name) {
+  if (cellIsValid(cell)) {
+    return true;
   }
+
+  failures++;
+  fprintf(stderr, "Bad pointer %p\n", cell);
+  dumpCellPath(name);
+  return false;
 }
 
-void CheckHeapTracer::check(AutoTraceSession& session) {
+bool CheckHeapTracer::cellIsValid(Cell* cell) {
+  if (!IsValidGCThingPointer(cell)) {
+    return false;
+  }
+
+  if (gcType == GCType::Moving) {
+    return IsGCThingValidAfterMovingGC(cell);
+  }
+
+  if (gcType == GCType::NonMoving) {
+    return !cell->isForwarded();
+  }
+
+  MOZ_ASSERT(gcType == GCType::VerifyPostBarriers);
+
+  // No reachable Cell* should be in the collected part of the nursery.
+  if (runtime()->gc.nursery().inCollectedRegion(cell)) {
+    return false;
+  }
+
+  // String data should also not be in the collected part of nursery.
+  if (cell->is<JSString>() && cell->as<JSString>()->isLinear()) {
+    if (cell->as<JSString>()->asLinear().hasCharsInCollectedNurseryRegion()) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+void CheckHeapTracer::check(AutoHeapSession& session) {
   if (!traceHeap(session)) {
     return;
   }
@@ -959,6 +1021,8 @@ void CheckHeapTracer::check(AutoTraceSession& session) {
 }
 
 void js::gc::CheckHeapAfterGC(JSRuntime* rt) {
+  MOZ_ASSERT(!rt->gc.isBackgroundDecommitting());
+
   AutoTraceSession session(rt);
   CheckHeapTracer::GCType gcType;
 
@@ -975,10 +1039,11 @@ void js::gc::CheckHeapAfterGC(JSRuntime* rt) {
 class CheckGrayMarkingTracer final : public HeapCheckTracerBase {
  public:
   explicit CheckGrayMarkingTracer(JSRuntime* rt);
-  bool check(AutoTraceSession& session);
+  bool check(AutoHeapSession& session);
 
  private:
-  void checkCell(Cell* cell, const char* name) override;
+  bool checkCell(Cell* cell, const char* name) override;
+  bool isBlackToGrayEdge(Cell* parent, Cell* child);
 };
 
 CheckGrayMarkingTracer::CheckGrayMarkingTracer(JSRuntime* rt)
@@ -987,34 +1052,42 @@ CheckGrayMarkingTracer::CheckGrayMarkingTracer(JSRuntime* rt)
   // Weak gray->black edges are allowed.
 }
 
-void CheckGrayMarkingTracer::checkCell(Cell* cell, const char* name) {
+bool CheckGrayMarkingTracer::checkCell(Cell* cell, const char* name) {
   Cell* parent = parentCell();
   if (!parent) {
-    return;
+    return true;
   }
 
-  if (parent->isMarkedBlack() && cell->isMarkedGray()) {
-    failures++;
+  if (!isBlackToGrayEdge(parent, cell)) {
+    return true;
+  }
 
-    fprintf(stderr, "Found black to gray edge to ");
-    dumpCellInfo(cell);
-    fprintf(stderr, "\n");
-    dumpCellPath(name);
+  failures++;
+
+  fprintf(stderr, "Found black to gray edge to ");
+  dumpCellInfo(cell);
+  fprintf(stderr, "\n");
+  dumpCellPath(name);
 
 #  ifdef DEBUG
-    if (parent->is<JSObject>()) {
-      fprintf(stderr, "\nSource: ");
-      DumpObject(parent->as<JSObject>(), stderr);
-    }
-    if (cell->is<JSObject>()) {
-      fprintf(stderr, "\nTarget: ");
-      DumpObject(cell->as<JSObject>(), stderr);
-    }
-#  endif
+  if (parent->is<JSObject>()) {
+    fprintf(stderr, "\nSource: ");
+    DumpObject(parent->as<JSObject>(), stderr);
   }
+  if (cell->is<JSObject>()) {
+    fprintf(stderr, "\nTarget: ");
+    DumpObject(cell->as<JSObject>(), stderr);
+  }
+#  endif
+
+  return false;
 }
 
-bool CheckGrayMarkingTracer::check(AutoTraceSession& session) {
+bool CheckGrayMarkingTracer::isBlackToGrayEdge(Cell* parent, Cell* child) {
+  return parent->isMarkedBlack() && child->isMarkedGray();
+}
+
+bool CheckGrayMarkingTracer::check(AutoHeapSession& session) {
   if (!traceHeap(session)) {
     return true;  // Ignore failure.
   }
@@ -1126,6 +1199,37 @@ bool js::gc::CheckWeakMapEntryMarking(const WeakMapBase* map, Cell* key,
 
 #endif  // defined(JS_GC_ZEAL) || defined(DEBUG)
 
+#ifdef JS_GC_ZEAL
+void GCRuntime::verifyPostBarriers(AutoHeapSession& session) {
+  // Walk the entire heap to check for pointers into the nursery that should
+  // have been tracked by the store buffer.
+  CheckHeapTracer tracer(rt, CheckHeapTracer::GCType::VerifyPostBarriers);
+  tracer.check(session);
+}
+
+void GCRuntime::checkHeapBeforeMinorGC(AutoHeapSession& session) {
+  // Similar to verifyPostBarriers but run before a minor GC. Checks for tenured
+  // dependent strings pointing to nursery chars but not in the store buffer. If
+  // a tenured string cell points to a nursery string cell, then it will be in
+  // the store buffer and is fine. So this looks for tenured strings that point
+  // to tenured strings but contain nursery data.
+
+  for (ZonesIter zone(rt, SkipAtoms); !zone.done(); zone.next()) {
+    for (ArenaIter aiter(zone, gc::AllocKind::STRING); !aiter.done();
+         aiter.next()) {
+      for (ArenaCellIterUnderGC cell(aiter.get()); !cell.done(); cell.next()) {
+        if (cell->is<JSString>() && cell->as<JSString>()->isDependent()) {
+          JSDependentString* str = &cell->as<JSString>()->asDependent();
+          if (str->isTenured() && str->base()->isTenured()) {
+            MOZ_RELEASE_ASSERT(!str->hasCharsInCollectedNurseryRegion());
+          }
+        }
+      }
+    }
+  }
+}
+#endif
+
 // Return whether an arbitrary pointer is within a cell with the given
 // traceKind. Only for assertions and js::debug::* APIs.
 bool GCRuntime::isPointerWithinTenuredCell(void* ptr, JS::TraceKind traceKind) {
@@ -1145,3 +1249,17 @@ bool GCRuntime::isPointerWithinTenuredCell(void* ptr, JS::TraceKind traceKind) {
 
   return false;
 }
+
+bool GCRuntime::isPointerWithinBufferAlloc(void* ptr) {
+  if (isPointerWithinTenuredCell(ptr, JS::TraceKind::SmallBuffer)) {
+    return true;
+  }
+
+  for (AllZonesIter zone(this); !zone.done(); zone.next()) {
+    if (zone->bufferAllocator.isPointerWithinMediumOrLargeBuffer(ptr)) {
+      return true;
+    }
+  }
+
+  return false;
+}
