@@ -22,6 +22,7 @@
 #include <algorithm>
 #include <cctype>
 #include <iterator>
+#include <optional>
 
 #include "irregexp/RegExpTypes.h"
 #include "irregexp/util/FlagsShim.h"
@@ -80,6 +81,7 @@ class Handle;
 #define CHECK_EQ(lhs, rhs) MOZ_RELEASE_ASSERT((lhs) == (rhs))
 #define CHECK_LE(lhs, rhs) MOZ_RELEASE_ASSERT((lhs) <= (rhs))
 #define CHECK_GE(lhs, rhs) MOZ_RELEASE_ASSERT((lhs) >= (rhs))
+#define CHECK_IMPLIES(lhs, rhs) MOZ_RELEASE_ASSERT(!(lhs) || (rhs))
 #define CONSTEXPR_DCHECK MOZ_ASSERT
 
 // These assertions are necessary to preserve the soundness of the V8
@@ -177,7 +179,7 @@ struct Use {
 template <typename Dst, typename Src>
 inline Dst saturated_cast(Src value);
 
-// This is the only specialization that is needed for regexp code.
+// These are the only specializations that are needed for regexp code.
 // Instead of pulling in dozens of lines of template goo
 // to derive it, I used the implementation from uint8_clamped in
 // ArrayBufferObject.h.
@@ -185,6 +187,10 @@ template <>
 inline uint8_t saturated_cast<uint8_t, int>(int x) {
   return (x >= 0) ? ((x < 255) ? uint8_t(x) : 255) : 0;
 }
+template <>
+inline uint8_t saturated_cast<uint8_t, uint32_t>(uint32_t x) {
+  return (x < 255) ? uint8_t(x) : 255;
+}
 
 // Origin:
 // https://github.com/v8/v8/blob/fc088cdaccadede84886eee881e67af9db53669a/src/base/bounds.h#L14-L28
@@ -201,7 +207,8 @@ inline constexpr bool IsInRange(T value, U lower_limit, U higher_limit) {
 }
 
 #define LAZY_INSTANCE_INITIALIZER \
-  {}
+  {                               \
+  }
 
 template <typename T>
 class LazyInstanceImpl {
@@ -244,21 +251,6 @@ template <typename... Args>
   return mozilla::AddToHash(aHash, aArgs...);
 }
 
-template <typename T>
-class Optional {
-  mozilla::Maybe<T> inner_;
-
- public:
-  Optional() = default;
-  Optional(T t) { inner_.emplace(t); }
-
-  bool has_value() const { return inner_.isSome(); }
-  const T& value() const { return inner_.ref(); }
-
-  T* operator->() { return &inner_.ref(); }
-  T& operator*() { return inner_.ref(); }
-};
-
 namespace bits {
 
 inline uint64_t CountTrailingZeros(uint64_t value) {
@@ -445,11 +437,13 @@ constexpr int kSystemPointerSize = sizeof(void*);
 
 // The largest integer n such that n and n + 1 are both exactly
 // representable as a Number value.  ES6 section 20.1.2.6
-constexpr double kMaxSafeInteger = 9007199254740991.0;  // 2^53-1
+constexpr uint64_t kMaxSafeIntegerUint64 = (uint64_t{1} << 53) - 1;
+constexpr double kMaxSafeInteger = static_cast<double>(kMaxSafeIntegerUint64);
 
 constexpr int kBitsPerByte = 8;
 constexpr int kBitsPerByteLog2 = 3;
 constexpr int kUInt16Size = sizeof(uint16_t);
+constexpr int kInt32Size = sizeof(int32_t);
 constexpr int kUInt32Size = sizeof(uint32_t);
 constexpr int kInt64Size = sizeof(int64_t);
 
@@ -661,6 +655,18 @@ class Tagged {
   T value_;
 };
 
+// Adapted from v8/src/objects/casting.h
+
+template <typename To, typename From>
+inline Tagged<To> UncheckedCast(Tagged<From> value) {
+  return Tagged<To>(To::cast(value));
+}
+
+template <typename To, typename From>
+inline Tagged<To> Cast(const From& value) {
+  return UncheckedCast<To>(Tagged(value));
+}
+
 // A fixed-size array with Objects (aka Values) as element types.
 // Implemented using the dense elements of an ArrayObject.
 // Used for named captures.
@@ -740,6 +746,17 @@ class ByteArray : public HeapObject {
   friend class SMRegExpMacroAssembler;
 };
 
+// A byte array that can be trusted to not contain malicious data.
+// See https://issues.chromium.org/issues/40069826.
+class TrustedByteArray : public ByteArray {
+ public:
+  static TrustedByteArray cast(Object object) {
+    TrustedByteArray b;
+    b.setValue(object.value());
+    return b;
+  }
+};
+
 // This is only used in assertions. In debug builds, we put a magic value
 // in the header of each ByteArrayData, and assert here that it matches.
 inline bool IsByteArray(Object obj) {
@@ -911,6 +928,21 @@ inline Handle<T> handle(T object, Isolate* isolate) {
   return Handle<T>(object, isolate);
 }
 
+// V8 is migrating to a conservative stack scanning approach. When that
+// is enabled, a DirectHandle points directly at the V8 heap, and an
+// IndirectHandle is an unmigrated old-style Handle with a layer of
+// indirection. When disabled (which matches our implementation) the two
+// types are the same. See:
+// https://github.com/v8/v8/blob/887ec63c43e23c4fefba1c52d4525654bdc71e5b/src/common/globals.h#L1000-L1012
+template <typename T>
+using DirectHandle = Handle<T>;
+
+template <typename T>
+using IndirectHandle = Handle<T>;
+
+template <typename T>
+using MaybeDirectHandle = MaybeHandle<T>;
+
 // RAII Guard classes
 
 using DisallowGarbageCollection = JS::AutoAssertNoGC;
@@ -1005,6 +1037,8 @@ class String : public HeapObject {
   template <typename Char>
   base::Vector<const Char> GetCharVector(
       const DisallowGarbageCollection& no_gc);
+
+  friend class RegExpUtils;
 };
 
 template <>
@@ -1023,39 +1057,20 @@ inline base::Vector<const base::uc16> String::GetCharVector(
   return flat.ToUC16Vector();
 }
 
-class JSRegExp : public HeapObject {
+using RegExpFlags = JS::RegExpFlags;
+using RegExpFlag = JS::RegExpFlags::Flag;
+
+class JSRegExp {
  public:
-  JSRegExp() : HeapObject() {}
-  JSRegExp(js::RegExpShared* re) { setValue(JS::PrivateGCThingValue(re)); }
-
-  // ******************************************************
-  // Methods that are called from inside the implementation
-  // ******************************************************
-  void TierUpTick() { inner()->tierUpTick(); }
-
-  Object bytecode(bool is_latin1) const {
-    return Object(JS::PrivateValue(inner()->getByteCode(is_latin1)));
-  }
-
-  // TODO: should we expose this?
-  uint32_t backtrack_limit() const { return 0; }
-
-  static JSRegExp cast(Object object) {
-    JSRegExp regexp;
-    js::gc::Cell* regexpShared = object.value().toGCThing();
-    MOZ_ASSERT(regexpShared->is<js::RegExpShared>());
-    regexp.setValue(JS::PrivateGCThingValue(regexpShared));
-    return regexp;
-  }
-
   // Each capture (including the match itself) needs two registers.
   static constexpr int RegistersForCaptureCount(int count) {
     return (count + 1) * 2;
   }
 
-  inline uint32_t max_register_count() const {
-    return inner()->getMaxRegisters();
-  }
+  static RegExpFlags AsRegExpFlags(RegExpFlags flags) { return flags; }
+  static RegExpFlags AsJSRegExpFlags(RegExpFlags flags) { return flags; }
+
+  static Handle<String> StringFromFlags(Isolate* isolate, RegExpFlags flags);
 
   // ******************************
   // Static constants
@@ -1064,6 +1079,44 @@ class JSRegExp : public HeapObject {
   static constexpr int kMaxCaptures = (1 << 15) - 1;
 
   static constexpr int kNoBacktrackLimit = 0;
+};
+
+class IrRegExpData : public HeapObject {
+ public:
+  IrRegExpData() : HeapObject() {}
+  IrRegExpData(js::RegExpShared* re) { setValue(JS::PrivateGCThingValue(re)); }
+
+  // ******************************************************
+  // Methods that are called from inside the implementation
+  // ******************************************************
+  void TierUpTick() { inner()->tierUpTick(); }
+
+  Tagged<TrustedByteArray> bytecode(bool is_latin1) const {
+    return TrustedByteArray::cast(
+        Object(JS::PrivateValue(inner()->getByteCode(is_latin1))));
+  }
+
+  // TODO: should we expose this?
+  uint32_t backtrack_limit() const { return 0; }
+
+  static IrRegExpData cast(Object object) {
+    IrRegExpData regexp;
+    js::gc::Cell* regexpShared = object.value().toGCThing();
+    MOZ_ASSERT(regexpShared->is<js::RegExpShared>());
+    regexp.setValue(JS::PrivateGCThingValue(regexpShared));
+    return regexp;
+  }
+
+  inline uint32_t max_register_count() const {
+    return inner()->getMaxRegisters();
+  }
+
+  RegExpFlags flags() const { return inner()->getFlags(); }
+
+  size_t capture_count() const {
+    // Subtract 1 because pairCount includes the implicit global capture.
+    return inner()->pairCount() - 1;
+  }
 
  private:
   js::RegExpShared* inner() const {
@@ -1071,9 +1124,6 @@ class JSRegExp : public HeapObject {
   }
 };
 
-using RegExpFlags = JS::RegExpFlags;
-using RegExpFlag = JS::RegExpFlags::Flag;
-
 inline bool IsUnicode(RegExpFlags flags) { return flags.unicode(); }
 inline bool IsGlobal(RegExpFlags flags) { return flags.global(); }
 inline bool IsIgnoreCase(RegExpFlags flags) { return flags.ignoreCase(); }
@@ -1085,13 +1135,15 @@ inline bool IsEitherUnicode(RegExpFlags flags) {
   return flags.unicode() || flags.unicodeSets();
 }
 
-inline base::Optional<RegExpFlag> TryRegExpFlagFromChar(char c) {
+inline std::optional<RegExpFlag> TryRegExpFlagFromChar(char c) {
   RegExpFlag flag;
 
   // The parser only calls this after verifying that it's a supported flag.
-  MOZ_ALWAYS_TRUE(JS::MaybeParseRegExpFlag(c, &flag));
+  if (JS::MaybeParseRegExpFlag(c, &flag)) {
+    return flag;
+  }
 
-  return base::Optional(flag);
+  return std::optional<RegExpFlag>{};
 }
 
 inline bool operator==(const RegExpFlags& lhs, const int& rhs) {
@@ -1169,6 +1221,9 @@ class Isolate {
   Handle<ByteArray> NewByteArray(
       int length, AllocationType allocation = AllocationType::kYoung);
 
+  Handle<TrustedByteArray> NewTrustedByteArray(
+      int length, AllocationType allocation = AllocationType::kYoung);
+
   // Allocates a fixed array initialized with undefined values.
   Handle<FixedArray> NewFixedArray(int length);
 
@@ -1298,6 +1353,9 @@ class Code : public HeapObject {
 // (NativeRegExpMacroAssembler::CheckStackGuardState)
 class InstructionStream {};
 
+// Only used in the definition of RegExpGlobalExecRunner, which we don't use.
+class RegExpResultVectorScope {};
+
 // Origin: https://github.com/v8/v8/blob/master/src/codegen/label.h
 class Label {
  public:
@@ -1322,6 +1380,12 @@ class Label {
   friend class SMRegExpMacroAssembler;
 };
 
+class RegExpUtils {
+ public:
+  static uint64_t AdvanceStringIndex(Tagged<String> string, uint64_t index,
+                                     bool unicode);
+};
+
 #define v8_flags js::jit::JitOptions
 
 #define V8_USE_COMPUTED_GOTO 1
