@@ -312,10 +312,7 @@ void RegExpBytecodeGenerator::CheckCharacterNotInRange(base::uc16 from,
   EmitOrLink(on_not_in_range);
 }
 
-void RegExpBytecodeGenerator::CheckBitInTable(Handle<ByteArray> table,
-                                              Label* on_bit_set) {
-  Emit(BC_CHECK_BIT_IN_TABLE, 0);
-  EmitOrLink(on_bit_set);
+void RegExpBytecodeGenerator::EmitSkipTable(DirectHandle<ByteArray> table) {
   for (int i = 0; i < kTableSize; i += kBitsPerByte) {
     int byte = 0;
     for (int j = 0; j < kBitsPerByte; j++) {
@@ -325,6 +322,25 @@ void RegExpBytecodeGenerator::CheckBitInTable(Handle<ByteArray> table,
   }
 }
 
+void RegExpBytecodeGenerator::CheckBitInTable(Handle<ByteArray> table,
+                                              Label* on_bit_set) {
+  Emit(BC_CHECK_BIT_IN_TABLE, 0);
+  EmitOrLink(on_bit_set);
+  EmitSkipTable(table);
+}
+
+void RegExpBytecodeGenerator::SkipUntilBitInTable(
+    int cp_offset, Handle<ByteArray> table, Handle<ByteArray> nibble_table,
+    int advance_by) {
+  Label cont;
+  Emit(BC_SKIP_UNTIL_BIT_IN_TABLE, cp_offset);
+  Emit32(advance_by);
+  EmitSkipTable(table);
+  EmitOrLink(&cont);  // goto_when_match
+  EmitOrLink(&cont);  // goto_on_failure
+  Bind(&cont);
+}
+
 void RegExpBytecodeGenerator::CheckNotBackReference(int start_reg,
                                                     bool read_backward,
                                                     Label* on_not_equal) {
@@ -373,16 +389,17 @@ void RegExpBytecodeGenerator::IfRegisterEqPos(int register_index,
   EmitOrLink(on_eq);
 }
 
-Handle<HeapObject> RegExpBytecodeGenerator::GetCode(Handle<String> source) {
+DirectHandle<HeapObject> RegExpBytecodeGenerator::GetCode(
+    DirectHandle<String> source, RegExpFlags flags) {
   Bind(&backtrack_);
   Backtrack();
 
-  Handle<ByteArray> array;
+  DirectHandle<TrustedByteArray> array;
   if (v8_flags.regexp_peephole_optimization) {
     array = RegExpBytecodePeepholeOptimization::OptimizeBytecode(
         isolate_, zone(), source, buffer_.data(), length(), jump_edges_);
   } else {
-    array = isolate_->factory()->NewByteArray(length());
+    array = isolate_->factory()->NewTrustedByteArray(length());
     Copy(array->begin());
   }
 
