@@ -24,7 +24,7 @@ class V8_EXPORT_PRIVATE RegExpBytecodeGenerator : public RegExpMacroAssembler {
   RegExpBytecodeGenerator(Isolate* isolate, Zone* zone);
   ~RegExpBytecodeGenerator() override;
   // The byte-code interpreter checks on each push anyway.
-  int stack_limit_slack() override { return 1; }
+  int stack_limit_slack_slot_count() override { return 1; }
   bool CanReadUnaligned() const override { return false; }
   void Bind(Label* label) override;
   void AdvanceCurrentPosition(int by) override;  // Signed cp change.
@@ -83,6 +83,9 @@ class V8_EXPORT_PRIVATE RegExpBytecodeGenerator : public RegExpMacroAssembler {
     return false;
   }
   void CheckBitInTable(Handle<ByteArray> table, Label* on_bit_set) override;
+  void SkipUntilBitInTable(int cp_offset, Handle<ByteArray> table,
+                           Handle<ByteArray> nibble_table,
+                           int advance_by) override;
   void CheckNotBackReference(int start_reg, bool read_backward,
                              Label* on_no_match) override;
   void CheckNotBackReferenceIgnoreCase(int start_reg, bool read_backward,
@@ -93,7 +96,8 @@ class V8_EXPORT_PRIVATE RegExpBytecodeGenerator : public RegExpMacroAssembler {
   void IfRegisterEqPos(int register_index, Label* if_eq) override;
 
   IrregexpImplementation Implementation() override;
-  Handle<HeapObject> GetCode(Handle<String> source) override;
+  DirectHandle<HeapObject> GetCode(DirectHandle<String> source,
+                                   RegExpFlags flags) override;
 
  private:
   void ExpandBuffer();
@@ -105,6 +109,7 @@ class V8_EXPORT_PRIVATE RegExpBytecodeGenerator : public RegExpMacroAssembler {
   inline void Emit8(uint32_t x);
   inline void Emit(uint32_t bc, uint32_t arg);
   inline void Emit(uint32_t bc, int32_t arg);
+  void EmitSkipTable(DirectHandle<ByteArray> table);
   // Bytecode buffer.
   int length();
   void Copy(uint8_t* a);
