@@ -1,17 +1,16 @@
 // Overview:
 // - The outer function is an IIFE which gets marked as a singleton.
-// - The |fn| inner function is then also marked as a singleton.
+// - The |o[index]| inner function is then also marked as a singleton.
+// - The |o[index]| inner function has a dynamic name from a computed property name.
 // - The |self| inner function uses |Function.prototype.caller| to reinvoke the outer function.
 //
 // When we reinvoke outer, we end up cloning a previously reused, i.e. non-cloned,
-// function.
+// function which triggered an assertion in js::SetFunctionNameIfNoOwnName().
 
 (function(index) {
-    var fn = function(a) {};
-
-    // Accessing |.length| sets the resolved-length flag, which should not be
-    // copied over to the function clone.
-    assertEq(fn.length, 1);
+    var o = {
+        [index]: function() {}
+    };
 
     // Reinvoke the IIFE through |Function.prototype.caller|.
     if (index === 0) {
