@@ -6,7 +6,7 @@ function f(i) {
     let val = 0;
     function g() {
         eval("");
-        val = 1;
+        val ||= 1;
     }
     g();
 }
