@@ -1,7 +1,7 @@
 load(libdir + 'asserts.js');
 
 function test() {
-  if (getBuildConfiguration()["pointer-byte-size"] == 4) {
+  if (getBuildConfiguration("pointer-byte-size") == 4) {
     let big_array = ctypes.int32_t.array(0xfffffff);
     assertRangeErrorMessage(() => { ctypes.StructType("x", [{a: big_array},
                                                             {b: big_array},
@@ -9,7 +9,7 @@ function test() {
                                                             {d: big_array},
                                                             {e: big_array}]); },
                             "struct size does not fit in size_t");
-  } else if (getBuildConfiguration()["pointer-byte-size"] == 8) {
+  } else if (getBuildConfiguration("pointer-byte-size") == 8) {
     let big_array = ctypes.int32_t.array(0xfffffffffffffff);
     assertRangeErrorMessage(() => { ctypes.StructType("x", [{a: big_array},
                                                             {b: big_array},
