@@ -1,4 +1,4 @@
-// |jit-test| allow-oom; skip-if: !('oomAfterAllocations' in this) || helperThreadCount() === 0
+// |jit-test| allow-oom; skip-if: !hasFunction.oomAfterAllocations || helperThreadCount() === 0
 
 lfcode = new Array();
 dbg = Debugger();
@@ -17,6 +17,6 @@ function loadFile(lfVarx) {
       if (!(lfLocal in lfGlobal))
           lfGlobal[lfLocal] = this[lfLocal]
   offThreadCompileToStencil(lfVarx);
-  var stencil = lfGlobal.finishOffThreadCompileToStencil();
+  var stencil = lfGlobal.finishOffThreadStencil();
   lfGlobal.evalStencil(stencil);
 }
