@@ -1,4 +1,4 @@
-// |jit-test| slow; skip-if: !('oomTest' in this)
+// |jit-test| slow
 
 loadFile(`
   var o = {}
@@ -19,5 +19,5 @@ function loadFile(lfVarx) {
         // In practice a crash occurs before iteration 4000.
         if (k++ <= 4000)
           eval(lfVarx);
-    })
+    }, {expectExceptionOnFailure: false});
 }
