@@ -1,12 +1,12 @@
 var obj;
 const proto = {
   get prop() {
-    Object.defineProperty(obj, "prop", { value:  true});
+    delete proto.prop;
     return false;
   }
 }
 obj = new Object(proto);
 obj[Symbol.unscopables] = proto;
 with (obj) {
-  assertEq(prop, true);
+  assertEq(prop, undefined);
 }
