@@ -1,6 +1,5 @@
-// |jit-test| skip-if: !('oomTest' in this)
 oomTest(function () {
     offThreadCompileModuleToStencil('');
-    var stencil = finishOffThreadCompileModuleToStencil();
+    var stencil = finishOffThreadStencil();
     instantiateModuleStencil(stencil);
 });
