@@ -9,7 +9,7 @@ let b = registerModule('b', parseModule(
     \`import * as ns from 'a';
      export var x = ns.a + ns.b;\`
 ));
-b.declarationInstantiation();
+moduleLink(b);
 let ns = getModuleEnvironmentValue(b, "ns");
 assertEq(ns.a, 1);
 while ( t.ArrayType() ) 1;
