@@ -9,8 +9,8 @@ let a = registerModule('a', parseModule(s));
 
 let b = registerModule('b', parseModule("import * as ns from 'a'"));
 
-b.declarationInstantiation();
-b.evaluation();
+moduleLink(b);
+moduleEvaluate(b);
 
 let ns = a.namespace;
 for (let i = 0; i < count; i++)
