@@ -7,5 +7,5 @@ function f() {}
 Object.freeze(this);
 `);
 
-evaluate(code, { global, saveIncrementalBytecode: true });
-evaluate(code, { global, saveIncrementalBytecode: true });
+evaluate(code, { global, saveBytecodeWithDelazifications: true });
+evaluate(code, { global, saveBytecodeWithDelazifications: true });
