@@ -1,4 +1,4 @@
-// |jit-test| skip-if: !wasmGcEnabled(); test-also=--gc-zeal=2
+// |jit-test| test-also=--gc-zeal=2
 
 // Test array instructions on different valtypes
 
@@ -101,7 +101,7 @@ for (let [valtype, def, nondef] of GENERAL_TESTS) {
     }
   }
 
-  for (let arrayLength = 0; arrayLength < 5; arrayLength++) {
+  for (let arrayLength = 0; arrayLength <= 25; arrayLength++) {
     checkArray(createDefault(arrayLength), arrayLength, def, nondef);
     checkArray(create(arrayLength, nondef), arrayLength, nondef, def);
   }
@@ -1256,14 +1256,14 @@ assertErrorMessage(() => wasmEvalText(`(module
   let { initData } = wasmEvalText(`(module
     (type $a (array (mut i8)))
     (data $other "\\\\9")
-    (data $d "1337")
+    (data $d "xX1337Xx")
     (func (export "initData") (result eqref)
       (local $arr (ref $a))
-      (local.set $arr (array.new_default $a (i32.const 4)))
+      (local.set $arr (array.new_default $a (i32.const 6)))
 
       (; array to init ;)       local.get $arr
-      (; offset=0 into array ;) i32.const 0
-      (; offset=0 into data ;)  i32.const 0
+      (; offset=1 into array ;) i32.const 1
+      (; offset=2 into data ;)  i32.const 2
       (; size=4 elements ;)     i32.const 4
       array.init_data $a $d
 
@@ -1272,11 +1272,13 @@ assertErrorMessage(() => wasmEvalText(`(module
     (func data.drop 0) ;; force write of data count section, see https://github.com/bytecodealliance/wasm-tools/pull/1194
   )`).exports;
   let arr = initData();
-  assertEq(wasmGcArrayLength(arr), 4);
-  assertEq(wasmGcReadField(arr, 0), 48+1);
-  assertEq(wasmGcReadField(arr, 1), 48+3);
+  assertEq(wasmGcArrayLength(arr), 6);
+  assertEq(wasmGcReadField(arr, 0), 0);
+  assertEq(wasmGcReadField(arr, 1), 48+1);
   assertEq(wasmGcReadField(arr, 2), 48+3);
-  assertEq(wasmGcReadField(arr, 3), 48+7);
+  assertEq(wasmGcReadField(arr, 3), 48+3);
+  assertEq(wasmGcReadField(arr, 4), 48+7);
+  assertEq(wasmGcReadField(arr, 5), 0);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1635,18 +1637,19 @@ assertErrorMessage(() => wasmEvalText(`(module
 {
   let { initElem, f1, f2, f3, f4 } = wasmEvalText(`(module
     (type $a (array (mut funcref)))
-    (elem $e func $f1 $f2 $f3 $f4)
+    (elem $e func $f5 $f5 $f1 $f2 $f3 $f4 $f5 $f5)
     (func $f1 (export "f1"))
     (func $f2 (export "f2"))
     (func $f3 (export "f3"))
     (func $f4 (export "f4"))
+    (func $f5)
     (func (export "initElem") (result eqref)
       (local $arr (ref $a))
-      (local.set $arr (array.new_default $a (i32.const 4)))
+      (local.set $arr (array.new_default $a (i32.const 6)))
 
       (; array to init ;)       local.get $arr
-      (; offset=0 into array ;) i32.const 0
-      (; offset=0 into elem ;)  i32.const 0
+      (; offset=1 into array ;) i32.const 1
+      (; offset=2 into elem ;)  i32.const 2
       (; size=4 into elem ;)    i32.const 4
       array.init_elem $a $e
 
@@ -1654,11 +1657,13 @@ assertErrorMessage(() => wasmEvalText(`(module
     )
   )`).exports;
   let arr = initElem();
-  assertEq(wasmGcArrayLength(arr), 4);
-  assertEq(wasmGcReadField(arr, 0), f1);
-  assertEq(wasmGcReadField(arr, 1), f2);
-  assertEq(wasmGcReadField(arr, 2), f3);
-  assertEq(wasmGcReadField(arr, 3), f4);
+  assertEq(wasmGcArrayLength(arr), 6);
+  assertEq(wasmGcReadField(arr, 0), null);
+  assertEq(wasmGcReadField(arr, 1), f1);
+  assertEq(wasmGcReadField(arr, 2), f2);
+  assertEq(wasmGcReadField(arr, 3), f3);
+  assertEq(wasmGcReadField(arr, 4), f4);
+  assertEq(wasmGcReadField(arr, 5), null);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -2062,57 +2067,6 @@ assertErrorMessage(() => wasmEvalText(`(module
   }
 }
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Checks for requests for oversize arrays (more than MaxArrayPayloadBytes),
-// where MaxArrayPayloadBytes == 1,987,654,321.
-
-// array.new
-assertErrorMessage(() => wasmEvalText(`(module
-    (type $a (array i32))
-    (func (export "test") (result eqref)
-        ;; request exactly 2,000,000,000 bytes
-        (array.new $a (i32.const 0xABCD1234) (i32.const 500000000))
-    )
-    (func $f
-      call 0
-      drop
-    )
-    (start $f)
-)
-`), WebAssembly.RuntimeError, /too many array elements/);
-
-// array.new_default
-assertErrorMessage(() => wasmEvalText(`(module
-    (type $a (array f64))
-    (func (export "test") (result eqref)
-        ;; request exactly 2,000,000,000 bytes
-        (array.new_default $a (i32.const 250000000))
-    )
-    (func $f
-      call 0
-      drop
-    )
-    (start $f)
-)
-`), WebAssembly.RuntimeError, /too many array elements/);
-
-// array.new_fixed
-// This is impossible to test because it would require to create, at a
-// minimum, 1,987,654,321 (MaxArrayPayloadBytes) / 16 = 124.3 million
-// values, if each value is a v128.  However, the max number of bytes per
-// function is 7,654,321 (MaxFunctionBytes).  Even if it were possible to
-// create a value using just one insn byte, there wouldn't be enough.
-
-// array.new_data
-// Similarly, impossible to test because the max data segment length is 1GB
-// (1,073,741,824 bytes) (MaxDataSegmentLengthPages * PageSize), which is less
-// than MaxArrayPayloadBytes.
-
-// array.new_element
-// Similarly, impossible to test because an element segment can contain at
-// most 10,000,000 (MaxElemSegmentLength) entries.
-
 // Test whether array data pointers are correctly tracked in stack maps.
 {
   const { newArray, test } = wasmEvalText(`(module
