@@ -43,28 +43,19 @@ assert_return(() => invoke($0, `store`, [-4n, 42]), []);
 assert_return(() => invoke($0, `load`, [-4n]), [value("i32", 42)]);
 
 // ./test/core/memory_trap64.wast:23
-assert_trap(
-  () => invoke($0, `store`, [-3n, 13]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($0, `store`, [-3n, 13]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:24
 assert_trap(() => invoke($0, `load`, [-3n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:25
-assert_trap(
-  () => invoke($0, `store`, [-2n, 13]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($0, `store`, [-2n, 13]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:26
 assert_trap(() => invoke($0, `load`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:27
-assert_trap(
-  () => invoke($0, `store`, [-1n, 13]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($0, `store`, [-1n, 13]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:28
 assert_trap(() => invoke($0, `load`, [-1n]), `out of bounds memory access`);
@@ -76,16 +67,10 @@ assert_trap(() => invoke($0, `store`, [0n, 13]), `out of bounds memory access`);
 assert_trap(() => invoke($0, `load`, [0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:31
-assert_trap(
-  () => invoke($0, `store`, [2147483648n, 13]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($0, `store`, [2147483648n, 13]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:32
-assert_trap(
-  () => invoke($0, `load`, [2147483648n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($0, `load`, [2147483648n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:34
 let $1 = instantiate(`(module
@@ -165,436 +150,220 @@ let $1 = instantiate(`(module
 )`);
 
 // ./test/core/memory_trap64.wast:110
-assert_trap(
-  () => invoke($1, `i32.store`, [65536n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [65536n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:111
-assert_trap(
-  () => invoke($1, `i32.store`, [65535n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [65535n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:112
-assert_trap(
-  () => invoke($1, `i32.store`, [65534n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [65534n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:113
-assert_trap(
-  () => invoke($1, `i32.store`, [65533n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [65533n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:114
-assert_trap(
-  () => invoke($1, `i32.store`, [-1n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [-1n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:115
-assert_trap(
-  () => invoke($1, `i32.store`, [-2n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [-2n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:116
-assert_trap(
-  () => invoke($1, `i32.store`, [-3n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [-3n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:117
-assert_trap(
-  () => invoke($1, `i32.store`, [-4n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store`, [-4n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:118
-assert_trap(
-  () => invoke($1, `i64.store`, [65536n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65536n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:119
-assert_trap(
-  () => invoke($1, `i64.store`, [65535n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65535n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:120
-assert_trap(
-  () => invoke($1, `i64.store`, [65534n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65534n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:121
-assert_trap(
-  () => invoke($1, `i64.store`, [65533n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65533n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:122
-assert_trap(
-  () => invoke($1, `i64.store`, [65532n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65532n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:123
-assert_trap(
-  () => invoke($1, `i64.store`, [65531n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65531n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:124
-assert_trap(
-  () => invoke($1, `i64.store`, [65530n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65530n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:125
-assert_trap(
-  () => invoke($1, `i64.store`, [65529n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [65529n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:126
-assert_trap(
-  () => invoke($1, `i64.store`, [-1n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-1n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:127
-assert_trap(
-  () => invoke($1, `i64.store`, [-2n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-2n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:128
-assert_trap(
-  () => invoke($1, `i64.store`, [-3n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-3n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:129
-assert_trap(
-  () => invoke($1, `i64.store`, [-4n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-4n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:130
-assert_trap(
-  () => invoke($1, `i64.store`, [-5n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-5n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:131
-assert_trap(
-  () => invoke($1, `i64.store`, [-6n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-6n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:132
-assert_trap(
-  () => invoke($1, `i64.store`, [-7n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-7n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:133
-assert_trap(
-  () => invoke($1, `i64.store`, [-8n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store`, [-8n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:134
-assert_trap(
-  () => invoke($1, `f32.store`, [65536n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [65536n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:135
-assert_trap(
-  () => invoke($1, `f32.store`, [65535n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [65535n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:136
-assert_trap(
-  () => invoke($1, `f32.store`, [65534n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [65534n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:137
-assert_trap(
-  () => invoke($1, `f32.store`, [65533n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [65533n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:138
-assert_trap(
-  () => invoke($1, `f32.store`, [-1n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [-1n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:139
-assert_trap(
-  () => invoke($1, `f32.store`, [-2n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [-2n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:140
-assert_trap(
-  () => invoke($1, `f32.store`, [-3n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [-3n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:141
-assert_trap(
-  () => invoke($1, `f32.store`, [-4n, value("f32", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.store`, [-4n, value("f32", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:142
-assert_trap(
-  () => invoke($1, `f64.store`, [65536n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65536n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:143
-assert_trap(
-  () => invoke($1, `f64.store`, [65535n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65535n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:144
-assert_trap(
-  () => invoke($1, `f64.store`, [65534n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65534n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:145
-assert_trap(
-  () => invoke($1, `f64.store`, [65533n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65533n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:146
-assert_trap(
-  () => invoke($1, `f64.store`, [65532n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65532n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:147
-assert_trap(
-  () => invoke($1, `f64.store`, [65531n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65531n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:148
-assert_trap(
-  () => invoke($1, `f64.store`, [65530n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65530n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:149
-assert_trap(
-  () => invoke($1, `f64.store`, [65529n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [65529n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:150
-assert_trap(
-  () => invoke($1, `f64.store`, [-1n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-1n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:151
-assert_trap(
-  () => invoke($1, `f64.store`, [-2n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-2n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:152
-assert_trap(
-  () => invoke($1, `f64.store`, [-3n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-3n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:153
-assert_trap(
-  () => invoke($1, `f64.store`, [-4n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-4n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:154
-assert_trap(
-  () => invoke($1, `f64.store`, [-5n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-5n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:155
-assert_trap(
-  () => invoke($1, `f64.store`, [-6n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-6n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:156
-assert_trap(
-  () => invoke($1, `f64.store`, [-7n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-7n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:157
-assert_trap(
-  () => invoke($1, `f64.store`, [-8n, value("f64", 0)]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.store`, [-8n, value("f64", 0)]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:158
-assert_trap(
-  () => invoke($1, `i32.store8`, [65536n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store8`, [65536n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:159
-assert_trap(
-  () => invoke($1, `i32.store8`, [-1n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store8`, [-1n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:160
-assert_trap(
-  () => invoke($1, `i32.store16`, [65536n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store16`, [65536n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:161
-assert_trap(
-  () => invoke($1, `i32.store16`, [65535n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store16`, [65535n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:162
-assert_trap(
-  () => invoke($1, `i32.store16`, [-1n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store16`, [-1n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:163
-assert_trap(
-  () => invoke($1, `i32.store16`, [-2n, 0]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.store16`, [-2n, 0]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:164
-assert_trap(
-  () => invoke($1, `i64.store8`, [65536n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store8`, [65536n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:165
-assert_trap(
-  () => invoke($1, `i64.store8`, [-1n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store8`, [-1n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:166
-assert_trap(
-  () => invoke($1, `i64.store16`, [65536n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store16`, [65536n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:167
-assert_trap(
-  () => invoke($1, `i64.store16`, [65535n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store16`, [65535n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:168
-assert_trap(
-  () => invoke($1, `i64.store16`, [-1n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store16`, [-1n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:169
-assert_trap(
-  () => invoke($1, `i64.store16`, [-2n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store16`, [-2n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:170
-assert_trap(
-  () => invoke($1, `i64.store32`, [65536n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [65536n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:171
-assert_trap(
-  () => invoke($1, `i64.store32`, [65535n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [65535n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:172
-assert_trap(
-  () => invoke($1, `i64.store32`, [65534n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [65534n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:173
-assert_trap(
-  () => invoke($1, `i64.store32`, [65533n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [65533n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:174
-assert_trap(
-  () => invoke($1, `i64.store32`, [-1n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [-1n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:175
-assert_trap(
-  () => invoke($1, `i64.store32`, [-2n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [-2n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:176
-assert_trap(
-  () => invoke($1, `i64.store32`, [-3n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [-3n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:177
-assert_trap(
-  () => invoke($1, `i64.store32`, [-4n, 0n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.store32`, [-4n, 0n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:178
-assert_trap(
-  () => invoke($1, `i32.load`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:179
-assert_trap(
-  () => invoke($1, `i32.load`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:180
-assert_trap(
-  () => invoke($1, `i32.load`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:181
-assert_trap(
-  () => invoke($1, `i32.load`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:182
 assert_trap(() => invoke($1, `i32.load`, [-1n]), `out of bounds memory access`);
@@ -609,52 +378,28 @@ assert_trap(() => invoke($1, `i32.load`, [-3n]), `out of bounds memory access`);
 assert_trap(() => invoke($1, `i32.load`, [-4n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:186
-assert_trap(
-  () => invoke($1, `i64.load`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:187
-assert_trap(
-  () => invoke($1, `i64.load`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:188
-assert_trap(
-  () => invoke($1, `i64.load`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:189
-assert_trap(
-  () => invoke($1, `i64.load`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:190
-assert_trap(
-  () => invoke($1, `i64.load`, [65532n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65532n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:191
-assert_trap(
-  () => invoke($1, `i64.load`, [65531n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65531n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:192
-assert_trap(
-  () => invoke($1, `i64.load`, [65530n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65530n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:193
-assert_trap(
-  () => invoke($1, `i64.load`, [65529n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load`, [65529n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:194
 assert_trap(() => invoke($1, `i64.load`, [-1n]), `out of bounds memory access`);
@@ -681,28 +426,16 @@ assert_trap(() => invoke($1, `i64.load`, [-7n]), `out of bounds memory access`);
 assert_trap(() => invoke($1, `i64.load`, [-8n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:202
-assert_trap(
-  () => invoke($1, `f32.load`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.load`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:203
-assert_trap(
-  () => invoke($1, `f32.load`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.load`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:204
-assert_trap(
-  () => invoke($1, `f32.load`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.load`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:205
-assert_trap(
-  () => invoke($1, `f32.load`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f32.load`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:206
 assert_trap(() => invoke($1, `f32.load`, [-1n]), `out of bounds memory access`);
@@ -717,52 +450,28 @@ assert_trap(() => invoke($1, `f32.load`, [-3n]), `out of bounds memory access`);
 assert_trap(() => invoke($1, `f32.load`, [-4n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:210
-assert_trap(
-  () => invoke($1, `f64.load`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:211
-assert_trap(
-  () => invoke($1, `f64.load`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:212
-assert_trap(
-  () => invoke($1, `f64.load`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:213
-assert_trap(
-  () => invoke($1, `f64.load`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:214
-assert_trap(
-  () => invoke($1, `f64.load`, [65532n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65532n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:215
-assert_trap(
-  () => invoke($1, `f64.load`, [65531n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65531n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:216
-assert_trap(
-  () => invoke($1, `f64.load`, [65530n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65530n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:217
-assert_trap(
-  () => invoke($1, `f64.load`, [65529n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `f64.load`, [65529n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:218
 assert_trap(() => invoke($1, `f64.load`, [-1n]), `out of bounds memory access`);
@@ -789,244 +498,124 @@ assert_trap(() => invoke($1, `f64.load`, [-7n]), `out of bounds memory access`);
 assert_trap(() => invoke($1, `f64.load`, [-8n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:226
-assert_trap(
-  () => invoke($1, `i32.load8_s`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load8_s`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:227
-assert_trap(
-  () => invoke($1, `i32.load8_s`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load8_s`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:228
-assert_trap(
-  () => invoke($1, `i32.load8_u`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load8_u`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:229
-assert_trap(
-  () => invoke($1, `i32.load8_u`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load8_u`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:230
-assert_trap(
-  () => invoke($1, `i32.load16_s`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_s`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:231
-assert_trap(
-  () => invoke($1, `i32.load16_s`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_s`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:232
-assert_trap(
-  () => invoke($1, `i32.load16_s`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_s`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:233
-assert_trap(
-  () => invoke($1, `i32.load16_s`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_s`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:234
-assert_trap(
-  () => invoke($1, `i32.load16_u`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_u`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:235
-assert_trap(
-  () => invoke($1, `i32.load16_u`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_u`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:236
-assert_trap(
-  () => invoke($1, `i32.load16_u`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_u`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:237
-assert_trap(
-  () => invoke($1, `i32.load16_u`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i32.load16_u`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:238
-assert_trap(
-  () => invoke($1, `i64.load8_s`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load8_s`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:239
-assert_trap(
-  () => invoke($1, `i64.load8_s`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load8_s`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:240
-assert_trap(
-  () => invoke($1, `i64.load8_u`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load8_u`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:241
-assert_trap(
-  () => invoke($1, `i64.load8_u`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load8_u`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:242
-assert_trap(
-  () => invoke($1, `i64.load16_s`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_s`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:243
-assert_trap(
-  () => invoke($1, `i64.load16_s`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_s`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:244
-assert_trap(
-  () => invoke($1, `i64.load16_s`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_s`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:245
-assert_trap(
-  () => invoke($1, `i64.load16_s`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_s`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:246
-assert_trap(
-  () => invoke($1, `i64.load16_u`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_u`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:247
-assert_trap(
-  () => invoke($1, `i64.load16_u`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_u`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:248
-assert_trap(
-  () => invoke($1, `i64.load16_u`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_u`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:249
-assert_trap(
-  () => invoke($1, `i64.load16_u`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load16_u`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:250
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:251
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:252
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:253
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:254
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:255
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:256
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [-3n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [-3n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:257
-assert_trap(
-  () => invoke($1, `i64.load32_s`, [-4n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_s`, [-4n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:258
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [65536n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [65536n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:259
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [65535n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [65535n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:260
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [65534n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [65534n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:261
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [65533n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [65533n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:262
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [-1n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [-1n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:263
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [-2n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [-2n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:264
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [-3n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [-3n]), `out of bounds memory access`);
 
 // ./test/core/memory_trap64.wast:265
-assert_trap(
-  () => invoke($1, `i64.load32_u`, [-4n]),
-  `out of bounds memory access`,
-);
+assert_trap(() => invoke($1, `i64.load32_u`, [-4n]), `out of bounds memory access`);
 
 // Bug 1737225 - do not observe the partial store caused by bug 1666747 on
 // some native platforms.
@@ -1041,4 +630,3 @@ if (!partialOobWriteMayWritePartialData()) {
         value("i64", 7523094288207667809n),
     ]);
 }
-
