@@ -12,7 +12,7 @@
 namespace js {
 namespace jit {
 
-MOZ_MUST_USE bool ReorderInstructions(MIRGraph& graph);
+[[nodiscard]] bool ReorderInstructions(MIRGenerator* mir, MIRGraph& graph);
 
 }  // namespace jit
 }  // namespace js
