@@ -413,6 +413,11 @@ class RegisterSet {
     return RegisterSet(GeneralRegisterSet::Not(in.gpr_),
                        FloatRegisterSet::Not(in.fpu_));
   }
+  static inline RegisterSet Subtract(const RegisterSet& lhs,
+                                     const RegisterSet& rhs) {
+    return RegisterSet(GeneralRegisterSet::Subtract(lhs.gpr_, rhs.gpr_),
+                       FloatRegisterSet::Subtract(lhs.fpu_, rhs.fpu_));
+  }
   static inline RegisterSet VolatileNot(const RegisterSet& in) {
     return RegisterSet(GeneralRegisterSet::VolatileNot(in.gpr_),
                        FloatRegisterSet::VolatileNot(in.fpu_));
@@ -658,9 +663,9 @@ class LiveSetAccessors<RegisterSet> {
 };
 
 #define DEFINE_ACCESSOR_CONSTRUCTORS_(REGSET)             \
-  typedef typename Parent::RegSet RegSet;                 \
-  typedef typename Parent::RegType RegType;               \
-  typedef typename Parent::SetType SetType;               \
+  using typename Parent::RegSet;                          \
+  using typename Parent::RegType;                         \
+  using typename Parent::SetType;                         \
                                                           \
   constexpr REGSET() : Parent() {}                        \
   explicit constexpr REGSET(SetType set) : Parent(set) {} \
@@ -900,7 +905,7 @@ class SpecializedRegSet<Accessors, RegisterSet> : public Accessors {
 // |TypedOrValueRegister|, and |Register64|.
 template <class Accessors, typename Set>
 class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
-  typedef SpecializedRegSet<Accessors, Set> Parent;
+  using Parent = SpecializedRegSet<Accessors, Set>;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_(CommonRegSet)
@@ -1019,7 +1024,7 @@ class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
 // only benefit of these classes is to provide user friendly names.
 template <typename Set>
 class LiveSet : public CommonRegSet<LiveSetAccessors<Set>, Set> {
-  typedef CommonRegSet<LiveSetAccessors<Set>, Set> Parent;
+  using Parent = CommonRegSet<LiveSetAccessors<Set>, Set>;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_(LiveSet)
@@ -1027,7 +1032,7 @@ class LiveSet : public CommonRegSet<LiveSetAccessors<Set>, Set> {
 
 template <typename Set>
 class AllocatableSet : public CommonRegSet<AllocatableSetAccessors<Set>, Set> {
-  typedef CommonRegSet<AllocatableSetAccessors<Set>, Set> Parent;
+  using Parent = CommonRegSet<AllocatableSetAccessors<Set>, Set>;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_(AllocatableSet)
@@ -1036,9 +1041,9 @@ class AllocatableSet : public CommonRegSet<AllocatableSetAccessors<Set>, Set> {
 };
 
 #define DEFINE_ACCESSOR_CONSTRUCTORS_FOR_REGISTERSET_(REGSET)          \
-  typedef Parent::RegSet RegSet;                                       \
-  typedef Parent::RegType RegType;                                     \
-  typedef Parent::SetType SetType;                                     \
+  using typename Parent::RegSet;                                       \
+  using typename Parent::RegType;                                      \
+  using typename Parent::SetType;                                      \
                                                                        \
   constexpr REGSET() : Parent() {}                                     \
   explicit constexpr REGSET(SetType) = delete;                         \
@@ -1054,7 +1059,7 @@ class LiveSet<RegisterSet>
   // Note: We have to provide a qualified name for LiveSetAccessors, as it is
   // interpreted as being the specialized class name inherited from the parent
   // class specialization.
-  typedef CommonRegSet<jit::LiveSetAccessors<RegisterSet>, RegisterSet> Parent;
+  using Parent = CommonRegSet<jit::LiveSetAccessors<RegisterSet>, RegisterSet>;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_FOR_REGISTERSET_(LiveSet)
@@ -1066,8 +1071,8 @@ class AllocatableSet<RegisterSet>
   // Note: We have to provide a qualified name for AllocatableSetAccessors, as
   // it is interpreted as being the specialized class name inherited from the
   // parent class specialization.
-  typedef CommonRegSet<jit::AllocatableSetAccessors<RegisterSet>, RegisterSet>
-      Parent;
+  using Parent =
+      CommonRegSet<jit::AllocatableSetAccessors<RegisterSet>, RegisterSet>;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_FOR_REGISTERSET_(AllocatableSet)
@@ -1318,8 +1323,8 @@ inline LiveGeneralRegisterSet SavedNonVolatileRegisters(
   result.add(Register::FromCode(Registers::lr));
 #elif defined(JS_CODEGEN_ARM64)
   result.add(Register::FromCode(Registers::lr));
-#elif defined(JS_CODEGEN_MIPS32) || defined(JS_CODEGEN_MIPS64) || \
-    defined(JS_CODEGEN_LOONG64) || defined(JS_CODEGEN_RISCV64)
+#elif defined(JS_CODEGEN_MIPS64) || defined(JS_CODEGEN_LOONG64) || \
+    defined(JS_CODEGEN_RISCV64)
   result.add(Register::FromCode(Registers::ra));
 #endif
 
