@@ -15,8 +15,6 @@
 #  include "jit/arm/SharedICRegisters-arm.h"
 #elif defined(JS_CODEGEN_ARM64)
 #  include "jit/arm64/SharedICRegisters-arm64.h"
-#elif defined(JS_CODEGEN_MIPS32)
-#  include "jit/mips32/SharedICRegisters-mips32.h"
 #elif defined(JS_CODEGEN_MIPS64)
 #  include "jit/mips64/SharedICRegisters-mips64.h"
 #elif defined(JS_CODEGEN_LOONG64)
