@@ -8,15 +8,13 @@
 #ifndef jit_Sink_h
 #define jit_Sink_h
 
-#include "mozilla/Attributes.h"
-
 namespace js {
 namespace jit {
 
 class MIRGenerator;
 class MIRGraph;
 
-MOZ_MUST_USE bool Sink(MIRGenerator* mir, MIRGraph& graph);
+[[nodiscard]] bool Sink(const MIRGenerator* mir, MIRGraph& graph);
 
 }  // namespace jit
 }  // namespace js
