@@ -32,8 +32,6 @@ using vixl::MemOperand;
 using vixl::Operand;
 
 struct ImmShiftedTag : public ImmWord {
-  explicit ImmShiftedTag(JSValueShiftedTag shtag) : ImmWord((uintptr_t)shtag) {}
-
   explicit ImmShiftedTag(JSValueType type)
       : ImmWord(uintptr_t(JSValueShiftedTag(JSVAL_TYPE_TO_SHIFTED_TAG(type)))) {
   }
@@ -47,7 +45,7 @@ class ScratchTagScope;
 
 class MacroAssemblerCompat : public vixl::MacroAssembler {
  public:
-  typedef vixl::Condition Condition;
+  using Condition = vixl::Condition;
 
  private:
   // Perform a downcast. Should be removed by Bug 996602.
@@ -61,10 +59,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 
  protected:
   bool enoughMemory_;
-  uint32_t framePushed_;
 
-  MacroAssemblerCompat()
-      : vixl::MacroAssembler(), enoughMemory_(true), framePushed_(0) {}
+  MacroAssemblerCompat() : vixl::MacroAssembler(), enoughMemory_(true) {}
 
  protected:
   MoveResolver moveResolver_;
@@ -101,27 +97,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Add(scratch64, base, Operand(index, vixl::LSL, scale));
     return LoadStoreMacro(rt, MemOperand(scratch64, addr.offset), op);
   }
-  void Push(ARMRegister reg) {
-    push(reg);
-    adjustFrame(reg.size() / 8);
-  }
-  void Push(Register reg) {
-    vixl::MacroAssembler::Push(ARMRegister(reg, 64));
-    adjustFrame(8);
-  }
-  void Push(Imm32 imm) {
-    push(imm);
-    adjustFrame(8);
-  }
-  void Push(FloatRegister f) {
-    push(ARMFPRegister(f, 64));
-    adjustFrame(8);
-  }
-  void Push(ImmPtr imm) {
-    push(imm);
-    adjustFrame(sizeof(void*));
-  }
   void push(FloatRegister f) {
+    MOZ_ASSERT(f.isDouble(), "float32 and simd128 not supported");
     vixl::MacroAssembler::Push(ARMFPRegister(f, 64));
   }
   void push(ARMFPRegister f) { vixl::MacroAssembler::Push(f); }
@@ -224,37 +201,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 
   void pop(const ValueOperand& v) { pop(v.valueReg()); }
   void pop(const FloatRegister& f) {
+    MOZ_ASSERT(f.isDouble(), "float32 and simd128 not supported");
     vixl::MacroAssembler::Pop(ARMFPRegister(f, 64));
   }
 
-  void implicitPop(uint32_t args) {
-    MOZ_ASSERT(args % sizeof(intptr_t) == 0);
-    adjustFrame(0 - args);
-  }
-  void Pop(ARMRegister r) {
-    vixl::MacroAssembler::Pop(r);
-    adjustFrame(0 - r.size() / 8);
-  }
-  // FIXME: This is the same on every arch.
-  // FIXME: If we can share framePushed_, we can share this.
-  // FIXME: Or just make it at the highest level.
-  CodeOffset PushWithPatch(ImmWord word) {
-    framePushed_ += sizeof(word.value);
-    return pushWithPatch(word);
-  }
-  CodeOffset PushWithPatch(ImmPtr ptr) {
-    return PushWithPatch(ImmWord(uintptr_t(ptr.value)));
-  }
-
-  uint32_t framePushed() const { return framePushed_; }
-  void adjustFrame(int32_t diff) { setFramePushed(framePushed_ + diff); }
-
-  void setFramePushed(uint32_t framePushed) { framePushed_ = framePushed; }
-
-  void freeStack(Register amount) {
-    vixl::MacroAssembler::Drop(Operand(ARMRegister(amount, 64)));
-  }
-
   // Update sp with the value of the current active stack pointer, if necessary.
   void syncStackPtr() {
     if (!GetStackPointer64().Is(vixl::sp)) {
@@ -487,9 +437,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     return scratch;
   }
 
-  inline void ensureDouble(const ValueOperand& source, FloatRegister dest,
-                           Label* failure);
-
   void emitSet(Condition cond, Register dest) {
     Cset(ARMRegister(dest, 64), cond);
   }
@@ -575,22 +522,39 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Fcvt(ARMFPRegister(dest, 32), ARMFPRegister(src, 64));
   }
 
+  void convertDoubleToFloat16(FloatRegister src, FloatRegister dest) {
+    Fcvt(ARMFPRegister(dest, 16), ARMFPRegister(src, 64));
+  }
+  void convertFloat16ToDouble(FloatRegister src, FloatRegister dest) {
+    Fcvt(ARMFPRegister(dest, 64), ARMFPRegister(src, 16));
+  }
+  void convertFloat32ToFloat16(FloatRegister src, FloatRegister dest) {
+    Fcvt(ARMFPRegister(dest, 16), ARMFPRegister(src, 32));
+  }
+  void convertFloat16ToFloat32(FloatRegister src, FloatRegister dest) {
+    Fcvt(ARMFPRegister(dest, 32), ARMFPRegister(src, 16));
+  }
+  void convertInt32ToFloat16(Register src, FloatRegister dest) {
+    // Direct "32-bit to half-precision" move requires (FEAT_FP16), so we
+    // instead use a "32-bit to single-precision" move.
+    convertInt32ToFloat32(src, dest);
+    convertFloat32ToFloat16(dest, dest);
+  }
+
   using vixl::MacroAssembler::B;
 
+  bool hasFjcvtzs() const {
+    return CPUHas(vixl::CPUFeatures::kFP, vixl::CPUFeatures::kJSCVT);
+  }
+
   void convertDoubleToInt32(FloatRegister src, Register dest, Label* fail,
                             bool negativeZeroCheck = true) {
     ARMFPRegister fsrc64(src, 64);
     ARMRegister dest32(dest, 32);
 
     // ARMv8.3 chips support the FJCVTZS instruction, which handles exactly this
-    // logic.  But the simulator does not implement it.
-#if defined(JS_SIMULATOR_ARM64)
-    const bool fjscvt = false;
-#else
-    const bool fjscvt =
-        CPUHas(vixl::CPUFeatures::kFP, vixl::CPUFeatures::kJSCVT);
-#endif
-    if (fjscvt) {
+    // logic.
+    if (hasFjcvtzs()) {
       // Convert double to integer, rounding toward zero.
       // The Z-flag is set iff the conversion is exact. -0 unsets the Z-flag.
       Fjcvtzs(dest32, fsrc64);
@@ -656,9 +620,9 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       Cbnz(dest32, &nonzero);
       // dest32 is 0, fail if fsrc64 is negative.
       Fmov(dest32, fsrc);
-      Cmp(dest32, xzr);
+      Cmp(dest32, wzr);
       B(fail, Assembler::Signed);
-      Mov(dest32, xzr);
+      Mov(dest32, wzr);
       bind(&nonzero);
     }
     Uxtw(dest64, dest64);
@@ -693,6 +657,29 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     }
   }
 
+  void truncateFloat32ModUint32(FloatRegister src, Register dest) {
+    vixl::UseScratchRegisterScope temps(this);
+    const ARMRegister scratch64 = temps.AcquireX();
+
+    ARMFPRegister src32(src, 32);
+    ARMRegister dest64(dest, 64);
+
+    MOZ_ASSERT(!scratch64.Is(dest64));
+
+    // Convert scalar to signed 64-bit fixed-point, rounding toward zero.
+    // In the case of overflow, the output is saturated.
+    // In the case of NaN and -0, the output is zero.
+    Fcvtzs(dest64, src32);
+
+    // Zero if the result is saturated, i.e. it's either INT64_MIN or INT64_MAX.
+    Add(scratch64, dest64, Operand(0x7fff'ffff'ffff'ffff));
+    Cmn(scratch64, 3);
+    Csel(dest64, dest64, vixl::xzr, Assembler::BelowOrEqual);
+
+    // Clear upper 32 bits.
+    Uxtw(dest64, dest64);
+  }
+
   void jump(Label* label) { B(label); }
   void jump(JitCode* code) { branch(code); }
   void jump(ImmPtr ptr) {
@@ -1107,27 +1094,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Add(scratch64, base, Operand(index, vixl::LSL, unsigned(src.scale)));
     return Ldr(ARMFPRegister(dest, 64), MemOperand(scratch64, src.offset));
   }
-  void loadFloatAsDouble(const Address& addr, FloatRegister dest) {
-    Ldr(ARMFPRegister(dest, 32), toMemOperand(addr));
-    fcvt(ARMFPRegister(dest, 64), ARMFPRegister(dest, 32));
-  }
-  void loadFloatAsDouble(const BaseIndex& src, FloatRegister dest) {
-    ARMRegister base = toARMRegister(src.base, 64);
-    ARMRegister index(src.index, 64);
-    if (src.offset == 0) {
-      Ldr(ARMFPRegister(dest, 32),
-          MemOperand(base, index, vixl::LSL, unsigned(src.scale)));
-    } else {
-      vixl::UseScratchRegisterScope temps(this);
-      const ARMRegister scratch64 = temps.AcquireX();
-      MOZ_ASSERT(scratch64.asUnsized() != src.base);
-      MOZ_ASSERT(scratch64.asUnsized() != src.index);
-
-      Add(scratch64, base, Operand(index, vixl::LSL, unsigned(src.scale)));
-      Ldr(ARMFPRegister(dest, 32), MemOperand(scratch64, src.offset));
-    }
-    fcvt(ARMFPRegister(dest, 64), ARMFPRegister(dest, 32));
-  }
 
   FaultingCodeOffset loadFloat32(const Address& addr, FloatRegister dest) {
     return Ldr(ARMFPRegister(dest, 32), toMemOperand(addr));
@@ -1149,6 +1115,29 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     }
   }
 
+  FaultingCodeOffset loadFloat16(const Address& addr, FloatRegister dest,
+                                 Register) {
+    return Ldr(ARMFPRegister(dest, 16), toMemOperand(addr));
+  }
+
+  FaultingCodeOffset loadFloat16(const BaseIndex& src, FloatRegister dest,
+                                 Register) {
+    ARMRegister base = toARMRegister(src.base, 64);
+    ARMRegister index(src.index, 64);
+    if (src.offset == 0) {
+      return Ldr(ARMFPRegister(dest, 16),
+                 MemOperand(base, index, vixl::LSL, unsigned(src.scale)));
+    } else {
+      vixl::UseScratchRegisterScope temps(this);
+      const ARMRegister scratch64 = temps.AcquireX();
+      MOZ_ASSERT(scratch64.asUnsized() != src.base);
+      MOZ_ASSERT(scratch64.asUnsized() != src.index);
+
+      Add(scratch64, base, Operand(index, vixl::LSL, unsigned(src.scale)));
+      return Ldr(ARMFPRegister(dest, 16), MemOperand(scratch64, src.offset));
+    }
+  }
+
   void moveDouble(FloatRegister src, FloatRegister dest) {
     fmov(ARMFPRegister(dest, 64), ARMFPRegister(src, 64));
   }
@@ -1162,9 +1151,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void moveFloat32(FloatRegister src, FloatRegister dest) {
     fmov(ARMFPRegister(dest, 32), ARMFPRegister(src, 32));
   }
-  void moveFloatAsDouble(Register src, FloatRegister dest) {
-    MOZ_CRASH("moveFloatAsDouble");
-  }
 
   void moveSimd128(FloatRegister src, FloatRegister dest) {
     fmov(ARMFPRegister(dest, 128), ARMFPRegister(src, 128));
@@ -1294,6 +1280,9 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void negs32(Register reg) {
     Negs(ARMRegister(reg, 32), Operand(ARMRegister(reg, 32)));
   }
+  void negs64(Register reg) {
+    Negs(ARMRegister(reg, 64), Operand(ARMRegister(reg, 64)));
+  }
 
   void ret() {
     pop(lr);
@@ -1329,13 +1318,11 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     addPendingJump(loc, ImmPtr(target->raw()), RelocationKind::JITCODE);
   }
 
-  void compareDouble(DoubleCondition cond, FloatRegister lhs,
-                     FloatRegister rhs) {
+  void compareDouble(FloatRegister lhs, FloatRegister rhs) {
     Fcmp(ARMFPRegister(lhs, 64), ARMFPRegister(rhs, 64));
   }
 
-  void compareFloat(DoubleCondition cond, FloatRegister lhs,
-                    FloatRegister rhs) {
+  void compareFloat(FloatRegister lhs, FloatRegister rhs) {
     Fcmp(ARMFPRegister(lhs, 32), ARMFPRegister(rhs, 32));
   }
 
@@ -1352,14 +1339,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void rightShiftInt64x2(FloatRegister lhs, Register rhs, FloatRegister dest,
                          bool isUnsigned);
 
-  void branchNegativeZero(FloatRegister reg, Register scratch, Label* label) {
-    MOZ_CRASH("branchNegativeZero");
-  }
-  void branchNegativeZeroFloat32(FloatRegister reg, Register scratch,
-                                 Label* label) {
-    MOZ_CRASH("branchNegativeZeroFloat32");
-  }
-
   void boxDouble(FloatRegister src, const ValueOperand& dest, FloatRegister) {
     Fmov(ARMRegister(dest.valueReg(), 64), ARMFPRegister(src, 64));
   }
@@ -1509,20 +1488,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void unboxBigInt(const Address& src, Register dest) {
     unboxNonDouble(src, dest, JSVAL_TYPE_BIGINT);
   }
-  // These two functions use the low 32-bits of the full value register.
-  void boolValueToDouble(const ValueOperand& operand, FloatRegister dest) {
-    convertInt32ToDouble(operand.valueReg(), dest);
-  }
-  void int32ValueToDouble(const ValueOperand& operand, FloatRegister dest) {
-    convertInt32ToDouble(operand.valueReg(), dest);
-  }
-
-  void boolValueToFloat32(const ValueOperand& operand, FloatRegister dest) {
-    convertInt32ToFloat32(operand.valueReg(), dest);
-  }
-  void int32ValueToFloat32(const ValueOperand& operand, FloatRegister dest) {
-    convertInt32ToFloat32(operand.valueReg(), dest);
-  }
 
   void loadConstantDouble(double d, FloatRegister dest) {
     ARMFPRegister r(dest, 64);
@@ -2035,8 +2000,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
  public:
-  void handleFailureWithHandlerTail(Label* profilerExitTail,
-                                    Label* bailoutTail);
+  void handleFailureWithHandlerTail(Label* profilerExitTail, Label* bailoutTail,
+                                    uint32_t* returnValueCheckOffset);
 
   void profilerEnterFrame(Register framePtr, Register scratch);
   void profilerExitFrame();
@@ -2226,7 +2191,7 @@ inline void MacroAssemblerCompat::splitTagForTest(const ValueOperand& value,
   splitSignExtTag(value, tag);
 }
 
-typedef MacroAssemblerCompat MacroAssemblerSpecific;
+using MacroAssemblerSpecific = MacroAssemblerCompat;
 
 }  // namespace jit
 }  // namespace js
