@@ -409,6 +409,10 @@ class Simulator {
   void setCallResultDouble(double result);
   void setCallResultFloat(float result);
   void setCallResult(int64_t res);
+#  ifdef XP_DARWIN
+  // add a dedicated setCallResult for intptr_t on Darwin
+  void setCallResult(intptr_t res);
+#  endif
   void setCallResult(__int128 res);
 
   void callInternal(uint8_t* entry);
