@@ -18,6 +18,7 @@
 #include "jit/JitFrames.h"
 #include "jit/JitSpewer.h"
 #include "jit/MacroAssembler.h"
+#include "jit/MIR-wasm.h"
 #include "jit/MIR.h"
 #include "jit/MIRGenerator.h"
 #include "jit/SafepointIndex.h"
@@ -47,10 +48,12 @@ MacroAssembler& CodeGeneratorShared::ensureMasm(MacroAssembler* masmArg,
 }
 
 CodeGeneratorShared::CodeGeneratorShared(MIRGenerator* gen, LIRGraph* graph,
-                                         MacroAssembler* masmArg)
+                                         MacroAssembler* masmArg,
+                                         const wasm::CodeMetadata* wasmCodeMeta)
     : masm(ensureMasm(masmArg, gen->alloc(), gen->realm)),
       gen(gen),
       graph(*graph),
+      wasmCodeMeta_(wasmCodeMeta),
       current(nullptr),
       recovers_(),
 #ifdef DEBUG
@@ -61,6 +64,7 @@ CodeGeneratorShared::CodeGeneratorShared(MIRGenerator* gen, LIRGraph* graph,
                   (gen->outerInfo().nargs() + 1) * sizeof(Value)),
       returnLabel_(),
       inboundStackArgBytes_(0),
+      safepointIndices_(gen->alloc()),
       nativeToBytecodeMap_(nullptr),
       nativeToBytecodeMapSize_(0),
       nativeToBytecodeTableOffset_(0),
@@ -98,7 +102,6 @@ CodeGeneratorShared::CodeGeneratorShared(MIRGenerator* gen, LIRGraph* graph,
       // argument stack depth separately.
       MOZ_ASSERT(graph->argumentSlotCount() == 0);
 
-#ifdef ENABLE_WASM_TAIL_CALLS
       // An MWasmCall does not align the stack pointer at calls sites but
       // instead relies on the a priori stack adjustment. We need to insert
       // padding so that pushing the callee's frame maintains frame alignment.
@@ -113,15 +116,6 @@ CodeGeneratorShared::CodeGeneratorShared(MIRGenerator* gen, LIRGraph* graph,
 
       // Add the callee frame padding and stack args to frameDepth.
       frameDepth_ += calleeFramePadding + stackArgsWithPadding;
-#else
-      frameDepth_ += gen->wasmMaxStackArgBytes();
-
-      // An MWasmCall does not align the stack pointer at calls sites but
-      // instead relies on the a priori stack adjustment. This must be the
-      // last adjustment of frameDepth_.
-      frameDepth_ += ComputeByteAlignment(sizeof(wasm::Frame) + frameDepth_,
-                                          WasmStackAlignment);
-#endif
     }
 
 #ifdef JS_CODEGEN_ARM64
@@ -203,11 +197,15 @@ bool CodeGeneratorShared::generateOutOfLineCode() {
   // instead of the block corresponding to the OOL path.
   current = nullptr;
 
-  for (size_t i = 0; i < outOfLineCode_.length(); i++) {
-    // Add native => bytecode mapping entries for OOL sites.
+  for (OutOfLineCode* ool : outOfLineCode_) {
+    if (gen->shouldCancel("Generate Code (OOL code loop)")) {
+      return false;
+    }
+
+    // Add native => bytecode mapping entries for OOL->sites.
     // Not enabled on wasm yet since it doesn't contain bytecode mappings.
     if (!gen->compilingWasm()) {
-      if (!addNativeToBytecodeEntry(outOfLineCode_[i]->bytecodeSite())) {
+      if (!addNativeToBytecodeEntry(ool->bytecodeSite())) {
         return false;
       }
     }
@@ -218,10 +216,10 @@ bool CodeGeneratorShared::generateOutOfLineCode() {
 
     JitSpew(JitSpew_Codegen, "# Emitting out of line code");
 
-    masm.setFramePushed(outOfLineCode_[i]->framePushed());
-    outOfLineCode_[i]->bind(&masm);
+    masm.setFramePushed(ool->framePushed());
+    ool->bind(&masm);
 
-    outOfLineCode_[i]->generate(this);
+    ool->generate(this);
   }
 
   return !masm.oom();
@@ -238,7 +236,7 @@ void CodeGeneratorShared::addOutOfLineCode(OutOfLineCode* code,
   MOZ_ASSERT_IF(!gen->compilingWasm(), site->script()->containsPC(site->pc()));
   code->setFramePushed(masm.framePushed());
   code->setBytecodeSite(site);
-  masm.propagateOOM(outOfLineCode_.append(code));
+  outOfLineCode_.pushBack(code);
 }
 
 bool CodeGeneratorShared::addNativeToBytecodeEntry(const BytecodeSite* site) {
@@ -444,8 +442,11 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
 
       JSValueType valueType = ValueTypeFromMIRType(type);
 
-      MOZ_DIAGNOSTIC_ASSERT(payload->isMemory() || payload->isRegister());
+      MOZ_DIAGNOSTIC_ASSERT(payload->isMemory() || payload->isAnyRegister());
       if (payload->isMemory()) {
+        MOZ_ASSERT_IF(payload->isStackSlot(),
+                      payload->toStackSlot()->width() ==
+                          LStackSlot::width(LDefinition::TypeFrom(type)));
         alloc = RValueAllocation::Typed(valueType, ToStackIndex(payload));
       } else if (payload->isGeneralReg()) {
         alloc = RValueAllocation::Typed(valueType, ToRegister(payload));
@@ -456,8 +457,7 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
       }
       break;
     }
-    case MIRType::Float32:
-    case MIRType::Simd128: {
+    case MIRType::Float32: {
       LAllocation* payload = snapshot->payloadOfSlot(*allocIndex);
       if (payload->isConstant()) {
         MConstant* constant = mir->toConstant();
@@ -468,11 +468,56 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
         break;
       }
 
-      MOZ_ASSERT(payload->isMemory() || payload->isFloatReg());
       if (payload->isFloatReg()) {
-        alloc = RValueAllocation::AnyFloat(ToFloatRegister(payload));
+        alloc = RValueAllocation::Float32(ToFloatRegister(payload));
+      } else if (payload->isMemory()) {
+        MOZ_ASSERT_IF(payload->isStackSlot(),
+                      payload->toStackSlot()->width() ==
+                          LStackSlot::width(LDefinition::TypeFrom(type)));
+        alloc = RValueAllocation::Float32(ToStackIndex(payload));
       } else {
-        alloc = RValueAllocation::AnyFloat(ToStackIndex(payload));
+        MOZ_CRASH("Unexpected payload type.");
+      }
+      break;
+    }
+    case MIRType::IntPtr: {
+      LAllocation* payload = snapshot->payloadOfSlot(*allocIndex);
+      if (payload->isConstant()) {
+        intptr_t constant = mir->toConstant()->toIntPtr();
+#if !defined(JS_64BIT)
+        uint32_t index;
+        masm.propagateOOM(
+            graph.addConstantToPool(Int32Value(constant), &index));
+
+        alloc = RValueAllocation::IntPtrConstant(index);
+#else
+        uint32_t lowIndex;
+        masm.propagateOOM(
+            graph.addConstantToPool(Int32Value(constant), &lowIndex));
+
+        uint32_t highIndex;
+        masm.propagateOOM(
+            graph.addConstantToPool(Int32Value(constant >> 32), &highIndex));
+
+        alloc = RValueAllocation::IntPtrConstant(lowIndex, highIndex);
+#endif
+        break;
+      }
+
+      if (payload->isGeneralReg()) {
+        alloc = RValueAllocation::IntPtr(ToRegister(payload));
+      } else if (payload->isStackSlot()) {
+        LStackSlot::Width width = payload->toStackSlot()->width();
+        MOZ_ASSERT(width == LStackSlot::width(LDefinition::GENERAL) ||
+                   width == LStackSlot::width(LDefinition::INT32));
+
+        if (width == LStackSlot::width(LDefinition::GENERAL)) {
+          alloc = RValueAllocation::IntPtr(ToStackIndex(payload));
+        } else {
+          alloc = RValueAllocation::IntPtrInt32(ToStackIndex(payload));
+        }
+      } else {
+        MOZ_CRASH("Unexpected payload type.");
       }
       break;
     }
@@ -500,13 +545,75 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
       alloc = RValueAllocation::ConstantPool(index);
       break;
     }
-    default: {
-      MOZ_ASSERT(mir->type() == MIRType::Value);
+    case MIRType::Int64: {
+      LAllocation* payload = snapshot->payloadOfSlot(*allocIndex);
+      if (payload->isConstant()) {
+        int64_t constant = mir->toConstant()->toInt64();
+
+        uint32_t lowIndex;
+        masm.propagateOOM(
+            graph.addConstantToPool(Int32Value(constant), &lowIndex));
+
+        uint32_t highIndex;
+        masm.propagateOOM(
+            graph.addConstantToPool(Int32Value(constant >> 32), &highIndex));
+
+        alloc = RValueAllocation::Int64Constant(lowIndex, highIndex);
+        break;
+      }
+
+#ifdef JS_NUNBOX32
+      LAllocation* type = snapshot->typeOfSlot(*allocIndex);
+
+      MOZ_ASSERT_IF(payload->isStackSlot(),
+                    payload->toStackSlot()->width() ==
+                        LStackSlot::width(LDefinition::GENERAL));
+      MOZ_ASSERT_IF(type->isStackSlot(),
+                    type->toStackSlot()->width() ==
+                        LStackSlot::width(LDefinition::GENERAL));
+
+      if (payload->isGeneralReg()) {
+        if (type->isGeneralReg()) {
+          alloc =
+              RValueAllocation::Int64(ToRegister(type), ToRegister(payload));
+        } else if (type->isStackSlot()) {
+          alloc =
+              RValueAllocation::Int64(ToStackIndex(type), ToRegister(payload));
+        } else {
+          MOZ_CRASH("Unexpected payload type.");
+        }
+      } else if (payload->isStackSlot()) {
+        if (type->isGeneralReg()) {
+          alloc =
+              RValueAllocation::Int64(ToRegister(type), ToStackIndex(payload));
+        } else if (type->isStackSlot()) {
+          alloc = RValueAllocation::Int64(ToStackIndex(type),
+                                          ToStackIndex(payload));
+        } else {
+          MOZ_CRASH("Unexpected payload type.");
+        }
+      } else {
+        MOZ_CRASH("Unexpected payload type.");
+      }
+#elif JS_PUNBOX64
+      if (payload->isGeneralReg()) {
+        alloc = RValueAllocation::Int64(ToRegister(payload));
+      } else if (payload->isStackSlot()) {
+        MOZ_ASSERT(payload->toStackSlot()->width() ==
+                   LStackSlot::width(LDefinition::GENERAL));
+        alloc = RValueAllocation::Int64(ToStackIndex(payload));
+      } else {
+        MOZ_CRASH("Unexpected payload type.");
+      }
+#endif
+      break;
+    }
+    case MIRType::Value: {
       LAllocation* payload = snapshot->payloadOfSlot(*allocIndex);
 #ifdef JS_NUNBOX32
       LAllocation* type = snapshot->typeOfSlot(*allocIndex);
-      if (type->isRegister()) {
-        if (payload->isRegister()) {
+      if (type->isGeneralReg()) {
+        if (payload->isGeneralReg()) {
           alloc =
               RValueAllocation::Untyped(ToRegister(type), ToRegister(payload));
         } else {
@@ -514,7 +621,7 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
                                             ToStackIndex(payload));
         }
       } else {
-        if (payload->isRegister()) {
+        if (payload->isGeneralReg()) {
           alloc = RValueAllocation::Untyped(ToStackIndex(type),
                                             ToRegister(payload));
         } else {
@@ -523,7 +630,7 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
         }
       }
 #elif JS_PUNBOX64
-      if (payload->isRegister()) {
+      if (payload->isGeneralReg()) {
         alloc = RValueAllocation::Untyped(ToRegister(payload));
       } else {
         alloc = RValueAllocation::Untyped(ToStackIndex(payload));
@@ -531,6 +638,8 @@ void CodeGeneratorShared::encodeAllocation(LSnapshot* snapshot,
 #endif
       break;
     }
+    default:
+      MOZ_CRASH("Unexpected MIR type");
   }
   MOZ_DIAGNOSTIC_ASSERT(alloc.valid());
 
@@ -876,18 +985,15 @@ class OutOfLineTruncateSlow : public OutOfLineCodeBase<CodeGeneratorShared> {
   Register dest_;
   bool widenFloatToDouble_;
   wasm::BytecodeOffset bytecodeOffset_;
-  bool preserveInstance_;
 
  public:
-  OutOfLineTruncateSlow(
-      FloatRegister src, Register dest, bool widenFloatToDouble = false,
-      wasm::BytecodeOffset bytecodeOffset = wasm::BytecodeOffset(),
-      bool preserveInstance = false)
+  OutOfLineTruncateSlow(FloatRegister src, Register dest,
+                        bool widenFloatToDouble,
+                        wasm::BytecodeOffset bytecodeOffset)
       : src_(src),
         dest_(dest),
         widenFloatToDouble_(widenFloatToDouble),
-        bytecodeOffset_(bytecodeOffset),
-        preserveInstance_(preserveInstance) {}
+        bytecodeOffset_(bytecodeOffset) {}
 
   void accept(CodeGeneratorShared* codegen) override {
     codegen->visitOutOfLineTruncateSlow(this);
@@ -895,17 +1001,16 @@ class OutOfLineTruncateSlow : public OutOfLineCodeBase<CodeGeneratorShared> {
   FloatRegister src() const { return src_; }
   Register dest() const { return dest_; }
   bool widenFloatToDouble() const { return widenFloatToDouble_; }
-  bool preserveInstance() const { return preserveInstance_; }
   wasm::BytecodeOffset bytecodeOffset() const { return bytecodeOffset_; }
 };
 
 OutOfLineCode* CodeGeneratorShared::oolTruncateDouble(
     FloatRegister src, Register dest, MInstruction* mir,
-    wasm::BytecodeOffset bytecodeOffset, bool preserveInstance) {
+    wasm::BytecodeOffset bytecodeOffset) {
   MOZ_ASSERT_IF(IsCompilingWasm(), bytecodeOffset.isValid());
 
-  OutOfLineTruncateSlow* ool = new (alloc()) OutOfLineTruncateSlow(
-      src, dest, /* float32 */ false, bytecodeOffset, preserveInstance);
+  OutOfLineTruncateSlow* ool = new (alloc())
+      OutOfLineTruncateSlow(src, dest, /* float32 */ false, bytecodeOffset);
   addOutOfLineCode(ool, mir);
   return ool;
 }
@@ -915,8 +1020,8 @@ void CodeGeneratorShared::emitTruncateDouble(FloatRegister src, Register dest,
   MOZ_ASSERT(mir->isTruncateToInt32() || mir->isWasmBuiltinTruncateToInt32());
   wasm::BytecodeOffset bytecodeOffset =
       mir->isTruncateToInt32()
-          ? mir->toTruncateToInt32()->bytecodeOffset()
-          : mir->toWasmBuiltinTruncateToInt32()->bytecodeOffset();
+          ? mir->toTruncateToInt32()->trapSiteDesc().bytecodeOffset
+          : mir->toWasmBuiltinTruncateToInt32()->trapSiteDesc().bytecodeOffset;
   OutOfLineCode* ool = oolTruncateDouble(src, dest, mir, bytecodeOffset);
 
   masm.branchTruncateDoubleMaybeModUint32(src, dest, ool->entry());
@@ -928,8 +1033,8 @@ void CodeGeneratorShared::emitTruncateFloat32(FloatRegister src, Register dest,
   MOZ_ASSERT(mir->isTruncateToInt32() || mir->isWasmBuiltinTruncateToInt32());
   wasm::BytecodeOffset bytecodeOffset =
       mir->isTruncateToInt32()
-          ? mir->toTruncateToInt32()->bytecodeOffset()
-          : mir->toWasmBuiltinTruncateToInt32()->bytecodeOffset();
+          ? mir->toTruncateToInt32()->trapSiteDesc().bytecodeOffset
+          : mir->toWasmBuiltinTruncateToInt32()->trapSiteDesc().bytecodeOffset;
   OutOfLineTruncateSlow* ool = new (alloc())
       OutOfLineTruncateSlow(src, dest, /* float32 */ true, bytecodeOffset);
   addOutOfLineCode(ool, mir);
@@ -993,10 +1098,10 @@ Label* CodeGeneratorShared::getJumpLabelForBranch(MBasicBlock* block) {
   return skipTrivialBlocks(block)->lir()->label();
 }
 
-// This function is not used for MIPS/MIPS64/LOONG64. They have
+// This function is not used for MIPS64/LOONG64/RISCV64. They have
 // branchToBlock.
-#if !defined(JS_CODEGEN_MIPS32) && !defined(JS_CODEGEN_MIPS64) && \
-    !defined(JS_CODEGEN_LOONG64) && !defined(JS_CODEGEN_RISCV64)
+#if !defined(JS_CODEGEN_MIPS64) && !defined(JS_CODEGEN_LOONG64) && \
+    !defined(JS_CODEGEN_RISCV64)
 void CodeGeneratorShared::jumpToBlock(MBasicBlock* mir,
                                       Assembler::Condition cond) {
   // Skip past trivial blocks.
