@@ -746,8 +746,10 @@ struct AssemblerBufferWithConstantPools
       // secondary range veneers assuming the worst case deadlines.
 
       // Total pending secondary range veneer size.
-      size_t secondaryVeneers = guardSize_ * (branchDeadlines_.size() -
-                                              branchDeadlines_.maxRangeSize());
+      size_t secondaryVeneers =
+          guardSize_ *
+          (branchDeadlines_.size() - branchDeadlines_.maxRangeSize()) *
+          InstSize;
 
       if (deadline < poolEnd + secondaryVeneers) {
         return false;
