@@ -54,6 +54,14 @@ void MacroAssembler::move32To64SignExtend(Register src, Register64 dest) {
   movslq(src, dest.reg);
 }
 
+void MacroAssembler::move8SignExtendToPtr(Register src, Register dest) {
+  movsbq(Operand(src), dest);
+}
+
+void MacroAssembler::move16SignExtendToPtr(Register src, Register dest) {
+  movswq(Operand(src), dest);
+}
+
 void MacroAssembler::move32SignExtendToPtr(Register src, Register dest) {
   movslq(src, dest);
 }
@@ -78,6 +86,13 @@ void MacroAssembler::andPtr(Register src, Register dest) { andq(src, dest); }
 
 void MacroAssembler::andPtr(Imm32 imm, Register dest) { andq(imm, dest); }
 
+void MacroAssembler::andPtr(Imm32 imm, Register src, Register dest) {
+  if (src != dest) {
+    movq(src, dest);
+  }
+  andq(imm, dest);
+}
+
 void MacroAssembler::and64(Imm64 imm, Register64 dest) {
   if (INT32_MIN <= int64_t(imm.value) && int64_t(imm.value) <= INT32_MAX) {
     andq(Imm32(imm.value), dest.reg);
@@ -112,6 +127,13 @@ void MacroAssembler::orPtr(Register src, Register dest) { orq(src, dest); }
 
 void MacroAssembler::orPtr(Imm32 imm, Register dest) { orq(imm, dest); }
 
+void MacroAssembler::orPtr(Imm32 imm, Register src, Register dest) {
+  if (src != dest) {
+    movq(src, dest);
+  }
+  orq(imm, dest);
+}
+
 void MacroAssembler::and64(Register64 src, Register64 dest) {
   andq(src.reg, dest.reg);
 }
@@ -128,6 +150,13 @@ void MacroAssembler::xorPtr(Register src, Register dest) { xorq(src, dest); }
 
 void MacroAssembler::xorPtr(Imm32 imm, Register dest) { xorq(imm, dest); }
 
+void MacroAssembler::xorPtr(Imm32 imm, Register src, Register dest) {
+  if (src != dest) {
+    movq(src, dest);
+  }
+  xorq(imm, dest);
+}
+
 void MacroAssembler::and64(const Operand& src, Register64 dest) {
   andq(src, dest.reg);
 }
@@ -324,6 +353,14 @@ void MacroAssembler::lshiftPtr(Imm32 imm, Register dest) {
   shlq(imm, dest);
 }
 
+void MacroAssembler::lshiftPtr(Imm32 imm, Register src, Register dest) {
+  MOZ_ASSERT(0 <= imm.value && imm.value < 64);
+  if (src != dest) {
+    movq(src, dest);
+  }
+  shlq(imm, dest);
+}
+
 void MacroAssembler::lshiftPtr(Register shift, Register srcDest) {
   if (Assembler::HasBMI2()) {
     shlxq(srcDest, shift, srcDest);
@@ -333,6 +370,21 @@ void MacroAssembler::lshiftPtr(Register shift, Register srcDest) {
   shlq_cl(srcDest);
 }
 
+void MacroAssembler::flexibleLshiftPtr(Register shift, Register srcDest) {
+  if (HasBMI2()) {
+    shlxq(srcDest, shift, srcDest);
+    return;
+  }
+  if (shift == rcx) {
+    shlq_cl(srcDest);
+  } else {
+    // Shift amount must be in rcx.
+    xchg(shift, rcx);
+    shlq_cl(shift == srcDest ? rcx : srcDest == rcx ? shift : srcDest);
+    xchg(shift, rcx);
+  }
+}
+
 void MacroAssembler::lshift64(Imm32 imm, Register64 dest) {
   MOZ_ASSERT(0 <= imm.value && imm.value < 64);
   lshiftPtr(imm, dest.reg);
@@ -352,6 +404,14 @@ void MacroAssembler::rshiftPtr(Imm32 imm, Register dest) {
   shrq(imm, dest);
 }
 
+void MacroAssembler::rshiftPtr(Imm32 imm, Register src, Register dest) {
+  MOZ_ASSERT(0 <= imm.value && imm.value < 64);
+  if (src != dest) {
+    movq(src, dest);
+  }
+  shrq(imm, dest);
+}
+
 void MacroAssembler::rshiftPtr(Register shift, Register srcDest) {
   if (Assembler::HasBMI2()) {
     shrxq(srcDest, shift, srcDest);
@@ -361,6 +421,21 @@ void MacroAssembler::rshiftPtr(Register shift, Register srcDest) {
   shrq_cl(srcDest);
 }
 
+void MacroAssembler::flexibleRshiftPtr(Register shift, Register srcDest) {
+  if (HasBMI2()) {
+    shrxq(srcDest, shift, srcDest);
+    return;
+  }
+  if (shift == rcx) {
+    shrq_cl(srcDest);
+  } else {
+    // Shift amount must be in rcx.
+    xchg(shift, rcx);
+    shrq_cl(shift == srcDest ? rcx : srcDest == rcx ? shift : srcDest);
+    xchg(shift, rcx);
+  }
+}
+
 void MacroAssembler::rshift64(Imm32 imm, Register64 dest) {
   rshiftPtr(imm, dest.reg);
 }
@@ -379,6 +454,40 @@ void MacroAssembler::rshiftPtrArithmetic(Imm32 imm, Register dest) {
   sarq(imm, dest);
 }
 
+void MacroAssembler::rshiftPtrArithmetic(Imm32 imm, Register src,
+                                         Register dest) {
+  MOZ_ASSERT(0 <= imm.value && imm.value < 64);
+  if (src != dest) {
+    movq(src, dest);
+  }
+  sarq(imm, dest);
+}
+
+void MacroAssembler::rshiftPtrArithmetic(Register shift, Register srcDest) {
+  if (Assembler::HasBMI2()) {
+    sarxq(srcDest, shift, srcDest);
+    return;
+  }
+  MOZ_ASSERT(shift == rcx);
+  sarq_cl(srcDest);
+}
+
+void MacroAssembler::flexibleRshiftPtrArithmetic(Register shift,
+                                                 Register srcDest) {
+  if (HasBMI2()) {
+    sarxq(srcDest, shift, srcDest);
+    return;
+  }
+  if (shift == rcx) {
+    sarq_cl(srcDest);
+  } else {
+    // Shift amount must be in rcx.
+    xchg(shift, rcx);
+    sarq_cl(shift == srcDest ? rcx : srcDest == rcx ? shift : srcDest);
+    xchg(shift, rcx);
+  }
+}
+
 void MacroAssembler::rshift64Arithmetic(Imm32 imm, Register64 dest) {
   MOZ_ASSERT(0 <= imm.value && imm.value < 64);
   rshiftPtrArithmetic(imm, dest.reg);
@@ -451,6 +560,21 @@ void MacroAssembler::rotateRight64(Imm32 count, Register64 src, Register64 dest,
 // ===============================================================
 // Condition functions
 
+void MacroAssembler::cmp64Set(Condition cond, Register64 lhs, Register64 rhs,
+                              Register dest) {
+  cmpPtrSet(cond, lhs.reg, rhs.reg, dest);
+}
+
+void MacroAssembler::cmp64Set(Condition cond, Register64 lhs, Imm64 rhs,
+                              Register dest) {
+  cmpPtrSet(cond, lhs.reg, ImmWord(static_cast<uintptr_t>(rhs.value)), dest);
+}
+
+void MacroAssembler::cmp64Set(Condition cond, Address lhs, Register64 rhs,
+                              Register dest) {
+  cmpPtrSet(cond, lhs, rhs.reg, dest);
+}
+
 void MacroAssembler::cmp64Set(Condition cond, Address lhs, Imm64 rhs,
                               Register dest) {
   cmpPtrSet(cond, lhs, ImmWord(static_cast<uintptr_t>(rhs.value)), dest);
@@ -458,37 +582,38 @@ void MacroAssembler::cmp64Set(Condition cond, Address lhs, Imm64 rhs,
 
 template <typename T1, typename T2>
 void MacroAssembler::cmpPtrSet(Condition cond, T1 lhs, T2 rhs, Register dest) {
+  bool destIsZero = maybeEmitSetZeroByteRegister(lhs, rhs, dest);
   cmpPtr(lhs, rhs);
-  emitSet(cond, dest);
+  emitSet(cond, dest, destIsZero);
 }
 
 // ===============================================================
 // Bit counting functions
 
-void MacroAssembler::clz64(Register64 src, Register dest) {
+void MacroAssembler::clz64(Register64 src, Register64 dest) {
   if (AssemblerX86Shared::HasLZCNT()) {
-    lzcntq(src.reg, dest);
+    lzcntq(src.reg, dest.reg);
     return;
   }
 
   Label nonzero;
-  bsrq(src.reg, dest);
+  bsrq(src.reg, dest.reg);
   j(Assembler::NonZero, &nonzero);
-  movq(ImmWord(0x7F), dest);
+  movq(ImmWord(0x7F), dest.reg);
   bind(&nonzero);
-  xorq(Imm32(0x3F), dest);
+  xorq(Imm32(0x3F), dest.reg);
 }
 
-void MacroAssembler::ctz64(Register64 src, Register dest) {
+void MacroAssembler::ctz64(Register64 src, Register64 dest) {
   if (AssemblerX86Shared::HasBMI1()) {
-    tzcntq(src.reg, dest);
+    tzcntq(src.reg, dest.reg);
     return;
   }
 
   Label nonzero;
-  bsfq(src.reg, dest);
+  bsfq(src.reg, dest.reg);
   j(Assembler::NonZero, &nonzero);
-  movq(ImmWord(64), dest);
+  movq(ImmWord(64), dest.reg);
   bind(&nonzero);
 }
 
@@ -498,11 +623,12 @@ void MacroAssembler::popcnt64(Register64 src64, Register64 dest64,
   Register dest = dest64.reg;
 
   if (AssemblerX86Shared::HasPOPCNT()) {
-    MOZ_ASSERT(tmp == InvalidReg);
     popcntq(src, dest);
     return;
   }
 
+  MOZ_ASSERT(tmp != InvalidReg);
+
   if (src != dest) {
     movq(src, dest);
   }
@@ -606,19 +732,37 @@ void MacroAssembler::branch64(Condition cond, Register64 lhs, Register64 rhs,
 }
 
 void MacroAssembler::branch64(Condition cond, const Address& lhs, Imm64 val,
-                              Label* label) {
-  MOZ_ASSERT(cond == Assembler::NotEqual || cond == Assembler::Equal,
+                              Label* success, Label* fail) {
+  MOZ_ASSERT(cond == Assembler::NotEqual || cond == Assembler::Equal ||
+                 cond == Assembler::LessThan ||
+                 cond == Assembler::LessThanOrEqual ||
+                 cond == Assembler::GreaterThan ||
+                 cond == Assembler::GreaterThanOrEqual ||
+                 cond == Assembler::Below || cond == Assembler::BelowOrEqual ||
+                 cond == Assembler::Above || cond == Assembler::AboveOrEqual,
              "other condition codes not supported");
 
-  branchPtr(cond, lhs, ImmWord(val.value), label);
+  branchPtr(cond, lhs, ImmWord(val.value), success);
+  if (fail) {
+    jump(fail);
+  }
 }
 
 void MacroAssembler::branch64(Condition cond, const Address& lhs,
-                              Register64 rhs, Label* label) {
-  MOZ_ASSERT(cond == Assembler::NotEqual || cond == Assembler::Equal,
+                              Register64 rhs, Label* success, Label* fail) {
+  MOZ_ASSERT(cond == Assembler::NotEqual || cond == Assembler::Equal ||
+                 cond == Assembler::LessThan ||
+                 cond == Assembler::LessThanOrEqual ||
+                 cond == Assembler::GreaterThan ||
+                 cond == Assembler::GreaterThanOrEqual ||
+                 cond == Assembler::Below || cond == Assembler::BelowOrEqual ||
+                 cond == Assembler::Above || cond == Assembler::AboveOrEqual,
              "other condition codes not supported");
 
-  branchPtr(cond, lhs, rhs.reg, label);
+  branchPtr(cond, lhs, rhs.reg, success);
+  if (fail) {
+    jump(fail);
+  }
 }
 
 void MacroAssembler::branch64(Condition cond, const Address& lhs,
@@ -681,8 +825,8 @@ void MacroAssembler::branchTruncateFloat32ToPtr(FloatRegister src,
 void MacroAssembler::branchTruncateFloat32MaybeModUint32(FloatRegister src,
                                                          Register dest,
                                                          Label* fail) {
-  branchTruncateFloat32ToPtr(src, dest, fail);
-  movl(dest, dest);  // Zero upper 32-bits.
+  // Infallible operation on x64.
+  truncateFloat32ModUint32(src, dest);
 }
 
 void MacroAssembler::branchTruncateFloat32ToInt32(FloatRegister src,
@@ -691,7 +835,7 @@ void MacroAssembler::branchTruncateFloat32ToInt32(FloatRegister src,
 
   // Check that the result is in the int32_t range.
   ScratchRegisterScope scratch(*this);
-  move32To64SignExtend(dest, Register64(scratch));
+  move32SignExtendToPtr(dest, scratch);
   cmpPtr(dest, scratch);
   j(Assembler::NotEqual, fail);
 
@@ -722,13 +866,21 @@ void MacroAssembler::branchTruncateDoubleToInt32(FloatRegister src,
 
   // Check that the result is in the int32_t range.
   ScratchRegisterScope scratch(*this);
-  move32To64SignExtend(dest, Register64(scratch));
+  move32SignExtendToPtr(dest, scratch);
   cmpPtr(dest, scratch);
   j(Assembler::NotEqual, fail);
 
   movl(dest, dest);  // Zero upper 32-bits.
 }
 
+void MacroAssembler::branchInt64NotInPtrRange(Register64 src, Label* label) {
+  // No-op on 64-bit platforms.
+}
+
+void MacroAssembler::branchUInt64NotInPtrRange(Register64 src, Label* label) {
+  branchTest64(Assembler::Signed, src, src, label);
+}
+
 void MacroAssembler::branchTest32(Condition cond, const AbsoluteAddress& lhs,
                                   Imm32 rhs, Label* label) {
   if (X86Encoding::IsAddressImmediate(lhs.addr)) {
@@ -741,10 +893,27 @@ void MacroAssembler::branchTest32(Condition cond, const AbsoluteAddress& lhs,
   j(cond, label);
 }
 
-template <class L>
+void MacroAssembler::branchTestPtr(Condition cond, Register lhs, ImmWord rhs,
+                                   Label* label) {
+  testPtr(lhs, rhs);
+  j(cond, label);
+}
+
 void MacroAssembler::branchTest64(Condition cond, Register64 lhs,
-                                  Register64 rhs, Register temp, L label) {
-  branchTestPtr(cond, lhs.reg, rhs.reg, label);
+                                  Register64 rhs, Register temp, Label* success,
+                                  Label* fail) {
+  branchTestPtr(cond, lhs.reg, rhs.reg, success);
+  if (fail) {
+    jump(fail);
+  }
+}
+
+void MacroAssembler::branchTest64(Condition cond, Register64 lhs, Imm64 rhs,
+                                  Label* success, Label* fail) {
+  branchTestPtr(cond, lhs.reg, ImmWord(rhs.value), success);
+  if (fail) {
+    jump(fail);
+  }
 }
 
 void MacroAssembler::branchTestBooleanTruthy(bool truthy,
@@ -761,7 +930,8 @@ void MacroAssembler::branchTestMagic(Condition cond, const Address& valaddr,
   j(cond, label);
 }
 
-void MacroAssembler::branchTestValue(Condition cond, const BaseIndex& lhs,
+template <typename T>
+void MacroAssembler::branchTestValue(Condition cond, const T& lhs,
                                      const ValueOperand& rhs, Label* label) {
   MOZ_ASSERT(cond == Assembler::Equal || cond == Assembler::NotEqual);
   branchPtr(cond, lhs, rhs.valueReg(), label);
@@ -771,6 +941,12 @@ void MacroAssembler::branchToComputedAddress(const BaseIndex& address) {
   jmp(Operand(address));
 }
 
+void MacroAssembler::cmpPtrMovePtr(Condition cond, Register lhs, Imm32 rhs,
+                                   Register src, Register dest) {
+  cmpPtr(lhs, rhs);
+  cmovCCq(cond, src, dest);
+}
+
 void MacroAssembler::cmpPtrMovePtr(Condition cond, Register lhs, Register rhs,
                                    Register src, Register dest) {
   cmpPtr(lhs, rhs);
@@ -804,6 +980,13 @@ void MacroAssembler::test32LoadPtr(Condition cond, const Address& addr,
   cmovCCq(cond, Operand(src), dest);
 }
 
+void MacroAssembler::test32MovePtr(Condition cond, Register operand, Imm32 mask,
+                                   Register src, Register dest) {
+  MOZ_ASSERT(cond == Assembler::Zero || cond == Assembler::NonZero);
+  test32(operand, mask);
+  cmovCCq(cond, Operand(src), dest);
+}
+
 void MacroAssembler::test32MovePtr(Condition cond, const Address& addr,
                                    Imm32 mask, Register src, Register dest) {
   MOZ_ASSERT(cond == Assembler::Zero || cond == Assembler::NonZero);
@@ -1013,7 +1196,7 @@ void MacroAssemblerX64::fallibleUnboxPtrImpl(const Operand& src, Register dest,
   //
   // Note: src and dest can be the same register.
   ScratchRegisterScope scratch(asMasm());
-  mov(ImmWord(JSVAL_TYPE_TO_SHIFTED_TAG(type)), scratch);
+  mov(ImmShiftedTag(type), scratch);
   xorq(src, scratch);
   mov(scratch, dest);
   shrq(Imm32(JSVAL_TAG_SHIFT), scratch);
@@ -1068,31 +1251,6 @@ void MacroAssemblerX64::loadInt32OrDouble(const T& src, FloatRegister dest) {
   bind(&end);
 }
 
-// If source is a double, load it into dest. If source is int32,
-// convert it to double. Else, branch to failure.
-void MacroAssemblerX64::ensureDouble(const ValueOperand& source,
-                                     FloatRegister dest, Label* failure) {
-  Label isDouble, done;
-  {
-    ScratchTagScope tag(asMasm(), source);
-    splitTagForTest(source, tag);
-    asMasm().branchTestDouble(Assembler::Equal, tag, &isDouble);
-    asMasm().branchTestInt32(Assembler::NotEqual, tag, failure);
-  }
-
-  {
-    ScratchRegisterScope scratch(asMasm());
-    unboxInt32(source, scratch);
-    convertInt32ToDouble(scratch, dest);
-  }
-  jump(&done);
-
-  bind(&isDouble);
-  unboxDouble(source, dest);
-
-  bind(&done);
-}
-
 }  // namespace jit
 }  // namespace js
 
