@@ -17,95 +17,61 @@ namespace jit {
 
 inline void EmitBaselineTailCallVM(TrampolinePtr target, MacroAssembler& masm,
                                    uint32_t argSize) {
+#ifdef DEBUG
   ScratchRegisterScope scratch(masm);
 
-  // We an assume during this that R0 and R1 have been pushed.
-  masm.movq(FramePointer, scratch);
-  masm.addq(Imm32(BaselineFrame::FramePointerOffset), scratch);
-  masm.subq(BaselineStackReg, scratch);
-
-#ifdef DEBUG
+  // We can assume during this that R0 and R1 have been pushed.
   // Store frame size without VMFunction arguments for debug assertions.
-  masm.movq(scratch, rdx);
-  masm.subq(Imm32(argSize), rdx);
-  Address frameSizeAddr(FramePointer,
-                        BaselineFrame::reverseOffsetOfDebugFrameSize());
-  masm.store32(rdx, frameSizeAddr);
-#endif
-
-  // Push frame descriptor and perform the tail call.
-  masm.makeFrameDescriptor(scratch, FrameType::BaselineJS,
-                           ExitFrameLayout::Size());
-  masm.push(scratch);
-  masm.push(ICTailCallReg);
-  masm.jump(target);
-}
-
-inline void EmitBaselineCreateStubFrameDescriptor(MacroAssembler& masm,
-                                                  Register reg,
-                                                  uint32_t headerSize) {
-  // Compute stub frame size. We have to add two pointers: the stub reg and
-  // previous frame pointer pushed by EmitEnterStubFrame.
-  masm.movq(FramePointer, reg);
-  masm.addq(Imm32(sizeof(void*) * 2), reg);
-  masm.subq(BaselineStackReg, reg);
-
-  masm.makeFrameDescriptor(reg, FrameType::BaselineStub, headerSize);
-}
-
-inline void EmitBaselineCallVM(TrampolinePtr target, MacroAssembler& masm) {
-  ScratchRegisterScope scratch(masm);
-  EmitBaselineCreateStubFrameDescriptor(masm, scratch, ExitFrameLayout::Size());
-  masm.push(scratch);
-  masm.call(target);
-}
-
-// Size of values pushed by EmitBaselineEnterStubFrame.
-static const uint32_t STUB_FRAME_SIZE = 4 * sizeof(void*);
-static const uint32_t STUB_FRAME_SAVED_STUB_OFFSET = sizeof(void*);
-
-inline void EmitBaselineEnterStubFrame(MacroAssembler& masm, Register) {
-  ScratchRegisterScope scratch(masm);
-
-  // Compute frame size. Because the return address is still on the stack,
-  // this is:
-  //
-  //   FramePointer
-  //   + BaselineFrame::FramePointerOffset
-  //   - BaselineStackReg
-  //   - sizeof(return address)
-  //
-  // The two constants cancel each other out, so we can just calculate
-  // FramePointer - BaselineStackReg.
-
-  static_assert(
-      BaselineFrame::FramePointerOffset == sizeof(void*),
-      "FramePointerOffset must be the same as the return address size");
-
   masm.movq(FramePointer, scratch);
-  masm.subq(BaselineStackReg, scratch);
-
-#ifdef DEBUG
+  masm.subq(StackPointer, scratch);
+  masm.subq(Imm32(argSize), scratch);
   Address frameSizeAddr(FramePointer,
                         BaselineFrame::reverseOffsetOfDebugFrameSize());
   masm.store32(scratch, frameSizeAddr);
 #endif
 
-  // Note: when making changes here,  don't forget to update STUB_FRAME_SIZE
-  // if needed.
+  // Push frame descriptor and perform the tail call.
+  masm.pushFrameDescriptor(FrameType::BaselineJS);
+  masm.push(ICTailCallReg);
+  masm.jump(target);
+}
+
+inline void EmitBaselineCallVM(TrampolinePtr target, MacroAssembler& masm) {
+  masm.pushFrameDescriptor(FrameType::BaselineStub);
+  masm.call(target);
+}
+
+inline void EmitBaselineEnterStubFrame(MacroAssembler& masm, Register) {
+#ifdef DEBUG
+  // Compute frame size. Because the return address is still on the stack,
+  // this is:
+  //
+  //   FramePointer
+  //   - StackPointer
+  //   - sizeof(return address)
+
+  ScratchRegisterScope scratch(masm);
+  masm.movq(FramePointer, scratch);
+  masm.subq(StackPointer, scratch);
+  masm.subq(Imm32(sizeof(void*)), scratch);  // Return address.
+
+  Address frameSizeAddr(FramePointer,
+                        BaselineFrame::reverseOffsetOfDebugFrameSize());
+  masm.store32(scratch, frameSizeAddr);
+#endif
 
   // Push the return address that's currently on top of the stack.
-  masm.Push(Operand(BaselineStackReg, 0));
+  masm.Push(Operand(StackPointer, 0));
 
   // Replace the original return address with the frame descriptor.
-  masm.makeFrameDescriptor(scratch, FrameType::BaselineJS,
-                           BaselineStubFrameLayout::Size());
-  masm.storePtr(scratch, Address(BaselineStackReg, sizeof(uintptr_t)));
+  masm.storePtr(ImmWord(MakeFrameDescriptor(FrameType::BaselineJS)),
+                Address(StackPointer, sizeof(uintptr_t)));
 
   // Save old frame pointer, stack pointer and stub reg.
-  masm.Push(ICStubReg);
   masm.Push(FramePointer);
-  masm.mov(BaselineStackReg, FramePointer);
+  masm.mov(StackPointer, FramePointer);
+
+  masm.Push(ICStubReg);
 }
 
 }  // namespace jit
