@@ -7,36 +7,32 @@
 #ifndef jsfriendapi_h
 #define jsfriendapi_h
 
-#include "mozilla/MemoryReporting.h"
-#include "mozilla/PodOperations.h"
-
 #include "jspubtd.h"
 
 #include "js/CallArgs.h"
-#include "js/CharacterEncoding.h"
 #include "js/Class.h"
-#include "js/ErrorReport.h"
-#include "js/Exception.h"
-#include "js/friend/PerformanceHint.h"
+#include "js/ColumnNumber.h"  // JS::LimitedColumnNumberOneOrigin
 #include "js/GCAPI.h"
 #include "js/HeapAPI.h"
 #include "js/Object.h"           // JS::GetClass
 #include "js/shadow/Function.h"  // JS::shadow::Function
 #include "js/shadow/Object.h"    // JS::shadow::Object
 #include "js/TypeDecls.h"
-#include "js/Utility.h"
 
 class JSJitInfo;
 
-namespace JS {
-template <class T>
-class Heap;
-
-class ExceptionStack;
-} /* namespace JS */
-
+/*
+ * Set a callback used to trace gray roots.
+ *
+ * The callback is called after the first slice of GC so the embedding must
+ * implement appropriate barriers on its gray roots to ensure correctness.
+ *
+ * This callback may be called multiple times for different sets of zones. Use
+ * JS::ZoneIsGrayMarking() to determine whether roots from a particular zone are
+ * required.
+ */
 extern JS_PUBLIC_API void JS_SetGrayGCRootsTracer(JSContext* cx,
-                                                  JSTraceDataOp traceOp,
+                                                  JSGrayRootsTracer traceOp,
                                                   void* data);
 
 extern JS_PUBLIC_API JSObject* JS_FindCompilationScope(JSContext* cx,
@@ -60,9 +56,9 @@ extern JS_PUBLIC_API bool JS_NondeterministicGetWeakSetKeys(
     JSContext* cx, JS::HandleObject obj, JS::MutableHandleObject ret);
 
 // Raw JSScript* because this needs to be callable from a signal handler.
-extern JS_PUBLIC_API unsigned JS_PCToLineNumber(JSScript* script,
-                                                jsbytecode* pc,
-                                                unsigned* columnp = nullptr);
+extern JS_PUBLIC_API unsigned JS_PCToLineNumber(
+    JSScript* script, jsbytecode* pc,
+    JS::LimitedColumnNumberOneOrigin* columnp = nullptr);
 
 /**
  * Determine whether the given object is backed by a DeadObjectProxy.
@@ -154,9 +150,9 @@ extern JS_PUBLIC_API bool ForceLexicalInitialization(JSContext* cx,
 /**
  * Whether we are poisoning unused/released data for error detection. Governed
  * by the JS_GC_ALLOW_EXTRA_POISONING #ifdef as well as the
- * $JSGC_EXTRA_POISONING environment variable.
+ * javascript.options.extra_gc_poisoning pref.
  */
-extern JS_PUBLIC_API int IsGCPoisoning();
+extern JS_PUBLIC_API bool IsGCPoisoning();
 
 extern JS_PUBLIC_API JSPrincipals* GetRealmPrincipals(JS::Realm* realm);
 
@@ -165,6 +161,8 @@ extern JS_PUBLIC_API void SetRealmPrincipals(JS::Realm* realm,
 
 extern JS_PUBLIC_API bool GetIsSecureContext(JS::Realm* realm);
 
+extern JS_PUBLIC_API bool GetDebuggerObservesWasm(JS::Realm* realm);
+
 }  // namespace JS
 
 /**
@@ -196,14 +194,11 @@ struct JSFunctionSpecWithHelp {
 };
 
 #define JS_FN_HELP(name, call, nargs, flags, usage, help) \
-  { name, call, nargs, (flags) | JSPROP_ENUMERATE, nullptr, usage, help }
+  {name, call, nargs, (flags) | JSPROP_ENUMERATE, nullptr, usage, help}
 #define JS_INLINABLE_FN_HELP(name, call, nargs, flags, native, usage, help)    \
-  {                                                                            \
-    name, call, nargs, (flags) | JSPROP_ENUMERATE, &js::jit::JitInfo_##native, \
-        usage, help                                                            \
-  }
-#define JS_FS_HELP_END \
-  { nullptr, nullptr, 0, 0, nullptr, nullptr }
+  {name,  call, nargs, (flags) | JSPROP_ENUMERATE, &js::jit::JitInfo_##native, \
+   usage, help}
+#define JS_FS_HELP_END {nullptr, nullptr, 0, 0, nullptr, nullptr}
 
 extern JS_PUBLIC_API bool JS_DefineFunctionsWithHelp(
     JSContext* cx, JS::HandleObject obj, const JSFunctionSpecWithHelp* fs);
@@ -224,6 +219,14 @@ namespace js {
  */
 extern JS_PUBLIC_API bool UseInternalJobQueues(JSContext* cx);
 
+#ifdef DEBUG
+/**
+ * Given internal job queues are used, return currently queued jobs as an
+ * array of job objects.
+ */
+extern JS_PUBLIC_API JSObject* GetJobsInInternalJobQueue(JSContext* cx);
+#endif
+
 /**
  * Enqueue |job| on the internal job queue.
  *
@@ -240,6 +243,12 @@ extern JS_PUBLIC_API bool EnqueueJob(JSContext* cx, JS::HandleObject job);
  */
 extern JS_PUBLIC_API void StopDrainingJobQueue(JSContext* cx);
 
+/**
+ * Instruct the runtime to restart draining the internal job queue after
+ * stopping it with StopDrainingJobQueue.
+ */
+extern JS_PUBLIC_API void RestartDrainingJobQueue(JSContext* cx);
+
 extern JS_PUBLIC_API void RunJobs(JSContext* cx);
 
 extern JS_PUBLIC_API JS::Zone* GetRealmZone(JS::Realm* realm);
@@ -383,6 +392,15 @@ JS_PUBLIC_API JSFunction* NewFunctionByIdWithReserved(JSContext* cx,
                                                       unsigned nargs,
                                                       unsigned flags, jsid id);
 
+JS_PUBLIC_API JSFunction* NewFunctionByIdWithReservedAndProto(
+    JSContext* cx, JSNative native, JS::Handle<JSObject*> proto, unsigned nargs,
+    unsigned flags, jsid id);
+
+/**
+ * Get or set function's reserved slot value.
+ * `fun` should be a function created with `*WithReserved` API above.
+ * Such functions have 2 reserved slots, and `which` can be either 0 or 1.
+ */
 JS_PUBLIC_API const JS::Value& GetFunctionNativeReserved(JSObject* fun,
                                                          size_t which);
 
@@ -435,7 +453,8 @@ JS_PUBLIC_API bool AppendUnique(JSContext* cx, JS::MutableHandleIdVector base,
  *
  * If it is, returns true and outputs the index in *indexp.
  */
-JS_PUBLIC_API bool StringIsArrayIndex(JSLinearString* str, uint32_t* indexp);
+JS_PUBLIC_API bool StringIsArrayIndex(const JSLinearString* str,
+                                      uint32_t* indexp);
 
 /**
  * Overload of StringIsArrayIndex taking a (char16_t*,length) pair. Behaves
@@ -465,8 +484,11 @@ JS_PUBLIC_API bool IsObjectInContextCompartment(JSObject* obj,
 
 using DOMInstanceClassHasProtoAtDepth = bool (*)(const JSClass*, uint32_t,
                                                  uint32_t);
+using DOMInstanceClassIsError = bool (*)(const JSClass*);
+
 struct JSDOMCallbacks {
   DOMInstanceClassHasProtoAtDepth instanceClassMatchesProto;
+  DOMInstanceClassIsError instanceClassIsError;
 };
 using DOMCallbacks = struct JSDOMCallbacks;
 
@@ -487,15 +509,12 @@ extern JS_PUBLIC_API JSLinearString* GetErrorTypeName(JSContext* cx,
                                                       int16_t exnType);
 
 /* Implemented in CrossCompartmentWrapper.cpp. */
-typedef enum NukeReferencesToWindow {
-  NukeWindowReferences,
-  DontNukeWindowReferences
-} NukeReferencesToWindow;
+enum NukeReferencesToWindow { NukeWindowReferences, DontNukeWindowReferences };
 
-typedef enum NukeReferencesFromTarget {
+enum NukeReferencesFromTarget {
   NukeAllReferences,
   NukeIncomingReferences,
-} NukeReferencesFromTarget;
+};
 
 /*
  * These filters are designed to be ephemeral stack classes, and thus don't
@@ -554,20 +573,21 @@ static const unsigned JS_FUNCTION_INTERPRETED_BITS = 0x0060;
 static MOZ_ALWAYS_INLINE const JSJitInfo* FUNCTION_VALUE_TO_JITINFO(
     const JS::Value& v) {
   JSObject* obj = &v.toObject();
-  MOZ_ASSERT(JS::GetClass(obj) == js::FunctionClassPtr);
+  MOZ_ASSERT(JS::GetClass(obj)->isJSFunction());
 
   auto* fun = reinterpret_cast<JS::shadow::Function*>(obj);
-  MOZ_ASSERT(!(fun->flags & js::JS_FUNCTION_INTERPRETED_BITS),
+  MOZ_ASSERT(!(fun->flagsAndArgCount() & js::JS_FUNCTION_INTERPRETED_BITS),
              "Unexpected non-native function");
 
-  return fun->jitinfo;
+  return static_cast<const JSJitInfo*>(fun->jitInfoOrScript());
 }
 
 static MOZ_ALWAYS_INLINE void SET_JITINFO(JSFunction* func,
                                           const JSJitInfo* info) {
   auto* fun = reinterpret_cast<JS::shadow::Function*>(func);
-  MOZ_ASSERT(!(fun->flags & js::JS_FUNCTION_INTERPRETED_BITS));
-  fun->jitinfo = info;
+  MOZ_ASSERT(!(fun->flagsAndArgCount() & js::JS_FUNCTION_INTERPRETED_BITS));
+
+  fun->setJitInfoOrScript(const_cast<JSJitInfo*>(info));
 }
 
 static_assert(sizeof(jsid) == sizeof(void*));
@@ -689,8 +709,8 @@ extern JS_PUBLIC_API bool IsSavedFrame(JSObject* obj);
 #if defined(XP_WIN)
 // Parameters use void* types to avoid #including windows.h. The return value of
 // this function is returned from the exception handler.
-typedef long (*JitExceptionHandler)(void* exceptionRecord,  // PEXECTION_RECORD
-                                    void* context);         // PCONTEXT
+using JitExceptionHandler = long (*)(void* exceptionRecord,  // PEXECTION_RECORD
+                                     void* context);         // PCONTEXT
 
 /**
  * Windows uses "structured exception handling" to handle faults. When a fault
@@ -726,11 +746,53 @@ class MOZ_STACK_CLASS JS_PUBLIC_API AutoAssertNoContentJS {
   bool prevAllowContentJS_;
 };
 
+/**
+ * This function reports memory used by a zone in bytes, this includes:
+ *  * The size of this JS GC zone.
+ *  * Malloc memory referred to from this zone.
+ *  * JIT memory for this zone.
+ *
+ * Note that malloc memory referred to from this zone can include
+ * SharedArrayBuffers which may also be referred to from other zones. Adding the
+ * memory usage of multiple zones may lead to an over-estimate.
+ */
+extern JS_PUBLIC_API uint64_t GetMemoryUsageForZone(JS::Zone* zone);
+
+enum class MemoryUse : uint8_t;
+
+namespace gc {
+
+struct SharedMemoryUse {
+  explicit SharedMemoryUse(MemoryUse use) : count(0), nbytes(0) {
+#ifdef DEBUG
+    this->use = use;
+#endif
+  }
+
+  size_t count;
+  size_t nbytes;
+#ifdef DEBUG
+  MemoryUse use;
+#endif
+};
+
+// A map which tracks shared memory uses (shared in the sense that an allocation
+// can be referenced by more than one GC thing in a zone). This allows us to
+// only account for the memory once.
+using SharedMemoryMap =
+    HashMap<void*, SharedMemoryUse, DefaultHasher<void*>, SystemAllocPolicy>;
+
+} /* namespace gc */
+
+extern JS_PUBLIC_API const gc::SharedMemoryMap& GetSharedMemoryUsageForZone(
+    JS::Zone* zone);
+
 /**
  * This function only reports GC heap memory,
  * and not malloc allocated memory associated with GC things.
+ * It reports the total of all memory for the whole Runtime.
  */
-extern JS_PUBLIC_API uint64_t GetGCHeapUsageForObjectZone(JSObject* obj);
+extern JS_PUBLIC_API uint64_t GetGCHeapUsage(JSContext* cx);
 
 class JS_PUBLIC_API CompartmentTransplantCallback {
  public:
