@@ -11,6 +11,8 @@
 #include "mozilla/Range.h"
 #include "mozilla/Utf8.h"
 
+#include <limits>
+
 #include "NamespaceImports.h"
 
 #include "js/Conversions.h"
@@ -26,20 +28,17 @@ class TaggedParserAtomIndex;
 }  // namespace frontend
 
 class GlobalObject;
-class StringBuffer;
+class StringBuilder;
 
 [[nodiscard]] extern bool InitRuntimeNumberState(JSRuntime* rt);
 
 // This is a no-op if built with JS_HAS_INTL_API.
 extern void FinishRuntimeNumberState(JSRuntime* rt);
 
-/* Initialize the Number class, returning its prototype object. */
-extern JSObject* InitNumberClass(JSContext* cx, Handle<GlobalObject*> global);
-
 /*
- * When base == 10, this function implements ToString() as specified by
- * ECMA-262-5 section 9.8.1; but note that it handles integers specially for
- * performance.  See also js::NumberToCString().
+ * This function implements ToString() as specified by ECMA-262-5 section 9.8.1;
+ * but note that it handles integers specially for performance.
+ * See also js::NumberToCString().
  */
 template <AllowGC allowGC>
 extern JSString* NumberToString(JSContext* cx, double d);
@@ -49,60 +48,77 @@ extern JSString* NumberToStringPure(JSContext* cx, double d);
 extern JSAtom* NumberToAtom(JSContext* cx, double d);
 
 frontend::TaggedParserAtomIndex NumberToParserAtom(
-    JSContext* cx, frontend::ParserAtomsTable& parserAtoms, double d);
+    FrontendContext* fc, frontend::ParserAtomsTable& parserAtoms, double d);
 
 template <AllowGC allowGC>
 extern JSLinearString* Int32ToString(JSContext* cx, int32_t i);
 
+template <AllowGC allowGC>
+extern JSLinearString* Int32ToStringWithHeap(JSContext* cx, int32_t i,
+                                             gc::Heap heap);
+
 extern JSLinearString* Int32ToStringPure(JSContext* cx, int32_t i);
 
+template <AllowGC allowGC>
+extern JSLinearString* Int32ToStringWithBase(JSContext* cx, int32_t i,
+                                             int32_t base, bool lowerCase);
+
 extern JSAtom* Int32ToAtom(JSContext* cx, int32_t si);
 
 frontend::TaggedParserAtomIndex Int32ToParserAtom(
-    JSContext* cx, frontend::ParserAtomsTable& parserAtoms, int32_t si);
+    FrontendContext* fc, frontend::ParserAtomsTable& parserAtoms, int32_t si);
 
 // ES6 15.7.3.12
-extern bool IsInteger(const Value& val);
-
 extern bool IsInteger(double d);
 
 /*
  * Convert an integer or double (contained in the given value) to a string and
- * append to the given buffer.
+ * append to the given string builder.
  */
-[[nodiscard]] extern bool NumberValueToStringBuffer(JSContext* cx,
-                                                    const Value& v,
-                                                    StringBuffer& sb);
+[[nodiscard]] extern bool NumberValueToStringBuilder(const Value& v,
+                                                     StringBuilder& sb);
 
 extern JSLinearString* IndexToString(JSContext* cx, uint32_t index);
 
-/*
- * Usually a small amount of static storage is enough, but sometimes we need
- * to dynamically allocate much more.  This struct encapsulates that.
- * Dynamically allocated memory will be freed when the object is destroyed.
- */
 struct ToCStringBuf {
-  /*
-   * The longest possible result that would need to fit in sbuf is
-   * (-0x80000000).toString(2), which has length 33.  Longer cases are
-   * possible, but they'll go in dbuf.
-   */
-  static const size_t sbufSize = 34;
-  char sbuf[sbufSize];
-  char* dbuf;
-
-  ToCStringBuf();
-  ~ToCStringBuf();
+  char sbuf[JS::MaximumNumberToStringLength] = {};
 };
 
-/*
- * Convert a number to a C string.  When base==10, this function implements
- * ToString() as specified by ECMA-262-5 section 9.8.1.  It handles integral
- * values cheaply.  Return nullptr if we ran out of memory.  See also
- * NumberToCString().
- */
-extern char* NumberToCString(JSContext* cx, ToCStringBuf* cbuf, double d,
-                             int base = 10);
+struct Int32ToCStringBuf {
+  // The amount of space large enough to store the null-terminated result of
+  // |ToString| on any int32.
+  //
+  // We use the same amount for uint32 (base 10 and base 16), even though uint32
+  // only need 11 characters (base 10) resp. 9 characters (base 16) instead of
+  // 12 characters for int32 in base 10.
+  static constexpr size_t MaximumInt32ToStringLength =
+      std::numeric_limits<int32_t>::digits10 +
+      1 +  // account for the largest possible int32 value
+      1 +  // sign for negative numbers
+      1    // null character
+      ;
+
+  char sbuf[MaximumInt32ToStringLength] = {};
+};
+
+// Convert a number to a C string.  This function implements ToString() as
+// specified by ECMA-262-5 section 9.8.1.  It handles integral values cheaply.
+// Infallible: always returns a non-nullptr string.
+// The optional `length` out-param is set to the string length of the result.
+extern char* NumberToCString(ToCStringBuf* cbuf, double d,
+                             size_t* length = nullptr);
+
+extern char* Int32ToCString(Int32ToCStringBuf* cbuf, int32_t value,
+                            size_t* length = nullptr);
+
+extern char* Uint32ToCString(Int32ToCStringBuf* cbuf, uint32_t value,
+                             size_t* length = nullptr);
+
+// Like NumberToCString, but accepts only unsigned integers and uses base 16.
+// Infallible: always returns a non-nullptr string.
+// The optional `length` out-param is set to the string length of the result.
+extern char* Uint32ToHexCString(Int32ToCStringBuf* cbuf, uint32_t value,
+                                size_t* length = nullptr);
 
 /*
  * The largest positive integer such that all positive integers less than it
@@ -110,6 +126,18 @@ extern char* NumberToCString(JSContext* cx, ToCStringBuf* cbuf, double d,
  */
 constexpr double DOUBLE_INTEGRAL_PRECISION_LIMIT = uint64_t(1) << 53;
 
+/*
+ * The smallest positive double such that all positive doubles larger or equal
+ * than it have an exact decimal representation without exponential form.
+ */
+constexpr double DOUBLE_DECIMAL_IN_SHORTEST_LOW = 1.0e-6;
+
+/*
+ * The largest positive double such that all positive doubles less than it
+ * have an exact decimal representation without exponential form.
+ */
+constexpr double DOUBLE_DECIMAL_IN_SHORTEST_HIGH = 1.0e21;
+
 /*
  * Parse a decimal number encoded in |chars|.  The decimal number must be
  * sufficiently small that it will not overflow the integrally-precise range of
@@ -140,7 +168,7 @@ enum class IntegerSeparatorHandling : bool { None, SkipUnderscore };
  */
 template <typename CharT>
 [[nodiscard]] extern bool GetPrefixInteger(
-    JSContext* cx, const CharT* start, const CharT* end, int base,
+    const CharT* start, const CharT* end, int base,
     IntegerSeparatorHandling separatorHandling, const CharT** endp, double* dp);
 
 inline const char16_t* ToRawChars(const char16_t* units) { return units; }
@@ -159,10 +187,10 @@ inline const unsigned char* ToRawChars(const mozilla::Utf8Unit* units) {
  */
 template <typename CharT>
 [[nodiscard]] extern bool GetFullInteger(
-    JSContext* cx, const CharT* start, const CharT* end, int base,
+    const CharT* start, const CharT* end, int base,
     IntegerSeparatorHandling separatorHandling, double* dp) {
   decltype(ToRawChars(start)) realEnd;
-  if (GetPrefixInteger(cx, ToRawChars(start), ToRawChars(end), base,
+  if (GetPrefixInteger(ToRawChars(start), ToRawChars(end), base,
                        separatorHandling, &realEnd, dp)) {
     MOZ_ASSERT(end == static_cast<const void*>(realEnd));
     return true;
@@ -177,7 +205,7 @@ template <typename CharT>
  * Numeric Literals.
  */
 template <typename CharT>
-[[nodiscard]] extern bool GetDecimalInteger(JSContext* cx, const CharT* start,
+[[nodiscard]] extern bool GetDecimalInteger(const CharT* start,
                                             const CharT* end, double* dp);
 
 /*
@@ -186,13 +214,11 @@ template <typename CharT>
  * cf. ES2020, 11.8.3 Numeric Literals.
  */
 template <typename CharT>
-[[nodiscard]] extern bool GetDecimalNonInteger(JSContext* cx,
-                                               const CharT* start,
-                                               const CharT* end, double* dp);
+[[nodiscard]] extern bool GetDecimal(const CharT* start, const CharT* end,
+                                     double* dp);
 
 template <typename CharT>
-bool CharsToNumber(JSContext* cx, const CharT* chars, size_t length,
-                   double* result);
+double CharsToNumber(const CharT* chars, size_t length);
 
 [[nodiscard]] extern bool StringToNumber(JSContext* cx, JSString* str,
                                          double* result);
@@ -200,13 +226,12 @@ bool CharsToNumber(JSContext* cx, const CharT* chars, size_t length,
 [[nodiscard]] extern bool StringToNumberPure(JSContext* cx, JSString* str,
                                              double* result);
 
-/*
- * Return true and set |*result| to the parsed number value if |str| can be
- * parsed as a number using the same rules as in |StringToNumber|. Otherwise
- * return false and leave |*result| in an indeterminate state.
- */
-[[nodiscard]] extern bool MaybeStringToNumber(JSLinearString* str,
-                                              double* result);
+// Infallible version of StringToNumber for linear strings.
+extern double LinearStringToNumber(const JSLinearString* str);
+
+// Parse the input string as if Number.parseInt had been called.
+extern bool NumberParseInt(JSContext* cx, JS::HandleString str, int32_t radix,
+                           JS::MutableHandleValue result);
 
 /* ES5 9.3 ToNumber, overwriting *vp with the appropriate number value. */
 [[nodiscard]] MOZ_ALWAYS_INLINE bool ToNumber(JSContext* cx,
@@ -251,19 +276,15 @@ bool ToInt32OrBigIntSlow(JSContext* cx, JS::MutableHandleValue vp);
 /*
  * Similar to strtod except that it replaces overflows with infinities of the
  * correct sign, and underflows with zeros of the correct sign.  Guaranteed to
- * return the closest double number to the given input in dp.
+ * return the closest double number to the given input.
  *
  * Also allows inputs of the form [+|-]Infinity, which produce an infinity of
  * the appropriate sign.  The case of the "Infinity" string must match exactly.
- * If the string does not contain a number, set *dEnd to begin and return 0.0
- * in *d.
- *
- * Return false if out of memory.
+ * If the string does not contain a number, set *dEnd to begin and return 0.0.
  */
 template <typename CharT>
-[[nodiscard]] extern bool js_strtod(JSContext* cx, const CharT* begin,
-                                    const CharT* end, const CharT** dEnd,
-                                    double* d);
+[[nodiscard]] extern double js_strtod(const CharT* begin, const CharT* end,
+                                      const CharT** dEnd);
 
 namespace js {
 
@@ -272,14 +293,12 @@ namespace js {
  * (and so |dEnd| would be a value already known).
  */
 template <typename CharT>
-[[nodiscard]] extern bool FullStringToDouble(JSContext* cx, const CharT* begin,
-                                             const CharT* end, double* d) {
+[[nodiscard]] extern double FullStringToDouble(const CharT* begin,
+                                               const CharT* end) {
   decltype(ToRawChars(begin)) realEnd;
-  if (js_strtod(cx, ToRawChars(begin), ToRawChars(end), &realEnd, d)) {
-    MOZ_ASSERT(end == static_cast<const void*>(realEnd));
-    return true;
-  }
-  return false;
+  double d = js_strtod(ToRawChars(begin), ToRawChars(end), &realEnd);
+  MOZ_ASSERT(end == static_cast<const void*>(realEnd));
+  return d;
 }
 
 [[nodiscard]] extern bool ThisNumberValueForToLocaleString(JSContext* cx,
@@ -288,6 +307,20 @@ template <typename CharT>
 
 [[nodiscard]] extern bool num_valueOf(JSContext* cx, unsigned argc, Value* vp);
 
+static inline bool IsNumberIndex(const Value& v) {
+  if (v.isInt32() && v.toInt32() >= 0) {
+    return true;
+  }
+
+  int64_t i;
+  if (v.isDouble() && mozilla::NumberEqualsInt64(v.toDouble(), &i) && i >= 0 &&
+      i <= MAX_ARRAY_INDEX) {
+    return true;
+  }
+
+  return false;
+}
+
 /*
  * Returns true if the given value is definitely an index: that is, the value
  * is a number that's an unsigned 32-bit integer.
@@ -371,6 +404,40 @@ static MOZ_ALWAYS_INLINE bool IsDefinitelyIndex(const Value& v,
   return ToIndex(cx, v, JSMSG_BAD_INDEX, index);
 }
 
+/**
+ * Convert |value| to an integer and clamp it to a valid integer index within
+ * the range `[0..length]`.
+ */
+template <typename ArrayLength>
+[[nodiscard]] extern bool ToIntegerIndexSlow(JSContext* cx, Handle<Value> value,
+                                             ArrayLength length,
+                                             ArrayLength* result);
+
+template <typename ArrayLength>
+[[nodiscard]] static inline bool ToIntegerIndex(JSContext* cx,
+                                                Handle<Value> value,
+                                                ArrayLength length,
+                                                ArrayLength* result) {
+  static_assert(std::is_unsigned_v<ArrayLength>);
+
+  // Optimize for the common case when |value| is an int32 to avoid unnecessary
+  // floating point computations.
+  if (value.isInt32()) {
+    int32_t relative = value.toInt32();
+
+    if (relative >= 0) {
+      *result = std::min(ArrayLength(relative), length);
+    } else if (mozilla::Abs(relative) <= length) {
+      *result = length - mozilla::Abs(relative);
+    } else {
+      *result = 0;
+    }
+    return true;
+  }
+
+  return ToIntegerIndexSlow(cx, value, length, result);
+}
+
 } /* namespace js */
 
 #endif /* jsnum_h */
