@@ -11,6 +11,8 @@
  * JS public API typedefs.
  */
 
+#include "mozilla/Assertions.h"  // MOZ_ASSERT_UNREACHABLE
+
 #include "jstypes.h"
 
 #include "js/ProtoKey.h"
@@ -20,6 +22,7 @@
 
 #if defined(JS_GC_ZEAL) || defined(DEBUG)
 #  define JSGC_HASH_TABLE_CHECKS
+#  define JS_CHECK_UNSAFE_CALL_WITH_ABI
 #endif
 
 namespace JS {
@@ -39,14 +42,34 @@ enum JSType {
   JSTYPE_NUMBER,    /* number */
   JSTYPE_BOOLEAN,   /* boolean */
   JSTYPE_SYMBOL,    /* symbol */
-  JSTYPE_BIGINT,    /* BigInt */
-#ifdef ENABLE_RECORD_TUPLE
-  JSTYPE_RECORD, /* record */
-  JSTYPE_TUPLE,  /* tuple */
-#endif
+  JSTYPE_BIGINT,    /* bigint */
   JSTYPE_LIMIT
 };
 
+inline const char* JSTypeToString(JSType type) {
+  switch (type) {
+    case JSTYPE_UNDEFINED:
+      return "undefined";
+    case JSTYPE_OBJECT:
+      return "object";
+    case JSTYPE_FUNCTION:
+      return "function";
+    case JSTYPE_STRING:
+      return "string";
+    case JSTYPE_NUMBER:
+      return "number";
+    case JSTYPE_BOOLEAN:
+      return "boolean";
+    case JSTYPE_SYMBOL:
+      return "symbol";
+    case JSTYPE_BIGINT:
+      return "bigint";
+    default:
+      MOZ_ASSERT_UNREACHABLE("Unknown JSType");
+  }
+  return "";
+}
+
 /* Dense index into cached prototypes and class atoms for standard objects. */
 enum JSProtoKey {
 #define PROTOKEY_AND_INITIALIZER(name, clasp) JSProto_##name,
@@ -85,6 +108,7 @@ inline JS::Zone* GetContextZone(const JSContext* cx);
 JS_PUBLIC_API bool CurrentThreadCanAccessRuntime(const JSRuntime* rt);
 
 #ifdef DEBUG
+JS_PUBLIC_API bool CurrentThreadIsMainThread();
 JS_PUBLIC_API bool CurrentThreadIsPerformingGC();
 #endif
 
