@@ -8,6 +8,7 @@
 
 #include <algorithm>
 #include <stdarg.h>
+#include <string_view>
 
 #include "util/Unicode.h"
 
@@ -15,8 +16,6 @@ using namespace js;
 using namespace js::cli;
 using namespace js::cli::detail;
 
-const char OptionParser::prognameMeta[] = "{progname}";
-
 #define OPTION_CONVERT_IMPL(__cls)                                             \
   bool Option::is##__cls##Option() const { return kind == OptionKind##__cls; } \
   __cls##Option* Option::as##__cls##Option() {                                 \
@@ -45,6 +44,12 @@ void OptionParser::setArgTerminatesOptions(const char* name, bool enabled) {
   findArgument(name)->setTerminatesOptions(enabled);
 }
 
+void OptionParser::setIgnoresUnknownOptions(const char* name, bool enabled) {
+  auto* opt = findOption(name);
+  MOZ_ASSERT(opt);
+  opt->setIgnoresUnknownOptions(enabled);
+}
+
 void OptionParser::setArgCapturesRest(const char* name) {
   MOZ_ASSERT(restArgument == -1,
              "only one argument may be set to capture the rest");
@@ -150,10 +155,12 @@ static const char* OptionFlagsToFormatInfo(char shortflag, bool isValued,
 }
 
 OptionParser::Result OptionParser::printHelp(const char* progname) {
-  const char* prefixEnd = strstr(usage, prognameMeta);
+  constexpr std::string_view prognameMeta = "{progname}";
+
+  const char* prefixEnd = strstr(usage, prognameMeta.data());
   if (prefixEnd) {
     printf("%.*s%s%s\n", int(prefixEnd - usage), usage, progname,
-           prefixEnd + sizeof(prognameMeta) - 1);
+           prefixEnd + prognameMeta.length());
   } else {
     puts(usage);
   }
@@ -348,10 +355,15 @@ OptionParser::Result OptionParser::handleArg(size_t argc, char** argv,
 OptionParser::Result OptionParser::parseArgs(int inputArgc, char** argv) {
   MOZ_ASSERT(inputArgc >= 0);
   size_t argc = inputArgc;
+
   // Permit a "no more options" capability, like |--| offers in many shell
   // interfaces.
   bool optionsAllowed = true;
 
+  // Whether unknown options should report a warning instead of an error. This
+  // is enabled by setIgnoresUnknownOptions and used for --fuzzing-safe.
+  bool ignoreUnknownOptions = false;
+
   for (size_t i = 1; i < argc; ++i) {
     char* arg = argv[i];
     Result r;
@@ -369,6 +381,11 @@ OptionParser::Result OptionParser::parseArgs(int inputArgc, char** argv) {
           /* Long option. */
           opt = findOption(arg + 2);
           if (!opt) {
+            if (ignoreUnknownOptions) {
+              fprintf(stderr, "Warning: Ignoring unknown shell flag: %s\n",
+                      arg);
+              continue;
+            }
             return error("Invalid long option: %s", arg);
           }
         }
@@ -383,6 +400,10 @@ OptionParser::Result OptionParser::parseArgs(int inputArgc, char** argv) {
         }
       }
 
+      if (opt->getIgnoresUnknownOptions()) {
+        ignoreUnknownOptions = true;
+      }
+
       r = handleOption(opt, argc, argv, &i, &optionsAllowed);
     } else {
       /* Argument. */
@@ -404,37 +425,38 @@ void OptionParser::setHelpOption(char shortflag, const char* longflag,
 bool OptionParser::getHelpOption() const { return helpOption.value; }
 
 bool OptionParser::getBoolOption(char shortflag) const {
-  return findOption(shortflag)->asBoolOption()->value;
+  return tryFindOption(shortflag)->asBoolOption()->value;
 }
 
 int OptionParser::getIntOption(char shortflag) const {
-  return findOption(shortflag)->asIntOption()->value;
+  return tryFindOption(shortflag)->asIntOption()->value;
 }
 
 const char* OptionParser::getStringOption(char shortflag) const {
-  return findOption(shortflag)->asStringOption()->value;
+  return tryFindOption(shortflag)->asStringOption()->value;
 }
 
 MultiStringRange OptionParser::getMultiStringOption(char shortflag) const {
-  const MultiStringOption* mso = findOption(shortflag)->asMultiStringOption();
+  const MultiStringOption* mso =
+      tryFindOption(shortflag)->asMultiStringOption();
   return MultiStringRange(mso->strings.begin(), mso->strings.end());
 }
 
 bool OptionParser::getBoolOption(const char* longflag) const {
-  return findOption(longflag)->asBoolOption()->value;
+  return tryFindOption(longflag)->asBoolOption()->value;
 }
 
 int OptionParser::getIntOption(const char* longflag) const {
-  return findOption(longflag)->asIntOption()->value;
+  return tryFindOption(longflag)->asIntOption()->value;
 }
 
 const char* OptionParser::getStringOption(const char* longflag) const {
-  return findOption(longflag)->asStringOption()->value;
+  return tryFindOption(longflag)->asStringOption()->value;
 }
 
 MultiStringRange OptionParser::getMultiStringOption(
     const char* longflag) const {
-  const MultiStringOption* mso = findOption(longflag)->asMultiStringOption();
+  const MultiStringOption* mso = tryFindOption(longflag)->asMultiStringOption();
   return MultiStringRange(mso->strings.begin(), mso->strings.end());
 }
 
@@ -465,6 +487,15 @@ const Option* OptionParser::findOption(char shortflag) const {
   return const_cast<OptionParser*>(this)->findOption(shortflag);
 }
 
+const Option* OptionParser::tryFindOption(char shortflag) const {
+  const Option* maybeOption = findOption(shortflag);
+  if (!maybeOption) {
+    fprintf(stderr, "Failed to find short option %c\n", shortflag);
+    MOZ_CRASH();
+  }
+  return maybeOption;
+}
+
 Option* OptionParser::findOption(const char* longflag) {
   for (Option* opt : options) {
     const char* target = opt->longflag;
@@ -498,6 +529,15 @@ const Option* OptionParser::findOption(const char* longflag) const {
   return const_cast<OptionParser*>(this)->findOption(longflag);
 }
 
+const Option* OptionParser::tryFindOption(const char* longflag) const {
+  const Option* maybeOption = findOption(longflag);
+  if (!maybeOption) {
+    fprintf(stderr, "Failed to find long option %s\n", longflag);
+    MOZ_CRASH();
+  }
+  return maybeOption;
+}
+
 /* Argument accessors */
 
 int OptionParser::findArgumentIndex(const char* name) const {
