@@ -2,7 +2,6 @@
 
 import math
 import sys
-
 from datetime import datetime, timedelta
 
 if sys.platform.startswith("win"):
@@ -11,7 +10,7 @@ else:
     from .terminal_unix import Terminal
 
 
-class NullProgressBar(object):
+class NullProgressBar:
     def update(self, current, data):
         pass
 
@@ -32,7 +31,7 @@ class NullProgressBar(object):
         return timedelta.max
 
 
-class ProgressBar(object):
+class ProgressBar:
     def __init__(self, limit, fmt):
         assert self.conservative_isatty()
 
@@ -82,7 +81,7 @@ class ProgressBar(object):
 
         # Build the bar.
         pct = int(100.0 * current / self.limit)
-        sys.stdout.write("{:3d}% ".format(pct))
+        sys.stdout.write(f"{pct:3d}% ")
 
         barlen = int(1.0 * self.barlen * current / self.limit) - 1
         bar = "=" * barlen + ">" + " " * (self.barlen - barlen - 1)
@@ -92,7 +91,7 @@ class ProgressBar(object):
         now = datetime.now()
         dt = now - self.t0
         dt = dt.seconds + dt.microseconds * 1e-6
-        sys.stdout.write("{:6.1f}s".format(dt))
+        sys.stdout.write(f"{dt:6.1f}s")
         Terminal.clear_right()
 
         # Force redisplay, since we didn't write a \n.
@@ -132,14 +131,6 @@ class ProgressBar(object):
     def conservative_isatty():
         """
         Prefer erring on the side of caution and not using terminal commands if
-        the current output stream may be a file.  We explicitly check for the
-        Android platform because terminal commands work poorly over ADB's
-        redirection.
+        the current output stream may be a file.
         """
-        try:
-            import android  # NOQA: F401
-
-            return False
-        except ImportError:
-            return sys.stdout.isatty()
-        return False
+        return sys.stdout.isatty()
