@@ -1,4 +1,4 @@
-// |reftest| skip-if(Android) silentfail slow
+// |reftest| skip-if(Android) silentfail skip -- disabled pending bug 657444
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,27 +11,26 @@ var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
-printStatus(summary);
+printStatus (summary);
 
 expectExitCode(0);
 expectExitCode(5);
-expectExitCode(3);
 
-try {
-  var fe = "v";
+var fe="vv";
 
-  for (i = 0; i < 25; i++)
-    fe += fe;
+for (i=0; i<24; i++)
+  fe += fe;
 
-  var fu = new Function(
-    fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe,
-    fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe,
-    "done"
+var fu=new Function(
+  fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe,
+  fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe, fe,
+  "done"
   );
 
-  print('Done');
-} catch (e) {
-  assertEq(e, "out of memory");
-}
+//alert("fu="+fu);
+//print("fu="+fu);
+var fuout = 'fu=' + fu;
+
+print('Done');
 
 reportCompare(expect, actual, summary);
