@@ -13,9 +13,9 @@ const numbers = [
 
 const options = {style: "currency", currency: "EUR"};
 
-// List of known approximately sign in CLDR 40.
+// List of known approximately sign in CLDR 46.
 const approximatelySigns = [
-  "~", "∼", "≈", "≃", "ca.", "約", "dáàṣì", "dáàshì",
+  "-", "~", "∼", "≈", "≃", "ca.", "約", "dáàṣì", "dáàshì",
 ];
 
 // Iterate over all locales and ensure we find exactly one approximately sign.
