@@ -18,15 +18,7 @@ function test()
 {
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  try
-  {
-    expect = 'SyntaxError: invalid range in character class';
-    (new RegExp("[\xDF-\xC7]]", "i")).exec("");
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
-  reportCompare(expect, actual, summary + '(new RegExp("[\xDF-\xC7]]", "i")).exec("")');
+ 
+  /[\s-:]/;
+  reportCompare(expect, actual, summary + '/[\s-:]/');
 }
