@@ -5,7 +5,7 @@
 
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 346645;
-var summary = 'Do not crash with non-empty array in destructuring assign LHS';
+var summary = 'Do not crash with empty object in destructuring assign LHS';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
@@ -18,10 +18,10 @@ function test()
 {
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
+ 
   try
   {
-    eval('({ a:[z] }) = 3;');
+    eval('({ a:{} }) = 3;');
   }
   catch(ex)
   {
