@@ -4,17 +4,31 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 474771;
-var summary = 'TM: do not assert: jumpTable == interruptJumpTable';
-var actual = '';
-var expect = '';
-
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+var BUGNUMBER = 355052;
+var summary = 'Do not crash with valueOf:gc and __iterator__';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
-this.__defineSetter__('x', function(){});
-for (var j = 0; j < 5; ++j) { x = 3; }
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
+function test()
+{
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
 
-reportCompare(expect, actual, summary);
+  expect = /TypeError: .+ is not a function/;
+  actual = 'No Error';
+  try
+  {
+    ( {valueOf: gc} - [].a )();
+  }
+  catch(ex)
+  {
+    actual = ex + '';
+  }
+
+  reportMatch(expect, actual, summary);
+}
