@@ -5,26 +5,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 358594;
+var summary = 'Do not crash on uneval(this).';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
-
-function test()
-{
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-// ------- Comment #101 From Gary Kwong [:nth10sd]
-
-  uneval(function(){with({functional: []}){x5, y = this;const y = undefined }});
-// Assertion failure: strcmp(rval, with_cookie) == 0, at ../jsopcode.cpp:2567
-
-  reportCompare(expect, actual, summary);
-}
+// don't crash|assert
+f = function () { };
+f.__proto__ = this; 
+Object.defineProperty(this, "m", { set: f, enumerable: true, configurable: true });
+uneval(this);
+reportCompare(expect, actual, summary);
