@@ -4,11 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452742;
-var summary = 'Do not do overzealous eval inside function optimization in BindNameToSlot';
-var actual = '';
-var expect = '';
-
+var BUGNUMBER = 433279;
+var summary = 'Do not assert: pn != tc->parseContext->nodeList';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 //-----------------------------------------------------------------------------
 test();
@@ -19,30 +18,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  expect = actual = 'No Error';
-
-  var obj = { x: -100 };
-
-  function a(x)
-  {
-    var orig_x = x;
-    var orig_obj_x = obj.x;
-
-    with (obj) { eval("x = x + 10"); }
-
-    if (x !== orig_x)
-      throw "Unexpected mutation of x: " + x;
-    if (obj.x !== orig_obj_x + 10)
-      throw "Unexpected mutation of obj.x: " + obj.x;
-  }
-
   try
   {
-    a(0);
+    eval('({a}) = b; with({}) { for(let y in z) { } }');
   }
   catch(ex)
   {
-    actual = ex + '';
   }
+
   reportCompare(expect, actual, summary);
 }
