@@ -2,7 +2,7 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-// https://github.com/tc39/test262/blob/master/INTERPRETING.md#host-defined-functions
+// https://github.com/tc39/test262/blob/main/INTERPRETING.md#host-defined-functions
 ;(function createHostObject(global) {
     "use strict";
 
@@ -126,6 +126,8 @@ $262.agent = (function (global) {
 
     var {getSharedArrayBuffer} = global;
 
+    var _finished = { done: false };
+
     var _ia = new Int32Array(getSharedArrayBuffer());
     var agent = {
         receiveBroadcast(receiver) {
@@ -138,6 +140,7 @@ $262.agent = (function (global) {
             while (Atomics_load(_ia, ${_MSG_LOC}) === k)
                 ;
             receiver(received_sab, received_id);
+            waitForDone(_finished);
         },
 
         report(msg) {
@@ -157,7 +160,9 @@ $262.agent = (function (global) {
             Atomics_wait(_ia, ${_SLEEP_LOC}, 0, s);
         },
 
-        leaving() {},
+        leaving() {
+            _finished.done = true;
+        },
 
         monotonicNow: global.monotonicNow,
     };
