@@ -13,16 +13,12 @@ assert.throws(
   TypeError,
   function () {
     subclass_regexp.compile();
-  },
-  "`subclass_regexp.compile()` throws TypeError"
-);
+  });
 
 assert.throws(
   TypeError,
   function () {
     RegExp.prototype.compile.call(subclass_regexp);
-  },
-  "`RegExp.prototype.compile.call(subclass_regexp)` throws TypeError"
-);
+  });
 
 reportCompare(0, 0);
