@@ -2,7 +2,6 @@
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
 esid: sec-array.from
-es6id: 22.1.2.1
 description: Setting property on result value (traversed via iterator)
 info: |
     [...]
@@ -47,7 +46,15 @@ items[Symbol.iterator] = function() {
 
 result = Array.from(items);
 
-assert.sameValue(result[0], firstIterResult.value);
-assert.sameValue(result[1], secondIterResult.value);
+assert.sameValue(
+  result[0],
+  firstIterResult.value,
+  'The value of result[0] is expected to equal the value of firstIterResult.value'
+);
+assert.sameValue(
+  result[1],
+  secondIterResult.value,
+  'The value of result[1] is expected to equal the value of secondIterResult.value'
+);
 
 reportCompare(0, 0);
