@@ -2,7 +2,6 @@
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
 esid: sec-array.from
-es6id: 22.1.2.1
 description: Setting length of object (traversed via iterator)
 info: |
     [...]
@@ -34,7 +33,7 @@ nextIterResult = lastIterResult = {
 };
 result = Array.from(items);
 
-assert.sameValue(result.length, 0);
+assert.sameValue(result.length, 0, 'The value of result.length is expected to be 0');
 
 nextIterResult = {
   done: false
@@ -44,6 +43,6 @@ lastIterResult = {
 };
 result = Array.from(items);
 
-assert.sameValue(result.length, 1);
+assert.sameValue(result.length, 1, 'The value of result.length is expected to be 1');
 
 reportCompare(0, 0);
