@@ -5,7 +5,6 @@
 
 /*---
 esid: sec-array.prototype.concat
-es6id: 22.1.3.1_3
 description: Array.prototype.concat sloppy arguments with dupes
 flags: [noStrict]
 includes: [compareArray.js]
@@ -15,11 +14,15 @@ var args = (function(a, a, a) {
   return arguments;
 })(1, 2, 3);
 args[Symbol.isConcatSpreadable] = true;
-assert(compareArray([].concat(args, args), [1, 2, 3, 1, 2, 3]));
+assert.compareArray([].concat(args, args), [1, 2, 3, 1, 2, 3],
+  '[].concat((function(a, a, a) {return arguments;})(1, 2, 3), (function(a, a, a) {return arguments;})(1, 2, 3)) must return [1, 2, 3, 1, 2, 3]'
+);
 
 Object.defineProperty(args, "length", {
   value: 6
 });
-assert(compareArray([].concat(args), [1, 2, 3, void 0, void 0, void 0]));
+assert.compareArray([].concat(args), [1, 2, 3, void 0, void 0, void 0],
+  '[].concat((function(a, a, a) {return arguments;})(1, 2, 3)) must return [1, 2, 3, void 0, void 0, void 0]'
+);
 
 reportCompare(0, 0);
