@@ -29,7 +29,7 @@ assert.sameValue(
 
 assert.throws(TypeError, () => {
   new Array.prototype.copyWithin();
-}, '`new Array.prototype.copyWithin()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
