@@ -19,9 +19,16 @@ info: |
        b. If C is null, let C be undefined.
     [...]
     9. If IsConstructor(C) is false, throw a TypeError exception.
-features: [Symbol.species]
+includes: [isConstructor.js]
+features: [Symbol.species, Reflect.construct]
 ---*/
 
+assert.sameValue(
+  isConstructor(parseInt),
+  false,
+  'precondition: isConstructor(parseInt) must return false'
+);
+
 var a = [];
 var callCount = 0;
 var cb = function() {
@@ -33,7 +40,7 @@ a.constructor[Symbol.species] = parseInt;
 
 assert.throws(TypeError, function() {
   a.filter(cb);
-});
+}, 'a.filter(cb) throws a TypeError exception');
 assert.sameValue(callCount, 0);
 
 reportCompare(0, 0);
