@@ -2,20 +2,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
 esid: sec-array.prototype.find
-es6id: 22.1.3.8
 description: Array.prototype.find.length value and descriptor.
 info: |
   17 ECMAScript Standard Built-in Objects
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(
-  Array.prototype.find.length, 1,
-  'The value of `Array.prototype.find.length` is `1`'
-);
-
-verifyNotEnumerable(Array.prototype.find, 'length');
-verifyNotWritable(Array.prototype.find, 'length');
-verifyConfigurable(Array.prototype.find, 'length');
+verifyProperty(Array.prototype.find, "length", {
+  value: 1,
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
