@@ -1,4 +1,3 @@
-// |reftest| skip -- array-find-from-last is not supported
 // Copyright (C) 2021 Microsoft. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -26,7 +25,7 @@ assert.sameValue(isConstructor(Array.prototype.findLast), false, 'isConstructor(
 
 assert.throws(TypeError, () => {
   new Array.prototype.findLast(() => {});
-}, '`new Array.prototype.findLast(() => {})` throws TypeError');
+});
 
 
 reportCompare(0, 0);
