@@ -16,8 +16,10 @@ includes: [propertyHelper.js]
 
 assert.sameValue(typeof Array.prototype.map, 'function', 'typeof');
 
-verifyNotEnumerable(Array.prototype, "map");
-verifyWritable(Array.prototype, "map");
-verifyConfigurable(Array.prototype, "map");
+verifyProperty(Array.prototype, "map", {
+  writable: true,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
