@@ -4,9 +4,8 @@
 /*---
 esid: sec-array.prototype.reduce
 description: >
-    Array.prototype.reduce - element to be retrieved is own data
-    property that overrides an inherited accessor property on an
-    Array-like object
+    Array.prototype.reduce - element to be retrieved is own accessor
+    property that overrides an inherited data property on an Array
 ---*/
 
 var testResult = false;
@@ -17,27 +16,17 @@ function callbackfn(prevVal, curVal, idx, obj) {
   }
 }
 
-var proto = {};
+Array.prototype[0] = 0;
+var arr = [, 1, 2];
 
-Object.defineProperty(proto, "0", {
+Object.defineProperty(arr, "0", {
   get: function() {
-    return 0;
+    return "9";
   },
   configurable: true
 });
 
-var Con = function() {};
-Con.prototype = proto;
-
-var child = new Con();
-child.length = 2;
-Object.defineProperty(child, "0", {
-  value: "9",
-  configurable: true
-});
-child[1] = "1";
-
-Array.prototype.reduce.call(child, callbackfn);
+arr.reduce(callbackfn);
 
 assert(testResult, 'testResult !== true');
 
