@@ -19,10 +19,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Array.prototype.reverse.name, "reverse");
-
-verifyNotEnumerable(Array.prototype.reverse, "name");
-verifyNotWritable(Array.prototype.reverse, "name");
-verifyConfigurable(Array.prototype.reverse, "name");
+verifyProperty(Array.prototype.reverse, "name", {
+  value: "reverse",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
