@@ -21,10 +21,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Array.prototype.splice.length, 2);
-
-verifyNotEnumerable(Array.prototype.splice, 'length');
-verifyNotWritable(Array.prototype.splice, 'length');
-verifyConfigurable(Array.prototype.splice, 'length');
+verifyProperty(Array.prototype.splice, "length", {
+  value: 2,
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
