@@ -21,9 +21,8 @@ var obj = {
 var arr = [undefined, obj, null, obj, obj];
 arr.toLocaleString();
 
-//CHECK#1
 if (n !== 3) {
-  $ERROR('#1: var n = 0; var obj = {toLocaleString: function() {n++}}; var arr = [undefined, obj, null, obj, obj]; arr.toLocaleString(); n === 3. Actual: ' + (n));
+  throw new Test262Error('#1: var n = 0; var obj = {toLocaleString: function() {n++}}; var arr = [undefined, obj, null, obj, obj]; arr.toLocaleString(); n === 3. Actual: ' + (n));
 }
 
 reportCompare(0, 0);
