@@ -21,10 +21,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Array.prototype.toLocaleString.length, 0);
-
-verifyNotEnumerable(Array.prototype.toLocaleString, 'length');
-verifyNotWritable(Array.prototype.toLocaleString, 'length');
-verifyConfigurable(Array.prototype.toLocaleString, 'length');
+verifyProperty(Array.prototype.toLocaleString, "length", {
+  value: 0,
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
