@@ -15,13 +15,11 @@ includes: [propertyHelper.js]
 
 var descriptor = Object.getOwnPropertyDescriptor(ArrayBuffer, Symbol.species);
 
-assert.sameValue(
-  descriptor.get.name,
-  'get [Symbol.species]'
-);
-
-verifyNotEnumerable(descriptor.get, 'name');
-verifyNotWritable(descriptor.get, 'name');
-verifyConfigurable(descriptor.get, 'name');
+verifyProperty(descriptor.get, "name", {
+  value: "get [Symbol.species]",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
