@@ -3,7 +3,6 @@
 
 /*---
 esid: sec-arraybuffer.isview
-es6id: 24.1.3.1
 description: >
   ArrayBuffer.isView.name is "isView".
 info: |
@@ -20,10 +19,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(ArrayBuffer.isView.name, "isView");
-
-verifyNotEnumerable(ArrayBuffer.isView, "name");
-verifyNotWritable(ArrayBuffer.isView, "name");
-verifyConfigurable(ArrayBuffer.isView, "name");
+verifyProperty(ArrayBuffer.isView, "name", {
+  value: "isView",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
