@@ -17,13 +17,11 @@ var descriptor = Object.getOwnPropertyDescriptor(
   ArrayBuffer.prototype, 'byteLength'
 );
 
-assert.sameValue(
-  descriptor.get.name, 'get byteLength',
-  'The value of `descriptor.get.name` is `"get byteLength"`'
-);
-
-verifyNotEnumerable(descriptor.get, 'name');
-verifyNotWritable(descriptor.get, 'name');
-verifyConfigurable(descriptor.get, 'name');
+verifyProperty(descriptor.get, "name", {
+  value: "get byteLength",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
