@@ -15,8 +15,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(ArrayBuffer.prototype, "slice");
-verifyWritable(ArrayBuffer.prototype, "slice");
-verifyConfigurable(ArrayBuffer.prototype, "slice");
+verifyProperty(ArrayBuffer.prototype, "slice", {
+  writable: true,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
