@@ -1,4 +1,3 @@
-// |reftest| skip -- resizable-arraybuffer is not supported
 // Copyright (C) 2021 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
@@ -13,7 +12,7 @@ info: |
     Annex B.2 has the attributes { [[Writable]]: true, [[Enumerable]]: false,
     [[Configurable]]: true } unless otherwise specified.
 includes: [propertyHelper.js]
-features: [resizable-arraybuffer]
+features: [arraybuffer-transfer]
 ---*/
 
 verifyProperty(ArrayBuffer.prototype, 'transfer', {
