@@ -34,10 +34,10 @@ async function* asyncg() {
 
 asyncg().next().then(function (result) {
   assert(
-    Object.hasOwnProperty.call(result, 'value'), 'Has "own" property `value`'
+    Object.prototype.hasOwnProperty.call(result, 'value'), 'Has "own" property `value`'
   );
   assert(
-    Object.hasOwnProperty.call(result, 'done'), 'Has "own" property `done`'
+    Object.prototype.hasOwnProperty.call(result, 'done'), 'Has "own" property `done`'
   );
   assert.sameValue(Object.getPrototypeOf(result), Object.prototype);
 }).then($DONE, $DONE);
