@@ -15,16 +15,16 @@ info: |
   If Type(argument) is not Object, return false.
   If argument has a [[Construct]] internal method, return true.
   Return false.
-includes: [isConstructor.js, hidden-constructors.js]
+includes: [isConstructor.js, wellKnownIntrinsicObjects.js]
 features: [Reflect.construct]
 ---*/
 
+var AsyncGeneratorFunction = getWellKnownIntrinsicObject('%AsyncGeneratorFunction%');
 assert.sameValue(
   isConstructor(AsyncGeneratorFunction),
   true,
   'isConstructor(AsyncGeneratorFunction) must return true'
 );
 new AsyncGeneratorFunction();
-  
 
 reportCompare(0, 0);
