@@ -19,10 +19,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(DataView.prototype.setFloat64.name, "setFloat64");
-
-verifyNotEnumerable(DataView.prototype.setFloat64, "name");
-verifyNotWritable(DataView.prototype.setFloat64, "name");
-verifyConfigurable(DataView.prototype.setFloat64, "name");
+verifyProperty(DataView.prototype.setFloat64, "name", {
+  value: "setFloat64",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
