@@ -15,39 +15,39 @@ function DateValue(year, month, date, hours, minutes, seconds, ms) {
 
 var x;
 x = DateValue(1899, 11);
-assert.sameValue(x, NaN, "(1899, 11)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1899, 12);
-assert.sameValue(x, NaN, "(1899, 12)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1900, 0);
-assert.sameValue(x, NaN, "(1900, 0)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1969, 11);
-assert.sameValue(x, NaN, "(1969, 11)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1969, 12);
-assert.sameValue(x, NaN, "(1969, 12)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1970, 0);
-assert.sameValue(x, NaN, "(1970, 0)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1999, 11);
-assert.sameValue(x, NaN, "(1999, 11)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(1999, 12);
-assert.sameValue(x, NaN, "(1999, 12)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(2000, 0);
-assert.sameValue(x, NaN, "(2000, 0)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(2099, 11);
-assert.sameValue(x, NaN, "(2099, 11)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(2099, 12);
-assert.sameValue(x, NaN, "(2099, 12)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 x = DateValue(2100, 0);
-assert.sameValue(x, NaN, "(2100, 0)");
+assert.sameValue(x, NaN, 'The value of x is expected to equal NaN');
 
 reportCompare(0, 0);
