@@ -4,12 +4,12 @@
 /*---
 info: The Date.prototype has the property "toLocaleString"
 esid: sec-properties-of-the-date-prototype-object
-es5id: 15.9.5_A05_T1
 description: The Date.prototype has the property "toLocaleString"
 ---*/
-
-if (Date.prototype.hasOwnProperty("toLocaleString") !== true) {
-  $ERROR('#1: The Date.prototype has the property "toLocaleString"');
-}
+assert.sameValue(
+  Date.prototype.hasOwnProperty("toLocaleString"),
+  true,
+  'Date.prototype.hasOwnProperty("toLocaleString") must return true'
+);
 
 reportCompare(0, 0);
