@@ -6,9 +6,10 @@ info: The Date.prototype has the property "setMinutes"
 esid: sec-properties-of-the-date-prototype-object
 description: The Date.prototype has the property "setMinutes"
 ---*/
-
-if (Date.prototype.hasOwnProperty("setMinutes") !== true) {
-  $ERROR('#1: The Date.prototype has the property "setMinutes"');
-}
+assert.sameValue(
+  Date.prototype.hasOwnProperty("setMinutes"),
+  true,
+  'Date.prototype.hasOwnProperty("setMinutes") must return true'
+);
 
 reportCompare(0, 0);
