@@ -6,9 +6,10 @@ info: The Date.prototype has the property "setUTCDate"
 esid: sec-properties-of-the-date-prototype-object
 description: The Date.prototype has the property "setUTCDate"
 ---*/
-
-if (Date.prototype.hasOwnProperty("setUTCDate") !== true) {
-  $ERROR('#1: The Date.prototype has the property "setUTCDate"');
-}
+assert.sameValue(
+  Date.prototype.hasOwnProperty("setUTCDate"),
+  true,
+  'Date.prototype.hasOwnProperty("setUTCDate") must return true'
+);
 
 reportCompare(0, 0);
