@@ -6,9 +6,10 @@ info: The Date.prototype has the property "toUTCString"
 esid: sec-properties-of-the-date-prototype-object
 description: The Date.prototype has the property "toUTCString"
 ---*/
-
-if (Date.prototype.hasOwnProperty("toUTCString") !== true) {
-  $ERROR('#1: The Date.prototype has the property "toUTCString"');
-}
+assert.sameValue(
+  Date.prototype.hasOwnProperty("toUTCString"),
+  true,
+  'Date.prototype.hasOwnProperty("toUTCString") must return true'
+);
 
 reportCompare(0, 0);
