@@ -3,7 +3,6 @@
 
 /*---
 esid: sec-date.prototype.tojson
-es6id: 20.3.4.37
 description: >
   Date.prototype.toJSON.name is "toJSON".
 info: |
@@ -20,10 +19,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Date.prototype.toJSON.name, "toJSON");
-
-verifyNotEnumerable(Date.prototype.toJSON, "name");
-verifyNotWritable(Date.prototype.toJSON, "name");
-verifyConfigurable(Date.prototype.toJSON, "name");
+verifyProperty(Date.prototype.toJSON, 'name', {
+  value: 'toJSON',
+  writable: false,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
