@@ -3,7 +3,6 @@
 
 /*---
 esid: sec-date.prototype.tolocaletimestring
-es6id: 20.3.4.40
 description: >
   Date.prototype.toLocaleTimeString.name is "toLocaleTimeString".
 info: |
@@ -20,10 +19,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Date.prototype.toLocaleTimeString.name, "toLocaleTimeString");
-
-verifyNotEnumerable(Date.prototype.toLocaleTimeString, "name");
-verifyNotWritable(Date.prototype.toLocaleTimeString, "name");
-verifyConfigurable(Date.prototype.toLocaleTimeString, "name");
+verifyProperty(Date.prototype.toLocaleTimeString, "name", {
+  value: "toLocaleTimeString",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
