@@ -3,14 +3,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.2.1-11-4-s
+es5id: 15.3.2.1-11-7-s
 description: >
     Function constructor call from strict code with formal parameter
-    named 'eval' does not throws SyntaxError if function body is not
-    strict mode
+    named arguments does not throws SyntaxError if function body is
+    not strict mode
 flags: [onlyStrict]
 ---*/
 
-Function('eval', 'return;');
+Function('arguments', 'return;');
 
 reportCompare(0, 0);
