@@ -3,19 +3,23 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-3gs
+es5id: 15.3.5.4_2-21gs
 description: >
     Strict mode - checking access to strict function caller from
-    strict function (FunctionExpression defined within strict mode)
+    strict function (FunctionDeclaration defined within a
+    FunctionDeclaration inside strict mode)
 flags: [onlyStrict]
 ---*/
 
-var f = function() {
-  gNonStrict();
+function f1() {
+  function f() {
+    gNonStrict();
+  }
+  f();
 }
 
 assert.throws(TypeError, function() {
-  f();
+  f1();
 });
 
 function gNonStrict() {
