@@ -2,11 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-77gs
+es5id: 15.3.5.4_2-90gs
 description: >
     Strict mode - checking access to strict function caller from
     non-strict function (non-strict function declaration called by
-    strict Function constructor)
+    strict Function.prototype.bind(null)())
 flags: [noStrict]
 features: [caller]
 ---*/
@@ -16,7 +16,7 @@ function f() {
 };
 (function() {
   "use strict";
-  Function("return f();")();
+  f.bind(null)();
 })();
 
 
