@@ -11,21 +11,13 @@ description: >
     arg3", "return arg1+arg2+arg3;"
 ---*/
 
-//CHECK#1
 try {
   var f = Function("arg1, arg2, arg3", "return arg1+arg2+arg3;");
 } catch (e) {
-  $ERROR('#1: test failed');
+  throw new Test262Error('#1: test failed');
 }
 
-//CHECK#2
-if (!(f instanceof Function)) {
-  $ERROR('#2: It is permissible but not necessary to have one argument for each formal parameter to be specified');
-}
-
-//CHECK#3
-if (f(1, 1, "ABBA") !== "2ABBA") {
-  $ERROR('#3: It is permissible but not necessary to have one argument for each formal parameter to be specified');
-}
+assert(f instanceof Function, 'The result of evaluating (f instanceof Function) is expected to be true');
+assert.sameValue(f(1, 1, "ABBA"), "2ABBA", 'f(1, 1, ABBA) must return "2ABBA"');
 
 reportCompare(0, 0);
