@@ -16,9 +16,6 @@ description: >
 })();
 
 
-//CHECK#1
-if (this["feat"] !== "in da haus") {
-  $ERROR('#1: If thisArg is null or undefined, the called function is passed the global object as the this value');
-}
+assert.sameValue(this["feat"], "in da haus", 'The value of this["feat"] is expected to be "in da haus"');
 
 reportCompare(0, 0);
