@@ -1,7 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
-es6id: 23.1.3.3
+esid: sec-map.prototype.delete
 description: >
   Throws a TypeError if `this` is a Set object.
 info: |
