@@ -16,8 +16,10 @@ assert.sameValue(
   'typeof Map.prototype.delete is "function"'
 );
 
-verifyNotEnumerable(Map.prototype, 'delete');
-verifyWritable(Map.prototype, 'delete');
-verifyConfigurable(Map.prototype, 'delete');
+verifyProperty(Map.prototype, 'delete', {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
