@@ -1,7 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
-es6id: 23.1.3.7
+esid: sec-map.prototype.has
 description: >
   Property type and descriptor.
 info: |
@@ -17,8 +17,10 @@ assert.sameValue(
   '`typeof Map.prototype.has` is `function`'
 );
 
-verifyNotEnumerable(Map.prototype, 'has');
-verifyWritable(Map.prototype, 'has');
-verifyConfigurable(Map.prototype, 'has');
+verifyProperty(Map.prototype, 'has', {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
