@@ -1,7 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
-es6id: 23.1.3.8
+esid: sec-map.prototype.keys
 description: >
   Map.prototype.keys.name value and descriptor.
 info: |
@@ -12,13 +12,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(
-  Map.prototype.keys.name, 'keys',
-  'The value of `Map.prototype.keys.name` is `"keys"`'
-);
-
-verifyNotEnumerable(Map.prototype.keys, 'name');
-verifyNotWritable(Map.prototype.keys, 'name');
-verifyConfigurable(Map.prototype.keys, 'name');
+verifyProperty(Map.prototype.keys, "name", {
+  value: "keys",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
