@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Math.abs), false, 'isConstructor(Math.abs) must r
 
 assert.throws(TypeError, () => {
   new Math.abs();
-}, '`new Math.abs()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
