@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Math.asinh), false, 'isConstructor(Math.asinh) mu
 
 assert.throws(TypeError, () => {
   new Math.asinh();
-}, '`new Math.asinh()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
