@@ -12,8 +12,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(Math, "clz32");
-verifyWritable(Math, "clz32");
-verifyConfigurable(Math, "clz32");
+verifyProperty(Math, "clz32", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
