@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Math.cosh), false, 'isConstructor(Math.cosh) must
 
 assert.throws(TypeError, () => {
   new Math.cosh();
-}, '`new Math.cosh()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
