@@ -7,8 +7,10 @@ includes: [propertyHelper.js]
 es6id: 20.2.2.21
 ---*/
 
-verifyNotEnumerable(Math, "log1p");
-verifyWritable(Math, "log1p");
-verifyConfigurable(Math, "log1p");
+verifyProperty(Math, "log1p", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
