@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Math.tanh), false, 'isConstructor(Math.tanh) must
 
 assert.throws(TypeError, () => {
   new Math.tanh();
-}, '`new Math.tanh()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
