@@ -14,8 +14,10 @@ includes: [propertyHelper.js]
 
 assert.sameValue(SyntaxError.prototype, Object.getPrototypeOf(new SyntaxError));
 
-verifyNotEnumerable(SyntaxError, "prototype");
-verifyNotWritable(SyntaxError, "prototype");
-verifyNotConfigurable(SyntaxError, "prototype");
+verifyProperty(SyntaxError, "prototype", {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 reportCompare(0, 0);
