@@ -18,8 +18,10 @@ includes: [propertyHelper.js]
 
 assert.sameValue(SyntaxError.prototype.constructor, SyntaxError);
 
-verifyNotEnumerable(SyntaxError.prototype, "constructor");
-verifyWritable(SyntaxError.prototype, "constructor");
-verifyConfigurable(SyntaxError.prototype, "constructor");
+verifyProperty(SyntaxError.prototype, "constructor", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
