@@ -18,8 +18,10 @@ includes: [propertyHelper.js]
 
 assert.sameValue(URIError.prototype.constructor, URIError);
 
-verifyNotEnumerable(URIError.prototype, "constructor");
-verifyWritable(URIError.prototype, "constructor");
-verifyConfigurable(URIError.prototype, "constructor");
+verifyProperty(URIError.prototype, "constructor", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
