@@ -26,8 +26,10 @@ assert(
   "value is smaller than 0.000001"
 );
 
-verifyNotEnumerable(Number, "EPSILON");
-verifyNotWritable(Number, "EPSILON");
-verifyNotConfigurable(Number, "EPSILON");
+verifyProperty(Number, "EPSILON", {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 reportCompare(0, 0);
