@@ -17,8 +17,10 @@ includes: [propertyHelper.js]
 
 assert.sameValue(Number.NaN, NaN);
 
-verifyNotEnumerable(Number, "NaN");
-verifyNotWritable(Number, "NaN");
-verifyNotConfigurable(Number, "NaN");
+verifyProperty(Number, "NaN", {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 reportCompare(0, 0);
