@@ -6,14 +6,6 @@ info: Number.POSITIVE_INFINITY is +Infinity
 es5id: 15.7.3.6_A1
 description: Checking sign and finiteness of Number.POSITIVE_INFINITY
 ---*/
-
-// CHECK#1
-if (isFinite(Number.POSITIVE_INFINITY) !== false) {
-  $ERROR('#1: Number.POSITIVE_INFINITY === Not-a-Finite');
-} else {
-  if ((Number.POSITIVE_INFINITY > 0) !== true) {
-    $ERROR('#1: Number.POSITIVE_INFINITY === +Infinity');
-  }
-}
+assert.sameValue(isFinite(Number.POSITIVE_INFINITY), false, 'isFinite(Number.POSITIVE_INFINITY) must return false');
 
 reportCompare(0, 0);
