@@ -8,20 +8,22 @@ info: |
 es5id: 9.3.1_A13
 description: Compare '12' with Number("1")*10+Number("2") and analogous
 ---*/
+assert.sameValue(
+  +("12"),
+  12,
+  'The value of `+("12")` is expected to be 12'
+);
 
-// CHECK#1
-if (+("12") !== Number("1") * 10 + Number("2")) {
-  $ERROR('#1: +("12") === Number("1")*10+Number("2")');
-}
+assert.sameValue(
+  Number("123"),
+  123,
+  'Number("123") must return 123'
+);
 
-// CHECK#2
-if (Number("123") !== Number("12") * 10 + Number("3")) {
-  $ERROR('#2: Number("123") === Number("12")*10+Number("3")');
-}
-
-// CHECK#2
-if (Number("1234") !== Number("123") * 10 + Number("4")) {
-  $ERROR('#2: Number("1234") === Number("123")*10+Number("4")');
-}
+assert.sameValue(
+  Number("1234"),
+  1234,
+  'Number("1234") must return 1234'
+);
 
 reportCompare(0, 0);
