@@ -13,8 +13,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(Number, "prototype");
-verifyNotWritable(Number, "prototype");
-verifyNotConfigurable(Number, "prototype");
+verifyProperty(Number, "prototype", {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 reportCompare(0, 0);
