@@ -29,7 +29,7 @@ assert.sameValue(
 
 assert.throws(TypeError, () => {
   new Number.prototype.toExponential();
-}, '`new Number.prototype.toExponential()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
