@@ -2,7 +2,6 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es6id: 20.1.3.3
 esid: sec-number.prototype.tofixed
 description: >
   "toFixed" property of Number.prototype
@@ -15,8 +14,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(Number.prototype, "toFixed");
-verifyWritable(Number.prototype, "toFixed");
-verifyConfigurable(Number.prototype, "toFixed");
+verifyProperty(Number.prototype, "toFixed", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
