@@ -10,9 +10,9 @@ es5id: 15.7.4.2_A4_T03
 description: transferring to the Date objects
 ---*/
 
+var s1 = new Date(0);
+Object.defineProperty(s1, "toString", {value: Number.prototype.toString});
 try {
-  var s1 = new Date();
-  s1.toString = Number.prototype.toString;
   var v1 = s1.toString();
   throw new Test262Error('#1: Number.prototype.toString on not a Number object should throw TypeError');
 }
@@ -20,9 +20,9 @@ catch (e) {
   assert(e instanceof TypeError, 'The result of evaluating (e instanceof TypeError) is expected to be true');
 }
 
+var s2 = new Date(0);
+s2.myToString = Number.prototype.toString;
 try {
-  var s2 = new Date();
-  s2.myToString = Number.prototype.toString;
   var v2 = s2.myToString();
   throw new Test262Error('#2: Number.prototype.toString on not a Number object should throw TypeError');
 }
