@@ -10,30 +10,24 @@ es5id: 15.7.4.2_A4_T04
 description: transferring to the Object objects
 ---*/
 
-//CHECK#1
 try {
   var s1 = new Object();
   s1.toString = Number.prototype.toString;
   var v1 = s1.toString();
-  $ERROR('#1: Number.prototype.toString on not a Number object should throw TypeError');
+  throw new Test262Error('#1: Number.prototype.toString on not a Number object should throw TypeError');
 }
 catch (e) {
-  if (!(e instanceof TypeError)) {
-    $ERROR('#1: Number.prototype.toString on not a Number object should throw TypeError, not ' + e);
-  }
+  assert(e instanceof TypeError, 'The result of evaluating (e instanceof TypeError) is expected to be true');
 }
 
-//CHECK#2
 try {
   var s2 = new Object();
   s2.myToString = Number.prototype.toString;
   var v2 = s2.myToString();
-  $ERROR('#2: Number.prototype.toString on not a Number object should throw TypeError');
+  throw new Test262Error('#2: Number.prototype.toString on not a Number object should throw TypeError');
 }
 catch (e) {
-  if (!(e instanceof TypeError)) {
-    $ERROR('#2: Number.prototype.toString on not a Number object should throw TypeError, not ' + e);
-  }
+  assert(e instanceof TypeError, 'The result of evaluating (e instanceof TypeError) is expected to be true');
 }
 
 reportCompare(0, 0);
