@@ -8,24 +8,19 @@ info: |
 es5id: 15.2.1.1_A2_T11
 description: Calling Object function with function declaration as argument value
 ---*/
-
-//CHECK#1
-if (typeof func !== 'undefined') {
-  $ERROR('#1:  function expression can\'t be declarated');
-}
+assert.sameValue(typeof func, 'undefined', 'The value of `typeof func` is expected to be "undefined"');
 
 var n_obj = Object(function func() {
   return 1;
 });
 
-//CHECK#2
-if ((n_obj.constructor !== Function) || (n_obj() !== 1)) {
-  $ERROR('#2: Object(function func(){return 1;}) returns function');
-}
+assert.sameValue(
+  n_obj.constructor,
+  Function,
+  'The value of n_obj.constructor is expected to equal the value of Function'
+);
+assert.sameValue(n_obj(), 1, 'n_obj() must return 1')
+assert.sameValue(typeof func, 'undefined', 'The value of `typeof func` is expected to be "undefined"');
 
-//CHECK#3
-if (typeof func !== 'undefined') {
-  $ERROR('#3:  function expression can\'t be declarated');
-}
 
 reportCompare(0, 0);
