@@ -15,15 +15,7 @@ var func = function() {
 
 var n_obj = new Object(func);
 
-//CHECK#1
-if (n_obj !== func) {
-  $ERROR('#1: When the Object constructor is called and if the value is an Object simply value returns');
-}
-
-//CHECK#2
-if (n_obj() !== 1) {
-  $ERROR('When the Object constructor is called and if the value is an Object simply value returns');
-}
-//
+assert.sameValue(n_obj, func, 'The value of n_obj is expected to equal the value of func');
+assert.sameValue(n_obj(), 1, 'n_obj() must return 1');
 
 reportCompare(0, 0);
