@@ -22,10 +22,11 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Object.getOwnPropertySymbols.length, 1);
-
-verifyNotEnumerable(Object.getOwnPropertySymbols, "length");
-verifyNotWritable(Object.getOwnPropertySymbols, "length");
-verifyConfigurable(Object.getOwnPropertySymbols, "length");
+verifyProperty(Object.getOwnPropertySymbols, "length", {
+  value: 1,
+  writable: false,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
