@@ -2,12 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-3
+es5id: 15.2.3.2-2-25
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Object)
+    (Date object)
 ---*/
 
-assert.sameValue(Object.getPrototypeOf(Object), Function.prototype, 'Object.getPrototypeOf(Object)');
+var obj = new Date(0);
+
+assert.sameValue(Object.getPrototypeOf(obj), Date.prototype, 'Object.getPrototypeOf(obj)');
 
 reportCompare(0, 0);
