@@ -2,12 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.12-1-1
+es5id: 15.2.3.12-3-18
 description: >
-    Object.isFrozen - TypeError is not thrown when the first param 'O'
-    is undefined
+    Object.isFrozen returns false for all built-in objects
+    (RegExp.prototype)
 ---*/
 
-Object.isFrozen(undefined);
+var b = Object.isFrozen(RegExp.prototype);
+
+assert.sameValue(b, false, 'b');
 
 reportCompare(0, 0);
