@@ -13,17 +13,17 @@ verifyNotWritable(Object, "prototype", null, function() {
   return "shifted";
 });
 
-//CHECK#1
-if (Object.prototype !== obj) {
-  $ERROR('#1: the Object.prototype property has the attributes ReadOnly.');
-}
+assert.sameValue(Object.prototype, obj, 'The value of Object.prototype is expected to equal the value of obj');
 
-//CHECK#2
 try {
   Object.prototype();
-  $ERROR('#2: the Object.prototype property has the attributes ReadOnly');
+  throw new Test262Error('#2: the Object.prototype property has the attributes ReadOnly');
 } catch (e) {
-  if (e instanceof Test262Error) throw e;
+  if (e instanceof Test262Error) {
+    throw e;
+  }
 }
 
+// TODO: Convert to verifyProperty() format.
+
 reportCompare(0, 0);
