@@ -16,8 +16,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyWritable(Object.prototype, "toString");
-verifyNotEnumerable(Object.prototype, "toString");
-verifyConfigurable(Object.prototype, "toString");
+verifyProperty(Object.prototype, "toString", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
