@@ -30,7 +30,9 @@ Object.defineProperty(obj, "foo", {
 assert(Object.isExtensible(obj));
 Object.seal(obj);
 
-verifyNotConfigurable(obj, "foo");
-assert.sameValue(obj.foo, 10);
+verifyProperty(obj, "foo", {
+  value: 10,
+  configurable: false,
+});
 
 reportCompare(0, 0);
