@@ -8,8 +8,10 @@ author: Jordan Harband
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(Object, 'values');
-verifyWritable(Object, 'values');
-verifyConfigurable(Object, 'values');
+verifyProperty(Object, "values", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
