@@ -5,7 +5,6 @@
 description: >
     Explicit iterator closing in response to error
 esid: sec-promise.all
-es6id: 25.4.4.1
 info: |
     11. Let result be PerformPromiseAll(iteratorRecord, C, promiseCapability).
     12. If result is an abrupt completion,
@@ -42,7 +41,7 @@ iterDoneSpy[Symbol.iterator] = function() {
 };
 
 Promise.resolve = function() {
-  throw new Error();
+  throw new Test262Error();
 };
 
 Promise.all(iterDoneSpy);
