@@ -16,8 +16,10 @@ features: [Symbol.toStringTag]
 
 assert.sameValue(Promise.prototype[Symbol.toStringTag], 'Promise');
 
-verifyNotEnumerable(Promise.prototype, Symbol.toStringTag);
-verifyNotWritable(Promise.prototype, Symbol.toStringTag);
-verifyConfigurable(Promise.prototype, Symbol.toStringTag);
+verifyProperty(Promise.prototype, Symbol.toStringTag, {
+  writable: false,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
