@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Promise.race), false, 'isConstructor(Promise.race
 
 assert.throws(TypeError, () => {
   new Promise.race([]);
-}, '`new Promise.race([])` throws TypeError');
+});
 
 
 reportCompare(0, 0);
