@@ -14,8 +14,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(Promise, 'reject');
-verifyWritable(Promise, 'reject');
-verifyConfigurable(Promise, 'reject');
+verifyProperty(Promise, 'reject', {
+  writable: true,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
