@@ -1,7 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
-es6id: 9.5.14
+esid: sec-proxy-object-internal-methods-and-internal-slots-construct-argumentslist-newtarget
 description: >
     Return the result from the trap method.
 info: |
@@ -11,17 +11,15 @@ info: |
 features: [Proxy]
 ---*/
 
-function Target(a, b) {
-  this.sum = a + b;
-};
-var handler = {
+var P = new Proxy(function() {
+  throw new Test262Error('target should not be called');
+}, {
   construct: function(t, c, args) {
     return {
       sum: 42
     };
   }
-};
-var P = new Proxy(Target, handler);
+});
 
 assert.sameValue((new P(1, 2)).sum, 42);
 
