@@ -2,7 +2,6 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es6id: 9.5.14
 esid: sec-proxy-object-internal-methods-and-internal-slots-construct-argumentslist-newtarget
 description: >
     If the construct trap value is undefined, propagate the construct to the target object.
@@ -21,26 +20,33 @@ info: |
     ...
     3. If func is either undefined or null, return undefined.
     ...
-features: [new.target, Proxy, Reflect, Reflect.construct]
+features: [Proxy, Reflect, Reflect.construct]
 ---*/
 
 var calls = 0;
+var _NewTarget;
 
-function NewTarget() {}
-
-function Target(a, b) {
-  assert.sameValue(new.target, NewTarget);
-  calls += 1;
-  return {
-    sum: a + b
-  };
-}
+var Target = new Proxy(function() {
+  throw new Test262Error('target should not be called');
+}, {
+  construct: function(_Target, args, NewTarget) {
+    calls += 1;
+    _NewTarget = NewTarget;
+    return {
+      sum: args[0] + args[1]
+    };
+  },
+});
 
 var P = new Proxy(Target, {
   construct: undefined
 });
+
+var NewTarget = function() {};
 var obj = Reflect.construct(P, [3, 4], NewTarget);
-assert.sameValue(obj.sum, 7, "`construct` trap is `undefined`");
-assert.sameValue(calls, 1, "target is called once");
+
+assert.sameValue(calls, 1, "construct is undefined: [[Construct]] is invoked once");
+assert.sameValue(_NewTarget, NewTarget, "construct is undefined: NewTarget is passed to [[Construct]]");
+assert.sameValue(obj.sum, 7, "construct is undefined: result of [[Construct]] is returned");
 
 reportCompare(0, 0);
