@@ -13,8 +13,10 @@ includes: [propertyHelper.js]
 features: [Reflect]
 ---*/
 
-verifyNotEnumerable(Reflect, 'defineProperty');
-verifyWritable(Reflect, 'defineProperty');
-verifyConfigurable(Reflect, 'defineProperty');
+verifyProperty(Reflect, 'defineProperty', {
+  writable: true,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
