@@ -13,13 +13,11 @@ includes: [propertyHelper.js]
 features: [Reflect]
 ---*/
 
-assert.sameValue(
-  Reflect.has.name, 'has',
-  'The value of `Reflect.has.name` is `"has"`'
-);
-
-verifyNotEnumerable(Reflect.has, 'name');
-verifyNotWritable(Reflect.has, 'name');
-verifyConfigurable(Reflect.has, 'name');
+verifyProperty(Reflect.has, "name", {
+  value: "has",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
