@@ -3,14 +3,14 @@
 
 /*---
 info: |
-    The | regular expression operator separates two alternatives.
-    The pattern first tries to match the left Alternative (followed by the sequel of the regular expression).
-    If it fails, it tries to match the right Disjunction (followed by the sequel of the regular expression)
-es5id: 15.10.2.3_A1_T12
-description: Execute /xyz|.../.exec("abc") and check results
+    The production CharacterClass :: [ [lookahead \notin {^}] ClassRanges ]
+    evaluates by evaluating ClassRanges to obtain a CharSet and returning
+    that CharSet and the boolean false
+es5id: 15.10.2.13_A1_T12
+description: Execute /a[b]c/.exec("abc") and check results
 ---*/
 
-var __executed = /xyz|.../.exec("abc");
+var __executed = /a[b]c/.exec("abc");
 
 var __expected = ["abc"];
 __expected.index = 0;
