@@ -3,20 +3,18 @@
 
 /*---
 info: |
-    The form (?! Disjunction ) specifies a zero-width negative lookahead.
-    In order for it to succeed, the pattern inside Disjunction must fail to match at the current position.
-    The current position is not advanced before matching the sequel
-es5id: 15.10.2.8_A2_T5
-description: >
-    Execute /Java(?!Script)([A-Z]\w*)/.exec("JavaScr oops ipt ") and
-    check results
+    The production CharacterClass :: [ ^ ClassRanges ] evaluates by
+    evaluating ClassRanges to  obtain a CharSet and returning that CharSet
+    and the boolean true
+es5id: 15.10.2.13_A2_T5
+description: Execute /a[^1-9]c/.exec("abc") and check results
 ---*/
 
-var __executed = /Java(?!Script)([A-Z]\w*)/.exec("JavaScr oops ipt ");
+var __executed = /a[^1-9]c/.exec("abc");
 
-var __expected = ["JavaScr", "Scr"];
+var __expected = ["abc"];
 __expected.index = 0;
-__expected.input = "JavaScr oops ipt ";
+__expected.input = "abc";
 
 assert.sameValue(
   __executed.length,
