@@ -16,13 +16,16 @@ description: >
     the correct exception
 ---*/
 
-//CHECK#1
 try {
-  $ERROR('#1.1: /[b-G\\S]/.exec("a") throw SyntaxError. Actual: ' + (new RegExp("[b-G\\S]").exec("a")));
+  throw new Test262Error('#1.1: /[b-G\\S]/.exec("a") throw SyntaxError. Actual: ' + (new RegExp("[b-G\\S]").exec("a")));
 } catch (e) {
-  if((e instanceof SyntaxError) !== true){
-    $ERROR('#1.2: /[b-G\\S]/.exec("a") throw SyntaxError. Actual: ' + (e));
-  }
+  assert.sameValue(
+    e instanceof SyntaxError,
+    true,
+    'The result of evaluating (e instanceof SyntaxError) is expected to be true'
+  );
 }
 
+// TODO: Convert to assert.throws() format.
+
 reportCompare(0, 0);
