@@ -3,18 +3,18 @@
 
 /*---
 info: |
-    The production CharacterClass :: [ [lookahead \notin {^}] ClassRanges ]
-    evaluates by evaluating ClassRanges to obtain a CharSet and returning
-    that CharSet and the boolean false
-es5id: 15.10.2.13_A1_T14
-description: Execute /[*&$]{3}/.exec("123*&$abc") and check results
+    The | regular expression operator separates two alternatives.
+    The pattern first tries to match the left Alternative (followed by the sequel of the regular expression).
+    If it fails, it tries to match the right Disjunction (followed by the sequel of the regular expression)
+es5id: 15.10.2.3_A1_T14
+description: "Execute /.+: gr(a|e)y/.exec(\"color: grey\") and check results"
 ---*/
 
-var __executed = /[*&$]{3}/.exec("123*&$abc");
+var __executed = /.+: gr(a|e)y/.exec("color: grey");
 
-var __expected = ["*&$"];
-__expected.index = 3;
-__expected.input = "123*&$abc";
+var __expected = ["color: grey","e"];
+__expected.index = 0;
+__expected.input = "color: grey";
 
 assert.sameValue(
   __executed.length,
