@@ -3,19 +3,19 @@
 
 /*---
 info: |
-    Parentheses of the form ( Disjunction ) serve both to group the components of the Disjunction pattern together and to save the result of the match.
-    The result can be used either in a backreference (\ followed by a nonzero decimal number),
-    referenced in a replace string,
-    or returned as part of an array from the regular expression matching function
-es5id: 15.10.2.8_A3_T12
-description: Execute /(a(b(c)))(d(e(f)))/.exec("xabcdefg") and check results
+    The production Assertion :: \b evaluates by returning an internal
+    AssertionTester closure that takes a State argument x and performs the ...
+es5id: 15.10.2.6_A3_T12
+description: >
+    Execute /\bop/.exec("pilot\nsoviet robot\topenoffice") and check
+    results
 ---*/
 
-var __executed = /(a(b(c)))(d(e(f)))/.exec("xabcdefg");
+var __executed = /\bop/.exec("pilot\nsoviet robot\topenoffice");
 
-var __expected = ["abcdef","abc","bc","c","def","ef","f"];
-__expected.index = 1;
-__expected.input = "xabcdefg";
+var __expected = ["op"];
+__expected.index = 19;
+__expected.input = "pilot\nsoviet robot\topenoffice";
 
 assert.sameValue(
   __executed.length,
