@@ -3,19 +3,19 @@
 
 /*---
 info: |
-    The production Atom :: . evaluates as follows:
-    i) Let A be the set of all characters except the four line terminator characters <LF>, <CR>, <LS>, or <PS>
-    ii) Call CharacterSetMatcher(A, false) and return its Matcher result
-es5id: 15.10.2.8_A4_T2
-description: Execute /.+/.exec("line 1\nline 2") and check results
+    The production Assertion :: \B evaluates by returning an internal
+    AssertionTester closure that takes a State argument x and performs the ...
+es5id: 15.10.2.6_A4_T2
+description: >
+    Execute /[f-z]e\B/.exec("devils arise\tfor\nevil") and check
+    results
 ---*/
 
-var __string = "line 1\nline 2";
-var __executed = /.+/.exec(__string);
+var __executed = /[f-z]e\B/.exec("devils arise\tfor\nrevil");
 
-var __expected = ["line 1"];
-__expected.index = 0;
-__expected.input = __string;
+var __expected = ["re"];
+__expected.index = 17;
+__expected.input = "devils arise\tfor\nrevil";
 
 assert.sameValue(
   __executed.length,
