@@ -3,18 +3,17 @@
 
 /*---
 info: |
-    The production CharacterClass :: [ [lookahead \notin {^}] ClassRanges ]
-    evaluates by evaluating ClassRanges to obtain a CharSet and returning
-    that CharSet and the boolean false
-es5id: 15.10.2.13_A1_T6
-description: Execute /ab[ercst]de/.exec("abcde") and check results
+    The production QuantifierPrefix :: { DecimalDigits , DecimalDigits }
+    evaluates as ...
+es5id: 15.10.2.7_A1_T6
+description: Execute /\d{2,4}/.exec("0a0\u0031\u0031b") and check results
 ---*/
 
-var __executed = /ab[ercst]de/.exec("abcde");
+var __executed = /\d{2,4}/.exec("0a0\u0031\u0031b");
 
-var __expected = ["abcde"];
-__expected.index = 0;
-__expected.input = "abcde";
+var __expected = ["011"];
+__expected.index = 2;
+__expected.input = "0a011b";
 
 assert.sameValue(
   __executed.length,
