@@ -3,19 +3,19 @@
 
 /*---
 info: |
-    The production Atom :: . evaluates as follows:
-    i) Let A be the set of all characters except the four line terminator characters <LF>, <CR>, <LS>, or <PS>
-    ii) Call CharacterSetMatcher(A, false) and return its Matcher result
-es5id: 15.10.2.8_A4_T9
-description: "Execute /.+/.exec(\"|\\\\[{]};:\\\"\\',<>.?/\") and check results"
+    The production QuantifierPrefix :: * evaluates by returning the two
+    results 0 and \infty
+es5id: 15.10.2.7_A4_T9
+description: >
+    Execute /["'][^"']*["']/.exec('alice cries out:\"\"') and check
+    results
 ---*/
 
-var __string = "|\\[{]};:\"\',<>.?/";
-var __executed = /.+/.exec(__string);
+var __executed = /["'][^"']*["']/.exec('alice cries out:\"\"');
 
-var __expected = ["|\\[{]};:\"\',<>.?/"];
-__expected.index = 0;
-__expected.input = __string;
+var __expected = ['\"\"'];
+__expected.index = 16;
+__expected.input = 'alice cries out:\"\"';
 
 assert.sameValue(
   __executed.length,
