@@ -11,8 +11,10 @@ info: |
 includes: [propertyHelper.js]
 ---*/
 
-verifyNotEnumerable(this, "RegExp");
-verifyWritable(this, "RegExp");
-verifyConfigurable(this, "RegExp");
+verifyProperty(this, "RegExp", {
+  writable: true,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
