@@ -21,10 +21,11 @@ includes: [propertyHelper.js]
 features: [Symbol.matchAll]
 ---*/
 
-assert.sameValue(RegExp.prototype[Symbol.matchAll].name, '[Symbol.matchAll]');
-
-verifyNotEnumerable(RegExp.prototype[Symbol.matchAll], 'name');
-verifyNotWritable(RegExp.prototype[Symbol.matchAll], 'name');
-verifyConfigurable(RegExp.prototype[Symbol.matchAll], 'name');
+verifyProperty(RegExp.prototype[Symbol.matchAll], "name", {
+  value: "[Symbol.matchAll]",
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
