@@ -14,8 +14,10 @@ includes: [propertyHelper.js]
 features: [Symbol.split]
 ---*/
 
-verifyNotEnumerable(RegExp.prototype, Symbol.split);
-verifyWritable(RegExp.prototype, Symbol.split);
-verifyConfigurable(RegExp.prototype, Symbol.split);
+verifyProperty(RegExp.prototype, Symbol.split, {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
