@@ -29,7 +29,7 @@ assert.sameValue(
 
 assert.throws(TypeError, () => {
   let re = new RegExp(''); new re.toString();
-}, '`let re = new RegExp(\'\'); new re.toString()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
