@@ -16,8 +16,10 @@ features: [Symbol.toStringTag]
 
 assert.sameValue(Set.prototype[Symbol.toStringTag], 'Set');
 
-verifyNotEnumerable(Set.prototype, Symbol.toStringTag);
-verifyNotWritable(Set.prototype, Symbol.toStringTag);
-verifyConfigurable(Set.prototype, Symbol.toStringTag);
+verifyProperty(Set.prototype, Symbol.toStringTag, {
+  writable: false,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
