@@ -10,10 +10,11 @@ description: >
 includes: [propertyHelper.js]
 ---*/
 
-assert.sameValue(Set.prototype.forEach.length, 1, "The value of `Set.prototype.forEach.length` is `1`");
-
-verifyNotEnumerable(Set.prototype.forEach, "length");
-verifyNotWritable(Set.prototype.forEach, "length");
-verifyConfigurable(Set.prototype.forEach, "length");
+verifyProperty(Set.prototype.forEach, "length", {
+  value: 1,
+  writable: false,
+  enumerable: false,
+  configurable: true
+});
 
 reportCompare(0, 0);
