@@ -12,8 +12,10 @@ features: [SharedArrayBuffer]
 
 assert.sameValue(SharedArrayBuffer.prototype.constructor, SharedArrayBuffer);
 
-verifyNotEnumerable(SharedArrayBuffer.prototype, "constructor");
-verifyWritable(SharedArrayBuffer.prototype, "constructor");
-verifyConfigurable(SharedArrayBuffer.prototype, "constructor");
+verifyProperty(SharedArrayBuffer.prototype, "constructor", {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
