@@ -1,4 +1,4 @@
-// |reftest| skip -- resizable-arraybuffer is not supported
+// |reftest| skip-if(!this.hasOwnProperty('SharedArrayBuffer')) -- SharedArrayBuffer is not enabled unconditionally
 // Copyright (C) 2021 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
@@ -23,7 +23,7 @@ includes: [propertyHelper.js]
 verifyProperty(SharedArrayBuffer.prototype.grow, 'name', {
   value: 'grow',
   enumerable: false,
-  wrtiable: false,
+  writable: false,
   configurable: true
 });
 
