@@ -5,13 +5,14 @@
 info: |
     15.5.5.2 defines [[GetOwnProperty]] for Strings. It supports using indexing
     notation to look up non numeric property names.
-es5id: 15.5.5.5.2-3-2
-description: String value indexing returns undefined for missing data properties
+es5id: 15.5.5.5.2-7-2
+description: >
+    String value indexing returns undefined if the numeric index is
+    less than 0
 ---*/
 
 var s = String("hello world");
 
-
-assert.sameValue(s["foo"], undefined, 's["foo"]');
+assert.sameValue(s[-1], undefined, 's[-1]');
 
 reportCompare(0, 0);
