@@ -15,7 +15,7 @@ var __str = new String(.12345);
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if (typeof __str !== "object") {
-  $ERROR('#1: __str =new String(.12345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
+  throw new Test262Error('#1: __str =new String(.12345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -23,7 +23,7 @@ if (typeof __str !== "object") {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1.5
 if (__str.constructor !== String) {
-  $ERROR('#1.5: __str =new String(.12345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
+  throw new Test262Error('#1.5: __str =new String(.12345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -31,7 +31,7 @@ if (__str.constructor !== String) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2
 if (__str != "0.12345") {
-  $ERROR('#2: __str =new String(.12345); __str =="0.12345". Actual: __str ==' + __str);
+  throw new Test262Error('#2: __str =new String(.12345); __str =="0.12345". Actual: __str ==' + __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -40,7 +40,7 @@ __str = new String(.012345);
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#3
 if (typeof __str !== "object") {
-  $ERROR('#3: __str =new  String(.012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
+  throw new Test262Error('#3: __str =new  String(.012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -48,7 +48,7 @@ if (typeof __str !== "object") {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2.5
 if (__str.constructor !== String) {
-  $ERROR('#3.5: __str =new  String(.012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
+  throw new Test262Error('#3.5: __str =new  String(.012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -56,7 +56,7 @@ if (__str.constructor !== String) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#4
 if (__str != "0.012345") {
-  $ERROR('#4: __str =new  String(.012345); __str =="0.012345". Actual: __str ==' + __str);
+  throw new Test262Error('#4: __str =new  String(.012345); __str =="0.012345". Actual: __str ==' + __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -65,7 +65,7 @@ __str = new String(.0012345);
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#5
 if (typeof __str !== "object") {
-  $ERROR('#5: __str =new  String(.0012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
+  throw new Test262Error('#5: __str =new  String(.0012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -73,7 +73,7 @@ if (typeof __str !== "object") {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#5.5
 if (__str.constructor !== String) {
-  $ERROR('#5.5: __str =new  String(.0012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
+  throw new Test262Error('#5.5: __str =new  String(.0012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -81,7 +81,7 @@ if (__str.constructor !== String) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#6
 if (__str != "0.0012345") {
-  $ERROR('#6: __str =new  String(.0012345); __str =="0.0012345". Actual: __str ==' + __str);
+  throw new Test262Error('#6: __str =new  String(.0012345); __str =="0.0012345". Actual: __str ==' + __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -90,7 +90,7 @@ __str = new String(.00000012345);
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#7
 if (typeof __str !== "object") {
-  $ERROR('#7: __str =new  String(.00000012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
+  throw new Test262Error('#7: __str =new  String(.00000012345); typeof __str === "object". Actual: typeof __str ===' + typeof __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -98,7 +98,7 @@ if (typeof __str !== "object") {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#7.5
 if (__str.constructor !== String) {
-  $ERROR('#7.5: __str =new  String(.00000012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
+  throw new Test262Error('#7.5: __str =new  String(.00000012345); __str.constructor === String. Actual: __str.constructor ===' + __str.constructor);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -106,7 +106,7 @@ if (__str.constructor !== String) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#8
 if (__str != "1.2345e-7") {
-  $ERROR('#8: __str =new  String(.00000012345); __str =="1.2345e-7". Actual: __str ==' + __str);
+  throw new Test262Error('#8: __str =new  String(.00000012345); __str =="1.2345e-7". Actual: __str ==' + __str);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
