@@ -2,7 +2,6 @@
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
 esid: sec-properties-of-string-instances-length
-es6id: 21.1.4.1
 description: The "length" property of String objects
 info: |
   [...]
@@ -13,9 +12,11 @@ includes: [propertyHelper.js]
 
 var str = new String('');
 
-verifyNotEnumerable(str, 'length');
-verifyNotWritable(str, 'length');
-verifyNotConfigurable(str, 'length');
+verifyProperty(str, 'length', {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 assert.sameValue(str.length, 0, 'empty string');
 
