@@ -17,7 +17,7 @@ var __str = "lego";
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if (__str.charAt(__obj) !== "e") {
-  $ERROR('#1: var __obj = {toString:function(){return 1;}}; var __str = "lego"; __str.charAt(__obj) === "e". Actual: ' + __str.charAt(__obj));
+  throw new Test262Error('#1: var __obj = {toString:function(){return 1;}}; var __str = "lego"; __str.charAt(__obj) === "e". Actual: ' + __str.charAt(__obj));
 }
 //
 //////////////////////////////////////////////////////////////////////////////
