@@ -26,7 +26,7 @@ var __obj2 = {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if ("ABB\u0041BABAB".indexOf(__obj, __obj2) !== 3) {
-  $ERROR('#1: var __obj = {toString:function(){return "\u0041B";}}; var __obj2 = {valueOf:function(){return {};},toString:function(){return "1";}}; "ABB\\u0041BABAB".indexOf(__obj, __obj2)===3. Actual: ' + ("ABB\u0041BABAB".indexOf(__obj, __obj2)));
+  throw new Test262Error('#1: var __obj = {toString:function(){return "\u0041B";}}; var __obj2 = {valueOf:function(){return {};},toString:function(){return "1";}}; "ABB\\u0041BABAB".indexOf(__obj, __obj2)===3. Actual: ' + ("ABB\u0041BABAB".indexOf(__obj, __obj2)));
 }
 //
 //////////////////////////////////////////////////////////////////////////////
