@@ -34,10 +34,10 @@ var __instance = new __FACTORY(void 0);
 //CHECK#1
 try {
   var x = __instance.lastIndexOf(__obj, __obj2);
-  $ERROR('#1: var x = __instance.lastIndexOf(__obj, __obj2) lead to throwing exception');
+  throw new Test262Error('#1: var x = __instance.lastIndexOf(__obj, __obj2) lead to throwing exception');
 } catch (e) {
   if (e !== "intostr") {
-    $ERROR('#1.1: Exception === "intostr". Actual: ' + e);
+    throw new Test262Error('#1.1: Exception === "intostr". Actual: ' + e);
   }
 }
 //
