@@ -13,7 +13,7 @@ description: >
 var str1 = "h";
 var str2 = "H";
 if (str1.localeCompare(str2) !== -str2.localeCompare(str1)) {
-  $ERROR('#1.1: var str1 = "h"; var str2 = "H"; str1.localeCompare(str2)===-str2.localeCompare(str1). Actual: ' + str1.localeCompare(str2));
+  throw new Test262Error('#1.1: var str1 = "h"; var str2 = "H"; str1.localeCompare(str2)===-str2.localeCompare(str1). Actual: ' + str1.localeCompare(str2));
 }
 
 reportCompare(0, 0);
