@@ -19,7 +19,7 @@ var __string = "Boston, MA 02134";
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if (__string.match(__re).length !== __matches.length) {
-  $ERROR('#1: __string.match(__re).length=== __matches.length. Actual: ' + __string.match(__re).length);
+  throw new Test262Error('#1: __string.match(__re).length=== __matches.length. Actual: ' + __string.match(__re).length);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -27,7 +27,7 @@ if (__string.match(__re).length !== __matches.length) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#3
 if (__string.match(__re)[0] !== __matches[0]) {
-  $ERROR('#3: __string.match(__re)[0]===__matches[0]. Actual: ' + __string.match(__re)[0]);
+  throw new Test262Error('#3: __string.match(__re)[0]===__matches[0]. Actual: ' + __string.match(__re)[0]);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
