@@ -26,10 +26,10 @@ var __str = "ABB\u0041BABAB";
 //CHECK#1
 try {
   var x = __str.replace(__obj, __obj2);
-  $ERROR('#1: "var x = __str.replace(__obj,__obj2)" lead to throwing exception');
+  throw new Test262Error('#1: "var x = __str.replace(__obj,__obj2)" lead to throwing exception');
 } catch (e) {
   if (e !== "insearchValue") {
-    $ERROR('#1.1: Exception === "insearchValue". Actual: ' + e);
+    throw new Test262Error('#1.1: Exception === "insearchValue". Actual: ' + e);
   }
 }
 //
