@@ -10,7 +10,7 @@ description: Checking String.prototype.toUpperCase.length
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if (!(String.prototype.toUpperCase.hasOwnProperty("length"))) {
-  $ERROR('#1: String.prototype.toUpperCase.hasOwnProperty("length") return true. Actual: ' + String.prototype.toUpperCase.hasOwnProperty("length"));
+  throw new Test262Error('#1: String.prototype.toUpperCase.hasOwnProperty("length") return true. Actual: ' + String.prototype.toUpperCase.hasOwnProperty("length"));
 }
 //
 //////////////////////////////////////////////////////////////////////////////
@@ -18,7 +18,7 @@ if (!(String.prototype.toUpperCase.hasOwnProperty("length"))) {
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2
 if (String.prototype.toUpperCase.length !== 0) {
-  $ERROR('#2: String.prototype.toUpperCase.length === 0. Actual: ' + String.prototype.toUpperCase.length);
+  throw new Test262Error('#2: String.prototype.toUpperCase.length === 0. Actual: ' + String.prototype.toUpperCase.length);
 }
 //
 //////////////////////////////////////////////////////////////////////////////
