@@ -13,7 +13,7 @@ description: Call toUpperCase() function of function call
 if (function() {
     return "GnulLuNa"
   }().toUpperCase() !== "GNULLUNA") {
-  $ERROR('#1: function(){return "GnulLuNa"}().toUpperCase() === "GNULLUNA". Actual: ' + function() {
+  throw new Test262Error('#1: function(){return "GnulLuNa"}().toUpperCase() === "GNULLUNA". Actual: ' + function() {
     return "GnulLuNa"
   }().toUpperCase());
 }
