@@ -25,7 +25,7 @@ assert.sameValue(isConstructor(Symbol.for), false, 'isConstructor(Symbol.for) mu
 
 assert.throws(TypeError, () => {
   new Symbol.for();
-}, '`new Symbol.for()` throws TypeError');
+});
 
 
 reportCompare(0, 0);
