@@ -11,8 +11,10 @@ features: [Symbol.hasInstance]
 ---*/
 
 assert.sameValue(typeof Symbol.hasInstance, 'symbol');
-verifyNotEnumerable(Symbol, 'hasInstance');
-verifyNotWritable(Symbol, 'hasInstance');
-verifyNotConfigurable(Symbol, 'hasInstance');
+verifyProperty(Symbol, 'hasInstance', {
+  writable: false,
+  enumerable: false,
+  configurable: false,
+});
 
 reportCompare(0, 0);
