@@ -13,8 +13,10 @@ features: [Symbol]
 
 assert.sameValue(typeof Symbol.prototype.valueOf, 'function');
 
-verifyNotEnumerable(Symbol.prototype, 'valueOf');
-verifyWritable(Symbol.prototype, 'valueOf');
-verifyConfigurable(Symbol.prototype, 'valueOf');
+verifyProperty(Symbol.prototype, 'valueOf', {
+  writable: true,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
