@@ -1,4 +1,4 @@
-// |reftest| skip-if(!this.hasOwnProperty('Temporal')) -- Temporal is not enabled unconditionally
+// |reftest| shell-option(--enable-temporal) skip-if(!this.hasOwnProperty('Temporal')||!xulRuntime.shell) -- Temporal is not enabled unconditionally, requires shell-options
 // Copyright (C) 2022 Igalia, S.L. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -8,7 +8,6 @@ description: relativeTo property bag with offset property is rejected if offset
 features: [Temporal]
 ---*/
 
-const timeZone = new Temporal.TimeZone("UTC");
 const d1 = new Temporal.Duration(0, 1, 0, 280);
 const d2 = new Temporal.Duration(0, 1, 0, 281);
 
@@ -22,7 +21,7 @@ const badOffsets = [
   1000n,      // must be a string
 ];
 badOffsets.forEach((offset) => {
-  const relativeTo = { year: 2021, month: 10, day: 28, offset, timeZone };
+  const relativeTo = { year: 2021, month: 10, day: 28, offset, timeZone: "UTC" };
   assert.throws(
     typeof(offset) === 'string' ? RangeError : TypeError,
     () => Temporal.Duration.compare(d1, d2, { relativeTo }),
