@@ -1,4 +1,4 @@
-// |reftest| skip-if(!this.hasOwnProperty('Temporal')) -- Temporal is not enabled unconditionally
+// |reftest| shell-option(--enable-temporal) skip-if(!this.hasOwnProperty('Temporal')||!xulRuntime.shell) -- Temporal is not enabled unconditionally, requires shell-options
 // Copyright (C) 2022 André Bargull. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -9,9 +9,6 @@ description: >
 features: [Temporal]
 ---*/
 
-const plainDate = new Temporal.PlainDate(1970, 1, 1);
-const zonedDateTime = new Temporal.ZonedDateTime(0n, "UTC", "iso8601");
-
 // Largest temporal unit is "second".
 const duration = Temporal.Duration.from({seconds: Number.MAX_SAFE_INTEGER});
 
@@ -19,12 +16,4 @@ assert.throws(RangeError, () => {
   duration.add(duration);
 });
 
-assert.throws(RangeError, () => {
-  duration.add(duration, {relativeTo: plainDate});
-});
-
-assert.throws(RangeError, () => {
-  duration.add(duration, {relativeTo: zonedDateTime});
-});
-
 reportCompare(0, 0);
