@@ -1,4 +1,4 @@
-// |reftest| skip -- Temporal is not supported
+// |reftest| shell-option(--enable-temporal) skip-if(!this.hasOwnProperty('Temporal')||!xulRuntime.shell) -- Temporal is not enabled unconditionally, requires shell-options
 // Copyright (C) 2021 Igalia, S.L. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -8,7 +8,7 @@ description: Instant strings with UTC offset fractional part are not confused wi
 features: [Temporal]
 ---*/
 
-const str = "1970-01-01T00:02:00.000000000+00:02[+00:01:30.987654321]";
+const str = "1970-01-01T00:02:00.000000000+00:02[+01:30]";
 
 const result = Temporal.Instant.from(str);
 assert.sameValue(result.epochNanoseconds, 0n, "UTC offset determined from offset part of string");
