@@ -1,4 +1,4 @@
-// |reftest| skip-if(!this.hasOwnProperty('Temporal')) -- Temporal is not enabled unconditionally
+// |reftest| shell-option(--enable-temporal) skip-if(!this.hasOwnProperty('Temporal')||!xulRuntime.shell) -- Temporal is not enabled unconditionally, requires shell-options
 // Copyright (C) 2022 Igalia S.L. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -36,6 +36,8 @@ const invalidStrings = [
   "2020-01-01T001Z",
   "2020-01-01T01:001Z",
   "2020-01-01T01:01:001Z",
+  "2020-01-01T00:00-24:00",
+  "2020-01-01T00:00+24:00",
   // valid, but forms not supported in Temporal:
   "2020-W01-1T00:00Z",
   "2020-001T00:00Z",
